/*********************************************************************************

Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
* @file      adi_fir_acm_v3.h
*
* @brief     FIR driver header file in ACM mode for ADSP-SC83x processors.
*
* @details
*            This is the primary header file for the FIR Accelerator in ACM mode for ADSP-SC83x processors.
*/

/** @addtogroup FIR_Driver
 *  @{
 */

#ifndef __ADI_FIR_ACM_V3_H__
#define __ADI_FIR_ACM_V3_H__

/*==========  I N C L U D E  ==========*/
#include <adi_types.h>
#include <services/int/adi_int.h>

#include <drivers/fir/adi_fir_config_SC8xx.h>


/*! @cond PRIVATE */
/*==========  D E F I N E S  ==========*/
#define ADI_FIR_TASK_INFO_SIZE           (24u)
#define ADI_FIR_TCB_INFO_SIZE            (64u) 

#define FIR_MEM_SIZE(NO_OF_CHANNELS) (ADI_FIR_TASK_INFO_SIZE + ((NO_OF_CHANNELS) * ADI_FIR_TCB_INFO_SIZE))

/*==========  T Y P E    D E F S  ==========*/

/* FIR Device Handle */
typedef void* ADI_FIR_DEV_HANDLE;

/* FIR Task Handle */
typedef void* ADI_FIR_TASK_HANDLE;

/*==========  E N U M S  ==========*/

/* FIR TASK STATE*/
typedef enum
{
    ADI_FIR_TASK_STATE_NONE,            /* FIR Task Created */
    ADI_FIR_TASK_STATE_QUEUED,          /* FIR Task is pending in Task Queue */
    ADI_FIR_TASK_STATE_RUNNING,         /* FIR Task is running on HW Accelerator */
	ADI_FIR_TASK_STATE_COMPLETED        /* FIR Task is completed */
} ADI_FIR_TASK_STATE;

/* Return enumerations */
typedef enum
{
    ADI_FIR_RESULT_SUCCESS,                /* API call was successful */
    ADI_FIR_RESULT_FAILED,                 /* API call failed */
    ADI_FIR_RESULT_INVALID_HANDLE,         /* Handle passed to the API is invalid */
    ADI_FIR_RESULT_INVALID_STATE,          /* API call failed due to the current state of driver/task/channel */
    ADI_FIR_RESULT_INVALID_OPERATION,      /* Invalid Operation */
	ADI_FIR_RESULT_TASK_STATE_RUNNING      /* State of the FIR Task is Running */
}ADI_FIR_RESULT;

/* FIR DEVICE STATE*/
typedef enum
{
    ADI_FIR_DEV_STATE_NOTINIT,          /* FIR Device Not Initialized */
	ADI_FIR_DEV_STATE_INIT              /* FIR Device is Initialized */
} ADI_FIR_DEV_STATE;

/* Sampling supported by the FIR */
typedef enum
{
    ADI_FIR_SAMPLING_SINGLE_RATE,               /* No Decimation/Interpolation */
    ADI_FIR_SAMPLING_DECIMATION,                /* Decimation of input */
    ADI_FIR_SAMPLING_INTERPOLATION              /* Interpolation of input */
} ADI_FIR_SAMPLING;


/* Float Rounding Modes */
typedef enum
{
    ADI_FIR_FLOAT_ROUNDING_MODE_IEEE_ROUND_TO_NEAREST_EVEN         = 0,
    ADI_FIR_FLOAT_ROUNDING_MODE_IEEE_ROUND_TO_ZERO               = 1
} ADI_FIR_FLOAT_ROUNDING_MODE;


/* Specifies whether the fixed point input is unsigned/signed */
typedef enum
{
    ADI_FIR_FIXED_INPUT_FORMAT_UNSIGNED_INTEGER,
    ADI_FIR_FIXED_INPUT_FORMAT_SIGNED_INTEGER

} ADI_FIR_FIXED_INPUT_FORMAT;

/*! @endcond */

/*! Channel Configurations parameters */
typedef struct
{
    uint32_t                      nTapLength;         /*!< Tap Length of the FIR Filter */
    uint32_t                      nWindowSize;        /*!< Window Size */
    ADI_FIR_SAMPLING              eSampling;          /*!< Sampling Mode */
    uint32_t                      nSamplingRatio;     /*!< Sampling Ratio */

	/* New Additions */
	bool                          bCallbackEnable;    /*!< Enable/Disable Callback for the Channel. This field is not available in legacy mode*/
	bool                          bTriggerEnable;     /*!< Mask/Unmask Trigger Generation on Completion of the Channel. This field is not available in legacy mode */
	bool                          bTriggerWaitEnable; /*!< Enable/Disable Trigger Wait for the Channel. This field is not available in legacy mode */
	ADI_FIR_FLOAT_ROUNDING_MODE   eRoundingMode;      /*!< Rounding Mode of the Channel. This field is not available in legacy mode */
	bool                          bFixedPoint;        /*!< Enable/Disable Fixed Point for the Channel. This field is not available in legacy mode */
	ADI_FIR_FIXED_INPUT_FORMAT    eFixedPoint ;       /*!< Signed/Unsigned Fixed Point Format. This field is not available in legacy mode */

    uint32_t                      nCoefficientCount;  /*!< Number of elements in Coefficient Buffer */
    int32_t                       nCoefficientModify; /*!< Modify for the Coefficient Buffer */
    void*                         pCoefficientIndex;  /*!< Pointer to the start of the coefficient buffer */

    void*                         pOutputBuffBase;    /*!< Pointer to the base of the output circular buffer */
    uint32_t                      nOutputBuffCount;   /*!< Number of elements in the output circular buffer */
    int32_t                       nOutputBuffModify;  /*!< Modifier to be used for the output circular buffer */
    void*                         pOutputBuffIndex;   /*!< Pointer to the current index of the output circular buffer */

    void*                         pInputBuffBase;     /*!< Pointer to the base of the input circular buffer */
    uint32_t                      nInputBuffCount;    /*!< Number of elements in the input circular buffer */
    int32_t                       nInputBuffModify;   /*!< Modifier to be used for the input circular buffer */
    void*                         pInputBuffIndex;    /*!< Pointer to the current index of the input circular buffer */

}ADI_FIR_CHANNEL_INFO;

/*! @cond PRIVATE */
/*! Input and Output Buffer Indexes of the channel. When a task has to be reused in ACM Mode, the Input and Output Buffer Params need to be updated before Queuing the task again*/
typedef struct
{
    void*                         pOutputBuffBase;    /*!< Pointer to the base of the output circular buffer */
    uint32_t                      nOutputBuffCount;   /*!< Number of elements in the output circular buffer */
    int32_t                       nOutputBuffModify;  /*!< Modifier to be used for the output circular buffer */
    void*                         pOutputBuffIndex;   /*!< Pointer to the current index of the output circular buffer */

    void*                         pInputBuffBase;     /*!< Pointer to the base of the input circular buffer */
    uint32_t                      nInputBuffCount;    /*!< Number of elements in the input circular buffer */
    int32_t                       nInputBuffModify;   /*!< Modifier to be used for the input circular buffer */
    void*                         pInputBuffIndex;    /*!< Pointer to the current index of the input circular buffer */

}ADI_FIR_CHANNEL_BUFFER_INFO;


/* Event generated by the driver */
typedef enum
{
    ADI_FIR_EVENT_CHANNEL_DONE         /* A particular channel is done */
} ADI_FIR_EVENT;

/*! @endcond */
/* Global APIs */

/* This API will reset the FIR accelerator, and install the interrupt handler

   Device Num:  FIR device number
   hFirHandle:  Address to store the FIR device handle
*/
ADI_FIR_RESULT adi_fir_Open(uint8_t              nDeviceNum,
                            ADI_FIR_DEV_HANDLE *phFirHandle);


/* This API will do the following:
   - Create a FIR task for given FIR device,
   - Create a TCB chain from given Channel list

   hFirHandle:   FIR device handle
   pChannelList: Array of Channel information using which TCB chain will be created
   nNumChannels: Number of Channels(TCBs) in this task
   pMemory:      Memory to store the TCB chain.
   nMemSize:     Size of the memory provided (calculated on the basis of number of channels)
   phFirTask:    Address to store the FIR task handle
*/
ADI_FIR_RESULT adi_fir_CreateTask(ADI_FIR_DEV_HANDLE     hFirHandle,
                                  ADI_FIR_CHANNEL_INFO  *pChannelList,
                                  uint32_t               nNumChannels,
                                  void                  *pMemory,
                                  uint32_t               nMemSize,
                                  ADI_FIR_TASK_HANDLE   *phFirTask);

/* This API will do the following:
   - Allocate this task to accelerator if accelerator is free.
   - Push this task to queue if accelerator is busy.

   hFirTask:     FIR task handle
*/
ADI_FIR_RESULT adi_fir_QueueTask(ADI_FIR_TASK_HANDLE hFirTask);

/* This API will return the status of given task.

   hFirTask:     FIR task handle
   TaskStatus:   Address to store the task status
*/
ADI_FIR_RESULT adi_fir_GetFirTaskStatus(ADI_FIR_TASK_HANDLE hFirTask, ADI_FIR_TASK_STATE *TaskStatus);

/* This API will close this FIR accelerator instance and application will have to call adi_fir_Open() API again if it need to use the FIR accelerator

   hFirHandle:   FIR device handle
*/
ADI_FIR_RESULT adi_fir_Close(ADI_FIR_DEV_HANDLE hFirHandle);

/* This API will register a application defined callback with the driver

   hFirHandle:   FIR device handle
   pfCallback:   Pointer to the Application defined Callback function
   pfCBParam:    Pointer to application defined callback parameter
*/
ADI_FIR_RESULT adi_fir_RegisterCallback(ADI_FIR_DEV_HANDLE hFirHandle,
		                                ADI_CALLBACK       pfCallback,
										void *             pCBParam);

/* This API is used when a task is to be re-used in ACM Mode. In ACM Mode, the Buffer Indexes of the task need to be updated using this API as the hardware
 * doesn't update the buffer Indexes. In Legacy Mode, the Hardware updates the Buffer Indexes after completion of the Task. For More details, please refer to the Hardware
 * Reference Manual.

   hFirTask:     FIR task handle
   pChannelIndexList:   Array of Channel Buffer Indexes
   nNumChannels:    Number of Channels. Note that the number of channels must be the same as the value when creating a task.
*/
ADI_FIR_RESULT adi_fir_UpdateTask(ADI_FIR_TASK_HANDLE hFirTask,
		                          ADI_FIR_CHANNEL_BUFFER_INFO  *pChannelParamsList,
								  uint32_t               nNumChannels);

#if (defined(__ADSPSHARCFX__))
ADI_FIR_RESULT adi_fir_ConfigSecInt (ADI_FIR_DEV_HANDLE     hFirHandle,
                                                           bool bSecInt );
#endif


#endif /* __ADI_FIR_ACM_V3_H__ */
/*@}*/

