/*********************************************************************************

Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
* @file      adi_iir_legacy_v3.h
*
* @brief     IIR Accelerator Driver Legacy Mode header file for ADSP-SC83x processors.
*
* @details
*            This is the primary header file for the Legacy Mode Implementation of IIR Accelerator for ADSP-SC83x processors.
*/

/** @addtogroup IIR_Driver
 *  @{
 */

#ifndef __ADI_IIR_LEGACY_V3_H__
#define __ADI_IIR_LEGACY_V3_H__

/*==========  I N C L U D E  ==========*/
#include <adi_types.h>
#include <services/int/adi_int.h>
#include <drivers/iir/adi_iir_config_SC8xx.h>


/*==========  D E F I N E S  ==========*/
#define ADI_IIR_TASK_INFO_SIZE           (28u + ADI_CACHE_LINE_LENGTH)  /*!< Size of ADI_IIR_TASK_INFO. Legacy Mode-28u ACM Mode-24u. Extra bytes of cache line length are padded to make sure that Task Info and TCB Chain are on two different cache lines.   */
#if (ADI_IIR_CFG_SLEW_EN == 1)
#define ADI_IIR_TCB_INFO_SIZE            (64u)  /*!< Size of ADI_IIR_TCB_INFO. Additional 3 fields added in TCB for slewing */
#else
#define ADI_IIR_TCB_INFO_SIZE            (52u)  /*!< Size of ADI_IIR_TCB_INFO.  Legacy Mode-52u ACM Mode-64u */
#endif /* */

#define IIR_MEM_SIZE(NO_OF_CHANNELS) (ADI_IIR_TASK_INFO_SIZE + ((NO_OF_CHANNELS) * ADI_IIR_TCB_INFO_SIZE))  /*!< Macro for defining the size of memory application needs to pass while creating a IIR Task. The size of the memory to be passed depends on the Number of Channels in the Task */

/*==========  T Y P E    D E F S  ==========*/

/*! IIR Device Handle */
typedef void* ADI_IIR_DEV_HANDLE;

/*! IIR Task Handle */
typedef void* ADI_IIR_TASK_HANDLE;

/*==========  E N U M S  ==========*/

/*! IIR TASK STATE*/
typedef enum
{
#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
    ADI_IIR_TASK_STATE_NONE,            /*!< IIR Task Created */
    ADI_IIR_TASK_STATE_QUEUED,          /*!< IIR Task is pending in Task Queue */
    ADI_IIR_TASK_STATE_RUNNING,         /*!< IIR Task is running on HW Accelerator */
	ADI_IIR_TASK_STATE_COMPLETED        /*!< IIR Task is completed */
#else
    ADI_IIR_TASK_STATE_NONE,            /*!< IIR Task Created and unassigned */
    ADI_IIR_TASK_STATE_ASSIGNED,          /*!< IIR Task is assigned to accelerator */
    ADI_IIR_TASK_STATE_RUNNING         /*!< IIR Task is running on HW Accelerator */
#endif/* (ADI_IIR_CFG_NOQUEUE_MODE == 1) */
} ADI_IIR_TASK_STATE;

/*! Return enumerations */
typedef enum
{
    ADI_IIR_RESULT_SUCCESS,                /*!< API call was successful */
    ADI_IIR_RESULT_FAILED,                 /*!< API call failed */
    ADI_IIR_RESULT_INVALID_HANDLE,         /*!< Handle passed to the API is invalid */
    ADI_IIR_RESULT_INVALID_STATE,          /*!< API call failed due to the current state of driver/task/channel */
    ADI_IIR_RESULT_INVALID_OPERATION,      /*!< Invalid Operation */
	ADI_IIR_RESULT_TASK_STATE_RUNNING      /*!< State of the IIR Task is Running */
}ADI_IIR_RESULT;

/*! @cond PRIVATE */

/*! IIR DEVICE STATE*/
typedef enum
{
    ADI_IIR_DEV_STATE_NOTINIT,          /*!< IIR Device Not Initialized */
	ADI_IIR_DEV_STATE_IDLE,             /*!< IIR Device is Initialized but idle (not running any HW Accelerator task) */
    ADI_IIR_DEV_STATE_INUSE             /*!< IIR Device is in USE */
} ADI_IIR_DEV_STATE;

/*! @endcond */

/*! Float Rounding Modes */
typedef enum
{
    ADI_IIR_FLOAT_ROUNDING_MODE_IEEE_ROUND_TO_NEAREST_EVEN         = 0,    /*!< IEEE Round to nearest even */
    ADI_IIR_FLOAT_ROUNDING_MODE_IEEE_ROUND_TO_ZERO                 = 1     /*!< IEEE Round to zero */
} ADI_IIR_FLOAT_ROUNDING_MODE;

/*! @cond PRIVATE */

/*! Channel Configurations parameters */
typedef struct
{
    uint32_t           nBiquads;           /*!< Number of Biquads of the IIR Filter */
    uint32_t           nWindowSize;        /*!< Window Size */

	bool                          bTriggerEnable;             /*!< Mask/Unmask Trigger Generation on Completion of the Channel. In legacy mode, this field is only available for ADSP-SC594 */
	bool                          bTriggerWaitEnable;         /*!< Enable/Disable Trigger Wait for the Channel. In legacy mode, this field is only available for ADSP-SC594 */
#if (ADI_IIR_CFG_SLEW_EN == 1)
	bool                          bLinearAlpha;               /*!< Flag to select Constant alpha(False) or linearly changing alpha(True). In legacy mode, this field is only available for ADSP-SC83x */
    void*                         pTargetCoefficientIndex;    /*!< Pointer to the start of the TargetCoefficient buffer */
	uint32_t                      nAlpha;                     /*!< Alpha value in Hex. In legacy mode, this field is only available for ADSP-SC83x. Note that in linear alpha mode, alpha should be programmed as 1/N, where N is number of slewing iterations */
	uint32_t                      nNn;                        /*!< Slewing Iterations(N) are bits [23:8], Slew Frequency(n) is bits [7:0], bits [31:24] must be zero. In legacy mode, this field is only available for ADSP-SC83x. Note that in linear alpha mode, slewing frequency(n) should be multiple of window size.  */
    bool                          bSaturationEnable;          /*!< Flag to enable/disable coefficients assuming target coefficient values after last iteration of slewing. This flag is valid only in Constant Alpha Mode. In legacy mode, this field is only available for ADSP-SC83x */
#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */

    uint32_t           nCoefficientLength; /*!< Coefficient Buffer Length */
    int32_t            nCoefficientModify; /*!< Modify for the Coefficient Buffer */
    void*              pCoefficientIndex;  /*!< Pointer to the start of the coefficient buffer */

    void*              pOutputBuffBase;    /*!< Pointer to the base of the output circular buffer */
    uint32_t           nOutputBuffLength;  /*!< Length of the output circular buffer */
    int32_t            nOutputBuffModify;  /*!< Modifier to be used for the output circular buffer */
    void*              pOutputBuffIndex;   /*!< Pointer to the current index of the output circular buffer */

    void*              pInputBuffBase;     /*!< Pointer to the base of the input circular buffer */
    uint32_t           nInputBuffLength;   /*!< Length of the input circular buffer */
    int32_t            nInputBuffModify;   /*!< Modifier to be used for the input circular buffer */
    void*              pInputBuffIndex;    /*!< Pointer to the current index of the input circular buffer */

}ADI_IIR_CHANNEL_INFO;

/*! @endcond */

/*! Input and Output Buffer Indexes of the channel. When a task has to be reused , the Input and Output Buffer Params can be updated before Queuing the task again.
 *  Note that in ACM mode(only in ADSP-21569), the hardware doesn't update the Task Index. In legacy Mode,Task Indexes are updated by the
 *  Hardware after its complete*/
typedef struct
{
    void*                         pOutputBuffBase;    /*!< Pointer to the base of the output circular buffer */
    uint32_t                      nOutputBuffLength;   /*!< Number of elements in the output circular buffer */
    int32_t                       nOutputBuffModify;  /*!< Modifier to be used for the output circular buffer */
    void*                         pOutputBuffIndex;   /*!< Pointer to the current index of the output circular buffer */

    void*                         pInputBuffBase;     /*!< Pointer to the base of the input circular buffer */
    uint32_t                      nInputBuffLength;    /*!< Number of elements in the input circular buffer */
    int32_t                       nInputBuffModify;   /*!< Modifier to be used for the input circular buffer */
    void*                         pInputBuffIndex;    /*!< Pointer to the current index of the input circular buffer */
#if (ADI_IIR_CFG_SLEW_EN == 1)
	bool                          bLinearAlpha;               /*!< Flag to select Constant alpha(False) or linearly changing alpha(True). In legacy mode, this field is only available for ADSP-SC83x */
    void*                         pTargetCoefficientIndex;    /*!< Pointer to the start of the TargetCoefficient buffer */
	uint32_t                      nAlpha;                     /*!< Alpha value in Hex. In legacy mode, this field is only available for ADSP-SC83x. Note that in linear alpha mode, alpha should be programmed as 1/N, where N is number of slewing iterations */
	uint32_t                      nNn;                        /*!< Slewing Iterations(N) are bits [23:8], Slew Frequency(n) is bits [7:0], bits [31:24] must be zero. In legacy mode, this field is only available for ADSP-SC83x. Note that in linear alpha mode, slewing frequency(n) should be multiple of window size.  */
    bool                          bSaturationEnable;          /*!< Flag to enable/disable coefficients assuming target coefficient values after last iteration of slewing. This flag is valid only in Constant Alpha Mode. In legacy mode, this field is only available for ADSP-SC83x */
#endif/* (ADI_IIR_CFG_SLEW_EN == 1) */
}ADI_IIR_CHANNEL_BUFFER_INFO;

/*! Event generated by the driver */
typedef enum
{
    ADI_IIR_EVENT_CHANNEL_DONE,         /*!< A particular channel is done */
    ADI_IIR_EVENT_ALL_CHANNEL_DONE      /*!< All channels done. In ACM Mode, Since the interrupt is customizable for every channel, this enumeration is not available */
} ADI_IIR_EVENT;

/*! Global APIs */

/* This API will reset the IIR accelerator, and install the interrupt handler

   Device Num:  IIR device number
   hIirHandle:  Address to store the IIR device handle
*/
ADI_IIR_RESULT adi_iir_Open(uint8_t              nDeviceNum,
                            ADI_IIR_DEV_HANDLE *phIirHandle);


/* This API will do the following:
   - Create a IIR task for given IIR device,
   - Create a TCB chain from given Channel list

   hIirHandle:   IIR device handle
   pChannelList: Array of Channel information using which TCB chain will be created
   nNumChannels: Number of Channels(TCBs) in this task
   pMemory:      Memory to store the TCB chain.
   nMemSize:     Size of the memory provided (calculated on the basis of number of channels)
   phIirTask:    Address to store the IIR task handle
*/
ADI_IIR_RESULT adi_iir_CreateTask(ADI_IIR_DEV_HANDLE     hIirHandle,
                                  ADI_IIR_CHANNEL_INFO  *pChannelList,
                                  uint32_t               nNumChannels,
                                  void                  *pMemory,
                                  uint32_t               nMemSize,
                                  ADI_IIR_TASK_HANDLE   *phIirTask);

#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
/* This API will do the following:
   - Allocate this task to accelerator if accelerator is free.
   - Push this task to queue if accelerator is busy.

   hIirTask:     IIR task handle
*/
ADI_IIR_RESULT adi_iir_QueueTask(ADI_IIR_TASK_HANDLE hIirTask);
#endif /*(ADI_IIR_CFG_NOQUEUE_MODE == 0) */

#if (ADI_IIR_CFG_NOQUEUE_MODE == 1)
ADI_IIR_RESULT adi_iir_StartTask (ADI_IIR_TASK_HANDLE hIirTask);

ADI_IIR_RESULT adi_iir_UnassignTask (ADI_IIR_DEV_HANDLE hIirHandle);
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 1) */


/* This API will do the following:
   - This task will be deleted and application will not be able to reuse it
     Once task is created, and is completed by IIR accelerator, application need not create it again.
     If application want to reschedule that task, it can just Queue it again.

   hIirTask:     IIR task handle
*/
ADI_IIR_RESULT adi_iir_DeleteTask(ADI_IIR_TASK_HANDLE hIirTask);


/* This API will return the status of given task.

   hIirTask:     IIR task handle
   TaskStatus:   Address to store the task status
*/
ADI_IIR_RESULT adi_iir_GetIirTaskStatus(ADI_IIR_TASK_HANDLE hIirTask, ADI_IIR_TASK_STATE *TaskStatus);

/* This API will configure the mode of operation as Floating Point.

   hIirTask:     IIR task handle
*/
ADI_IIR_RESULT adi_iir_FloatingPointEnable (ADI_IIR_TASK_HANDLE         hIirTask,
                                            ADI_IIR_FLOAT_ROUNDING_MODE eRoundingMode);

/* This API will enable/disable the Save Biquad State Feature of the IIR accelerator for a given task.

   hIirTask:     IIR task handle
*/
ADI_IIR_RESULT adi_iir_SaveStateEnable (ADI_IIR_TASK_HANDLE         hIirTask,
		                                const bool                  bFlag);

/* This API will selects between 32-bit IEEE floating-point format or
 * 40-bit IEEE floating-point format .

   hIirTask:     IIR task handle
*/
ADI_IIR_RESULT adi_iir_FortyBitSelect (ADI_IIR_TASK_HANDLE         hIirTask,
		                                const bool                  bFlag);

/* This API will close this IIR accelerator instance and application will have to call adi_iir_Open() API again if it needs to use the IIR accelerator

   hIirHandle:   IIR device handle
*/
ADI_IIR_RESULT adi_iir_Close(ADI_IIR_DEV_HANDLE hIirHandle);

/* The adi_iir_RegisterCallback API registers an application defined IIR Task Done Callback

   hIirHandle:   IIR device handle
   pfCallback:   Pointer to application defined Callback function
   pCBParam:      Pointer to application defined Callback parameter
*/
ADI_IIR_RESULT adi_iir_RegisterCallback(ADI_IIR_DEV_HANDLE hIirHandle,
		                                ADI_CALLBACK       pfCallback,
										void *             pCBParam);

/* This API is used when a task is to be re-used in ACM Mode. In ACM Mode, the Buffer Indexes of the task need to be updated using this API as the hardware
 * doesn't update the buffer Indexes(only in ADSP-21569). In Legacy Mode, the Hardware updates the Buffer Indexes after completion of the Task. For More details, please refer to the Hardware
 * Reference Manual.

   hIirTask:     IIR task handle
   pChannelIndexList:   Array of Channel Buffer Indexes
   nNumChannels:    Number of Channels. Note that the number of channels must be the same as the value when creating a task.
*/
ADI_IIR_RESULT adi_iir_UpdateTask(ADI_IIR_TASK_HANDLE          hIirTask,
		                          ADI_IIR_CHANNEL_BUFFER_INFO  *pChannelParamsList,
								  uint32_t                     nNumChannels);

#if (defined(__ADSPSHARCFX__))
ADI_IIR_RESULT adi_iir_ConfigSecInt (ADI_IIR_DEV_HANDLE     hIirHandle,
                                                           bool bSecInt );
#endif /* (defined(__ADSPSHARCFX__)) */



#endif /* __ADI_IIR_LEGACY_V3_H__ */
/*@}*/
