/*********************************************************************************

Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_fir_acm_v3.c
*
* @brief     This is the primary source file for the FIR Driver implementation in ACM mode for ADSP-SC83x processors.
*
*            Contains the implementation of the APIs for the FIR accelerator in ACM mode for ADSP-SC83x processors.
*
*/

#ifndef _ADI_FIR_ACM_V3_C_
#define _ADI_FIR_ACM_V3_C_
/*==========  I N C L U D E  ==========*/
#include <stdlib.h>
#include <string.h>
#include <adi_osal.h>
#include <services/int/adi_int.h>
#include <assert.h>
#include <sys/platform.h>
#include <drivers/fir/adi_fir.h>
#include "adi_fir_def_v3.h"
#include <sys/adi_core.h>
#include <services/int/adi_sec.h>
#include <adi_sys_addr.h>

/*==========  L O C A L    F U N C    D E F S  ==========*/
static void FirInterruptHandler (uint32_t iid, void* pParam);

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_FIR_RESULT ResetAccelerator(ADI_FIR_DEV_INFO *pDevice);

/* Disables and Enables Accelerator */
static ADI_FIR_RESULT DisableandEnableAccelerator(ADI_FIR_DEV_INFO *pDevice);

/* Translates the Given TCB address by adding appropriate offsets */
static uint32_t TranslateAddr (uint32_t  MemAddr);

static uint32_t RevertTranslatedAddr (uint32_t  MemAddr);

/* Validate the given device handle */
static ADI_FIR_RESULT ValidateDeviceHandle (ADI_FIR_DEV_HANDLE hDevice);

/* Validate the given Task Handle */
static ADI_FIR_RESULT ValidateTaskHandle (ADI_FIR_TASK_HANDLE hTask);

/*==========  D E B U G  S T R U C T U R E S  ==========*/

#if defined(ADI_USE_DEBUG_DATA)
/* It is advised not to use debug Data while running infinite loop programs as the indexes for these
 * structures/data may go out of bound and cause unexpected results in the programs
 */


ADI_FIR_DEBUG_DATA gFirDebugData[] =
{
	{
	    {0,0},
	    {0,0},
	    {0},
	    0
	}
	,{
		{0,0},
		{0,0},
		{0},
		0
	}
};

#endif /* ADI_USE_DEBUG_DATA */


/*==========  D A T A  ==========*/
static ADI_FIR_DEV_INFO gFirDeviceInfo[] =
{
    {
        ADI_FIR_DEV_STATE_NOTINIT,                              /* FIR device state */
    	NULL,                                                   /* Pointer to FIR Callback function */
    	NULL,                                                   /* Pointer to Callback Data Parameter */
		NULL,                                                   /* This pointer points to the end TCB of the Accelerator TCB Chain. */
		NULL,                                                   /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
		0u,                                                     /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
		(ADI_FIR_TypeDef *)ADI_FIR0_BASE,                       /* Pointer to Base Address of FIR instance registers */
#if (ADI_FIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_FIR0_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_FIR0_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_FIR_CFG_SEC_INT == 1) */
#if defined(ADI_USE_DEBUG_DATA)
       ,(volatile ADI_FIR_DEBUG_DATA *)&gFirDebugData[0]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
    }
    ,{
        ADI_FIR_DEV_STATE_NOTINIT,                              /* FIR device state */
    	NULL,                                                   /* Pointer to FIR Callback function */
    	NULL,                                                   /* Pointer to Callback Data Parameter */
		NULL,                                                   /* This pointer points to the end TCB of the Accelerator TCB Chain. */
		NULL,                                                   /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
		0u,                                                     /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
		(ADI_FIR_TypeDef *)ADI_FIR1_BASE,                       /* Pointer to Base Address of FIR instance registers */

#if (ADI_FIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_FIR1_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_FIR1_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_FIR_CFG_SEC_INT == 1) */
#if defined(ADI_USE_DEBUG_DATA)
       ,(volatile ADI_FIR_DEBUG_DATA *)&gFirDebugData[1]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
    }
};

/*==========  C O D E  ==========*/


ADI_FIR_RESULT adi_fir_Open (uint8_t             nDeviceNum,
							 ADI_FIR_DEV_HANDLE *phFirHandle)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
	
#if defined (ADI_DEBUG)
    assert(nDeviceNum <= (FIR_NUM_DEVICES - 1u));
    assert(phFirHandle != NULL);
#endif

    {
		/* Get the pointer to device */
		pDevice = &gFirDeviceInfo[nDeviceNum];

		/* Reset FIR_CTL1 and FIR_CHNPTR Registers */
		ResetAccelerator(pDevice);

		/* Configure the Global Control Register using static configuration */
		pDevice->pFIRRegs->CTL1 = FIR_DEF_FIRCTL1;

		/* Install the FIR interrupt handler */
		adi_int_InstallHandler(pDevice->nFirDoneIntId, &FirInterruptHandler, pDevice, true);

#if ((ADI_FIR_CFG_SEC_INT == 1))
		/* Configure the FIR Interrupt as Edge Sensitive at SEC */
		adi_sec_EnableEdgeSense(ADI_IID_TO_SEC_INT(pDevice->nFirDoneIntId),true);
#endif /* ((ADI_FIR_CFG_SEC_INT == 1)) */

#if ((ADI_FIR_CFG_SEC_INT == 0))
		/* Configure the FIR Direct Interrupt as Edge Sensitive */
        xthal_interrupt_sens_set(ADI_IID_TO_CORE_INT(pDevice->nFirDoneIntId), 0);
#endif /* ((ADI_FIR_CFG_SEC_INT == 0)) */

		/* Initialize the Device State */
		pDevice->eDevState = ADI_FIR_DEV_STATE_INIT;

		/* Write the Handle to the location given by the application */
		*phFirHandle = pDevice;
    }
    /* Return the status back to the application */
    return (eResult);
}

ADI_FIR_RESULT adi_fir_Close (ADI_FIR_DEV_HANDLE hFirHandle)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);
#endif
    /* Get the device handle */
    pDevice = (ADI_FIR_DEV_INFO*)hFirHandle;

    /* Reset FIR_CTL1 and FIR_CHNPTR Registers */
    ResetAccelerator(pDevice);

    /* Uninitialize the Device State and Queue Info*/
    pDevice->eDevState = ADI_FIR_DEV_STATE_NOTINIT;
    pDevice->pLastTCBinQueue = NULL;
    pDevice->pActiveTaskTCB  = NULL;
    pDevice->nIntCntOfCurrentTask = 0u;
    pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX] = NULL;
    pDevice->pFIRCBParam[ADI_FIR_MASTER_CALLBACK_INDEX] = NULL;

    /* Uninstall FIR interrupt handler */
    if( adi_int_UninstallHandler (pDevice->nFirDoneIntId) != ADI_INT_SUCCESS)
    {
        eResult = ADI_FIR_RESULT_FAILED;
    }

    /* Return the status */
    return (eResult);
}

ADI_FIR_RESULT adi_fir_CreateTask (ADI_FIR_DEV_HANDLE     hFirHandle,
                                   ADI_FIR_CHANNEL_INFO  *pChannelList,
                                   uint32_t               nNumChannels,
                                   void                  *pMemory,
                                   uint32_t               nMemSize,
                                   ADI_FIR_TASK_HANDLE   *phFirTask)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
    ADI_FIR_TCB_INFO *pTCBChain;
    uint32_t FirCtl2;
    uint32_t FirCtl1;
    ADI_FIR_TCB_INFO * pCurrentTCB;
	ADI_FIR_TCB_INFO * pTempTCB;
    uint32_t count;
    uint32_t IntCount;
    uint32_t count1 = 0u;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);
    /* Check whether pointers passed are not NULL */
    assert((pChannelList != NULL) &&  (pMemory != NULL) && (phFirTask != NULL));
    /* Check if the memory supplied is sufficient */
    assert(nMemSize >=(ADI_FIR_TASK_INFO_SIZE + (nNumChannels * ADI_FIR_TCB_INFO_SIZE)));

    /* Check if the No of Taps for each channel is less than Max. No of Taps*/
    for(count1 = 0u; count1< nNumChannels; count1++)
    {
        assert((pChannelList[count1].nTapLength) <= FIR_MAX_NUM_TAPS_PER_CHANNEL);
    }
#endif /* ADI_DEBUG */
    /* Initialize Interrupt Count variable. This variable will be written to the Task Info Structure in the end  */
    IntCount = 0u;

    /* Get the device handle */
    pDevice = (ADI_FIR_DEV_INFO*)hFirHandle;

    /* Make the Task Head as the pMemory Supplied*/
    pTCBChain = (ADI_FIR_TCB_INFO*)pMemory;

    /*Create a TCB Chain */
    pCurrentTCB = pTCBChain;


	for(count = 0u; count< nNumChannels; count++)
	{
		/*Create FIRCTL2 register */
		FirCtl2 = (pChannelList[count].nTapLength - 1u) | ((pChannelList[count].nWindowSize -1u)<<BITP_FIR_CTL2_WINDOW);
		if(pChannelList[count].eSampling != ADI_FIR_SAMPLING_SINGLE_RATE)
		{
			FirCtl2 |= BITM_FIR_CTL2_SRCEN;
			FirCtl2 |= (pChannelList[count].eSampling == ADI_FIR_SAMPLING_INTERPOLATION) ? (((pChannelList[count].nSamplingRatio) -1u)<<BITP_FIR_CTL2_RATIO) | BITM_FIR_CTL2_UPSAMP : (((pChannelList[count].nSamplingRatio) -1u)<<BITP_FIR_CTL2_RATIO);
		}
		else
		{
			/* To avoid MISRA Warning */
		}

		if((pChannelList[count].bCallbackEnable == false) && (count != (nNumChannels - 1u)))
		{
			FirCtl2 |= BITM_FIR_CTL2_IMASK;
		}
		else
		{
		   /* Update the Number of Tasks that Contain Interrupt Enabled in the Chain.*/
			IntCount++;
		}
		if(pChannelList[count].bTriggerEnable == false)
		{
			FirCtl2 |= BITM_FIR_CTL2_TMASK;
		}

		if(pChannelList[count].bTriggerWaitEnable == true)
		{
			FirCtl2 |= BITM_FIR_CTL2_TWAIT;
		}

		/* Write to the nFirControl2 field */
		pCurrentTCB->nFirControl2 = FirCtl2;

		/*Create Secondary Global Control register */
		FirCtl1 = ((uint32_t)pChannelList[count].eRoundingMode) << BITP_FIR_CTL1_RND;
		if(pChannelList[count].bFixedPoint == true)
		{
			FirCtl1 |= BITM_FIR_CTL1_FXD;
			if(pChannelList[count].eFixedPoint == ADI_FIR_FIXED_INPUT_FORMAT_SIGNED_INTEGER)
			{
				FirCtl1 |= BITM_FIR_CTL1_TC;
			}
		}

		/* Write to the Secondary Global Control Register Field */
		pCurrentTCB->nFirSecondaryControl1 = FirCtl1;

		/*Copy the Coefficient, Input and Output Parameters */
		pCurrentTCB->nCoeffCount    = (uint32_t)pChannelList[count].nCoefficientCount;
		pCurrentTCB->nCoeffModify   = (int32_t)pChannelList[count].nCoefficientModify;
		pCurrentTCB->pCoeffIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pCoefficientIndex);

		pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffBase);
		pCurrentTCB->nOutputCount   = (uint32_t)pChannelList[count].nOutputBuffCount;
		pCurrentTCB->nOutputModify  = (int32_t)pChannelList[count].nOutputBuffModify;
		pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffIndex);

		pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffBase);
		pCurrentTCB->nInputCount    = (uint32_t)pChannelList[count].nInputBuffCount;
		pCurrentTCB->nInputModify   = (int32_t)pChannelList[count].nInputBuffModify;
		pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffIndex);

		/* Update Next Pointer of TCB and other parameters of the Last TCB */
		if(count == (nNumChannels - 1u))
		{
			pCurrentTCB->pNextTCB = NULL;

			/* Make the Software Control1 field of the First TCB point task Info Structure */
			pTCBChain->nFirSoftwareControl1 = pCurrentTCB+1;

			/* Write the Interrupt Count field of the Task Info structure */
			GET_INTERRUPT_COUNT((ADI_FIR_TCB_INFO *)pTCBChain) = IntCount;

			/* Write the Number of Channels field of the Task Info  structure */
			GET_CHANNEL_COUNT((ADI_FIR_TCB_INFO *)pTCBChain) = nNumChannels;

			/* Write to the pLastChannel field of the task info Structure */
			GET_LAST_TCB(pTCBChain) = pCurrentTCB;

			/* Update Task Status in Task Info Structure */
			GET_TASK_STATUS(pTCBChain) = ADI_FIR_TASK_STATE_NONE;

			/* Write the Device Handle in the Task Info Structure */
			GET_DEVICE_HANDLE(pTCBChain) = (void *)hFirHandle;

			/* Update the flag which indicates whether application wants Callback for the Last Channel or no */
			GET_LAST_CHANNEL_CALLBACK_ENABLE(pTCBChain)=  pChannelList[count].bCallbackEnable;
		}
		else
		{
			  pTempTCB = pCurrentTCB + 1;
			  pCurrentTCB->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTempTCB->nFirControl2));
		}

		pCurrentTCB = pCurrentTCB + 1;

	}

    /* Write the Task Handle to the location supplied */
    *phFirTask = pTCBChain;

    /* Return the Status */
    return (eResult);
}

ADI_FIR_RESULT adi_fir_QueueTask (ADI_FIR_TASK_HANDLE hFirTask)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_TCB_INFO *pTask = (ADI_FIR_TCB_INFO*)hFirTask;
	ADI_FIR_DEV_STATE eDevState;
	ADI_FIR_DEV_INFO *pDevice;
    uint32_t nLastChannelSCTL1;

#if (ADI_CACHE_MANAGEMENT == 1)
   ADI_FIR_TCB_INFO* pTCBChain;
   uint32_t nNumChannels = (uint32_t)0;
   uint32_t Count = (uint32_t)0;
   void *_start = NULL;
   void *_end = NULL;
   void  *temp;
   int32_t nCoeffModify = 0;
#endif /* (ADI_CACHE_MANAGEMENT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
#endif /* ADI_USE_DEBUG_DATA */
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
    /* Check if the Task is running or queued */
    assert((GET_TASK_STATUS(pTask) != ADI_FIR_TASK_STATE_QUEUED) && (GET_TASK_STATUS(pTask) != ADI_FIR_TASK_STATE_RUNNING));
#endif
	/* Additional Check in ACM Mode */
    pDevice = GET_DEVICE_HANDLE(pTask);
#if defined (ADI_DEBUG)
	assert(pDevice->eDevState == ADI_FIR_DEV_STATE_INIT);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

#if (ADI_CACHE_MANAGEMENT == 1)
    pTCBChain = (ADI_FIR_TCB_INFO *)hFirTask;
    nNumChannels = GET_CHANNEL_COUNT((ADI_FIR_TCB_INFO *)pTCBChain);

    /* Input Buffer-Flush,Output Buffer-Invalidate, Coefficient Buffer-Flush */
    for(Count=0u;Count<nNumChannels;Count++)
    {
       /* Flush Input Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pInputBase));
       _end = (float *)_start + (pTCBChain->nInputCount);
       flush_data_buffer(_start,_end,0);

       /* Invalidate Output Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pOutputBase));
       _end = (float *)_start + (pTCBChain->nOutputCount);
       flush_data_buffer(_start,_end,1);

       /* Flush Coefficient Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pCoeffIndex));
       nCoeffModify = pTCBChain->nCoeffModify;
       if(nCoeffModify < 0)
       {
           _end = (float *)_start - (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffCount));
       }
       else
       {
    	   _end = (float *)_start + (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffCount));
       }
       flush_data_buffer(_start,_end,0);

       if(Count!= (nNumChannels-1u))
       {
		   /* Update the TCB Pointer to point to the next TCB */
		   temp = pTCBChain->pNextTCB;
		   pTCBChain = (void *)((uint32_t *)RevertTranslatedAddr((uint32_t)(temp)) - ((ADI_FIR_TCB_INFO_SIZE/4u)-1u));
       }

    }

    /* Flush the TCB Chain */
    _start = (ADI_FIR_TCB_INFO *)hFirTask;
    _end = ((ADI_FIR_TCB_INFO *)_start) + nNumChannels;
    flush_data_buffer(_start,_end,0);

#endif /* (ADI_CACHE_MANAGEMENT == 1) */
    /* Halt Accelerator */
    (pDevice->pFIRRegs->CTL1) |= BITM_FIR_CTL1_HALT;

    /* Check if the Accelerator is Idle or Running */
    if(((pDevice->pFIRRegs->DMASTAT) & BITM_FIR_DMA_OVERALL_STATUS) != 0u)
    {
/*-------------------FOR_DEBUG------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
		eCoreID = adi_core_id();
		if(eCoreID == ADI_CORE_SHARC0)
		{
			pDevice->pFirDebugData->QueueTracker[pDevice->pFirDebugData->QueueTrackerIndex++] = 1;
		}
#if __NUM_SHARC_CORES__ > 1
	else if(eCoreID == ADI_CORE_SHARC1)
		{
		    pDevice->pFirDebugData->QueueTracker[pDevice->pFirDebugData->QueueTrackerIndex++] = 2;
		}
#endif /*  __NUM_SHARC_CORES__ */
	else
	    {
            /* MISRA Compliance */
	    }
#endif /* ADI_USE_DEBUG_DATA */
/*-------------------FOR_DEBUG------------------------------*/
        /* Running */
#if defined (ADI_DEBUG)
    	assert(pDevice->pLastTCBinQueue != NULL);
#endif
    	/* Update Queue */
    	pDevice->pLastTCBinQueue->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTask->nFirControl2));
    	/* If the previous TCB is in L2, the above update of linking the previous TCB to the Current TCB has to flushed */
    	flush_data_buffer((&(pDevice->pLastTCBinQueue->pNextTCB)),(&(pDevice->pLastTCBinQueue->pNextTCB)) + 1,0);
    	pDevice->pLastTCBinQueue = (ADI_FIR_TCB_INFO *)GET_LAST_TCB(pTask);

    	/* Check if the CP is NULL. If yes, write to CP. If accelerator is processing the last channel of the Task. */
    	if((pDevice->pFIRRegs->CHNPTR) == (uint32_t)NULL)
    	{
    		(pDevice->pFIRRegs->CHNPTR) = (uint32_t)TranslateAddr((uint32_t)(&pTask->nFirControl2));
    	}

    	/* Update the Task status */
    	GET_TASK_STATUS(pTask) = ADI_FIR_TASK_STATE_QUEUED;
    }
    else
    {
        /* Idle */

    	/* Check if all the interrupts have occurred. If not update the Queue as interrupts are still pending */
    	if(pDevice->pLastTCBinQueue != NULL)
    	{
    		/* Update Queue */
        	pDevice->pLastTCBinQueue->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTask->nFirControl2));
        	/* If the previous TCB is in L2, the above update of linking the previous TCB to the Current TCB has to flushed */
        	flush_data_buffer((&(pDevice->pLastTCBinQueue->pNextTCB)),(&(pDevice->pLastTCBinQueue->pNextTCB)) + 1,0);
        	pDevice->pLastTCBinQueue = (ADI_FIR_TCB_INFO *)GET_LAST_TCB(pTask);
        	/* Update the Task status */
        	GET_TASK_STATUS(pTask) =  ADI_FIR_TASK_STATE_QUEUED;
    	}
    	else
    	{
    		/* Update Queue */
    		pDevice->pLastTCBinQueue = (ADI_FIR_TCB_INFO *)GET_LAST_TCB(pTask);
    		pDevice->pActiveTaskTCB = pTask;

        	/* Update the Task status */
    		GET_TASK_STATUS(pTask) = ADI_FIR_TASK_STATE_RUNNING;

        	/* Load the Interrupt Count of the Current Task */
        	pDevice->nIntCntOfCurrentTask = GET_INTERRUPT_COUNT(pTask);
    	}
    	/* Write to CP, Disable and enable Accelerator */
    	(pDevice->pFIRRegs->CHNPTR) = (uint32_t)TranslateAddr((uint32_t)(&pTask->nFirControl2));
	    DisableandEnableAccelerator(pDevice);

    }

    /* Resume Accelerator */
    (pDevice->pFIRRegs->CTL1) &= ~BITM_FIR_CTL1_HALT;

    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the result */
    return (eResult);
}

ADI_FIR_RESULT adi_fir_GetFirTaskStatus (ADI_FIR_TASK_HANDLE hFirTask, ADI_FIR_TASK_STATE *TaskStatus)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_TCB_INFO *pTask;
    uint32_t nLastChannelSCTL1;
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
    /* Check if the address passed is NULL */
    assert(TaskStatus != NULL);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Get the Task structure */
    pTask = (ADI_FIR_TCB_INFO*) hFirTask;

    /* Write the Task Status to the given location */
    *TaskStatus = (ADI_FIR_TASK_STATE)GET_TASK_STATUS(pTask);

    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return status */
    return (eResult);
}

ADI_FIR_RESULT adi_fir_RegisterCallback (ADI_FIR_DEV_HANDLE hFirHandle,
                                         ADI_CALLBACK       pfCallback,
		                                 void *             pCBParam)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);
#endif
    /* Get the Device Handle */
	pDevice = (ADI_FIR_DEV_INFO*) hFirHandle;

	/* Register the Callback function and Callback parameter */
    pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX] = pfCallback;
    pDevice->pFIRCBParam[ADI_FIR_MASTER_CALLBACK_INDEX]   = pCBParam;

    /* Return status */
    return (eResult);
}

ADI_FIR_RESULT adi_fir_UpdateTask(ADI_FIR_TASK_HANDLE hFirTask,
		                          ADI_FIR_CHANNEL_BUFFER_INFO  *pChannelParamsList,
							      uint32_t               nNumChannels)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_TCB_INFO *pTask = (ADI_FIR_TCB_INFO*)hFirTask;
    ADI_FIR_TCB_INFO * pCurrentTCB = NULL;
    uint32_t count = 0u;
#if defined (ADI_DEBUG)
    /* Validate the Task handle*/
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
    /* Check whether pointers passed are not NULL */
    assert((pChannelParamsList != NULL));
#endif
    if( (GET_CHANNEL_COUNT((ADI_FIR_TCB_INFO *)pTask) != nNumChannels) || (GET_TASK_STATUS((ADI_FIR_TCB_INFO *)pTask) != ADI_FIR_TASK_STATE_COMPLETED) )
    {
    	eResult = ADI_FIR_RESULT_INVALID_OPERATION;
    }
    else
    {
    	pCurrentTCB = pTask;

    	/* Update the Circular Buffer Parameters in a loop */
    	for(count = 0u; count< nNumChannels; count++)
    	{
    		pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffBase);
    		pCurrentTCB->nOutputCount   = (uint32_t)pChannelParamsList[count].nOutputBuffCount;
    		pCurrentTCB->nOutputModify  = (int32_t)pChannelParamsList[count].nOutputBuffModify;
    		pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffIndex);

    		pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffBase);
    		pCurrentTCB->nInputCount    = (uint32_t)pChannelParamsList[count].nInputBuffCount;
    		pCurrentTCB->nInputModify   = (int32_t)pChannelParamsList[count].nInputBuffModify;
    		pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffIndex);

    		pCurrentTCB = pCurrentTCB + 1;
    	}

    }

    /* Return Result */
    return (eResult);

}


#if (defined(__ADSPSHARCFX__))
/*!
 * @brief       The adi_fir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct.
 *
 * @details     The adi_fir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct..
 *
 * @param [in]  hFirHandle         FIR Device handle.
 *
 * @param [in]  bSecInt    True - Non-direct Interrupt; False - Direct Interrupt
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully configured the Output Rounding Mode.
 *
 *  - #ADI_FIR_RESULT_FAILED           If Task State is running.
 *
 * @sa          adi_fir_FixedPointEnable().
 */
ADI_FIR_RESULT adi_fir_ConfigSecInt (ADI_FIR_DEV_HANDLE     hFirHandle,
                                                           bool bSecInt )
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
     assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);
#endif
     uint32_t sid, cid, iid;

     ADI_INT_ID_STATUS res;

     /* Get the device handle */
     pDevice = (ADI_FIR_DEV_INFO*)hFirHandle;

     iid = pDevice->nFirDoneIntId;

     if( ADI_IID_IS_CORE_INT(iid))
     {
    	 cid = iid;
    	 res = get_sid(iid, &sid);
     }
	 else
	 {
         sid = iid;
         res = get_cid(iid, &cid);
	 }

     if (res == ADI_INT_ID_SUCCESS)
     {
    	 if( bSecInt == true)
    	 {
    		 /* Install Non-Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(sid, FirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_FIR_RESULT_FAILED;
    		 }

    	 }
    	 else
    	 {
    		 /* Install Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(cid, FirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_FIR_RESULT_FAILED;
    		 }
    	 }
     }
     else
     {
    	 return ADI_FIR_RESULT_FAILED;
     }

     return ADI_FIR_RESULT_SUCCESS;
}
#endif /* defined(__ADSPSHARCFX__) */


/* FirInterrupt Handler */
static void FirInterruptHandler (uint32_t iid, void* pParam)
{
	ADI_FIR_DEV_INFO *pDevice = (ADI_FIR_DEV_INFO*)pParam;
	ADI_FIR_TCB_INFO *pTask;
    uint32_t nDMAStat;
/*-------------------FOR_DEBUG--------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
	ADI_CORE_ID eCoreID;
	eCoreID = adi_core_id();
	if(eCoreID == ADI_CORE_SHARC0)
	{
		pDevice->pFirDebugData->FirIntCnt[0]++;
	}
#if __NUM_SHARC_CORES__ > 1
	else if(eCoreID == ADI_CORE_SHARC1)
	{
		pDevice->pFirDebugData->FirIntCnt[1]++;
	}
#endif /* __NUM_SHARC_CORES__ > 1 */
	else
	{
            /* MISRA Compliance */
	}
#endif /* ADI_USE_DEBUG_DATA */
/*------------------FOR_DEBUG----------------------------------*/

    /* Get the Handle of the Task that is completed */
    pTask = pDevice->pActiveTaskTCB;

    /* Get DMA Status. Clear All Channels Done Interrupt Status */
    nDMAStat = (pDevice->pFIRRegs->DMASTAT);

	/* Decrement Count */
	(pDevice->nIntCntOfCurrentTask--);

	/* Check if the Task is Complete (Count == 0) */
	if(pDevice->nIntCntOfCurrentTask == 0u)
	{
		/* Update Status of the Completed Task */
		GET_TASK_STATUS(pTask) = ADI_FIR_TASK_STATE_COMPLETED;

		/* Check if there are more tasks in Queue */
		if(GET_LAST_TCB(pTask) == pDevice->pLastTCBinQueue)
		{
			/* Reset Queue Pointers */
			pDevice->pActiveTaskTCB = NULL;
			pDevice->pLastTCBinQueue = NULL;

		}
		else
		{
			/* Update pActiveTaskTCB of Device Info */
			void  *temp = (((ADI_FIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB);
			pDevice->pActiveTaskTCB = (ADI_FIR_TCB_INFO *)(((uint32_t *)RevertTranslatedAddr((uint32_t)temp)) - ((ADI_FIR_TCB_INFO_SIZE/4u)-1u));

			/* Update Task Status of the next Task */
			GET_TASK_STATUS(pDevice->pActiveTaskTCB) = ADI_FIR_TASK_STATE_RUNNING;

			/* Update Interrupt Count in Device Info */
			pDevice->nIntCntOfCurrentTask = GET_INTERRUPT_COUNT(pDevice->pActiveTaskTCB);

			/* Dequeue the Completed Task */
			((ADI_FIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB = NULL;
            /* If the TCB chain which is completed is in L2, then the above update of dequeing the completed TCB chain must be flushed */
			flush_data_buffer(&(((ADI_FIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB),(&(((ADI_FIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB)) + 1,0);
		}
		/*Check whether the Last Channel Callback is needed by the application or not */
		if(GET_LAST_CHANNEL_CALLBACK_ENABLE(pTask) == true)
		{
			/* Call Callback */
			if(pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX] != NULL)
			{
				pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX](pDevice->pFIRCBParam[ADI_FIR_MASTER_CALLBACK_INDEX],(uint32_t)ADI_FIR_EVENT_CHANNEL_DONE,(void *)pTask);
			}
		}
	}
	else
	{
		/* Call Callback */
		if(pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX] != NULL)
		{
			pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX](pDevice->pFIRCBParam[ADI_FIR_MASTER_CALLBACK_INDEX],(uint32_t)ADI_FIR_EVENT_CHANNEL_DONE,(void *)pTask);
		}

	}

}

/* Validate the Device Handle */
static ADI_FIR_RESULT ValidateDeviceHandle (ADI_FIR_DEV_HANDLE hDevice)
{
    unsigned int x;
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_INVALID_HANDLE;
    ADI_FIR_DEV_INFO* pDevice = (ADI_FIR_DEV_INFO*)hDevice;

    for (x = 0u; x < FIR_NUM_DEVICES; x++)
    {
        if (pDevice == &gFirDeviceInfo[x])
        {

            eResult = ADI_FIR_RESULT_SUCCESS;

        }
    }

    return (eResult);
}

/* Validate the Task Handle */
static ADI_FIR_RESULT ValidateTaskHandle(ADI_FIR_TASK_HANDLE hTask)
{
    unsigned int x;
    ADI_FIR_TCB_INFO *pTask = (ADI_FIR_TCB_INFO *)hTask;
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_INVALID_HANDLE;
    for (x = 0u; x < FIR_NUM_DEVICES; x++)
    {
        if (GET_DEVICE_HANDLE(pTask) == ((ADI_FIR_DEV_HANDLE)&gFirDeviceInfo[x]))
        {
            eResult = ADI_FIR_RESULT_SUCCESS;
        }
    }
    return (eResult);
}

/* Translates the supplied memory address for DMA operation based on Core ID. */
static uint32_t TranslateAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */
#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_internal_to_system_addr(MemAddr, 1);
#endif

    return (MemAddr >> 2u);
}

/*
 * Reverts the supplied memory address for DMA operation based on Core ID.
 *
 * Parameters
 *  - MemAddr   Memory address to work on
 *
 * Returns
 *  - Translated Memory address for DMA access.
 *
 */
static uint32_t RevertTranslatedAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */

    MemAddr = MemAddr << 2u;

#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_system_to_internal_addr(MemAddr);
#endif

    return MemAddr;
}

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_FIR_RESULT ResetAccelerator(ADI_FIR_DEV_INFO *pDevice)
{
	(pDevice->pFIRRegs->CTL1) = 0u;
	(pDevice->pFIRRegs->CHNPTR)= 0u;
	
	return ADI_FIR_RESULT_SUCCESS;
}

static ADI_FIR_RESULT DisableandEnableAccelerator(ADI_FIR_DEV_INFO *pDevice)
{
	(pDevice->pFIRRegs->CTL1) &= ~(BITM_FIR_CTL1_EN | BITM_FIR_CTL1_DMAEN); /* can be optimized( dmaen clear/set may not be required) */
	(pDevice->pFIRRegs->CTL1) |= (BITM_FIR_CTL1_EN | BITM_FIR_CTL1_DMAEN);

	return ADI_FIR_RESULT_SUCCESS;
}

#endif /*_ADI_FIR_ACM_V3_C_*/

