/*!
 *****************************************************************************
 * @file:    adi_fir_def_acm_v3.h
 * @brief:   FIR Device Driver definition for ACM Mode
 * @version: $Revision: 37632 $
 * @date:    $Date: 2019-10-09 12:35:37 +0530 (Wed, 09 Oct 2019) $
 *****************************************************************************
Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.
This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_fir_def_acm_v3.h
     Description  :   Static function and structure definition needed by the device driver
                      for managing the FIR  device for ADSP-SC83x processors.
================================================================================ */

#ifndef __ADI_FIR_DEF_ACM_V3_H__
#define __ADI_FIR_DEF_ACM_V3_H__



/*==========  D E F I N E S  ==========*/


#define FIR_MAX_NUM_TAPS_PER_CHANNEL  (4096u)

#define ADI_FIR_NUM_OF_CALLBACKS       (1u)
#define ADI_FIR_NUM_OF_CALLBACK_PARAMS (1u)
#define ADI_FIR_MASTER_CALLBACK_INDEX  (0u)

#define ADI_FIR_CFG_BURSTEN        (1ul << BITP_FIR_CTL1_BURSTEN)

#define ADI_FIR_CFG_SMQ_LIUPS_EN    (1ul << BITP_FIR_CTL1_SMQ_LIUPS_EN)
#define ADI_FIR_CFG_BURST16_DIS     (1ul << BITP_FIR_CTL1_BURST16_DIS)

/* Default FIRCTL1 Configuration */
#define FIR_DEF_FIRCTL1             (((uint32_t)ADI_FIR_CFG_ACCELERATOR_MODE) << BITP_FIR_CTL1_ACM) \
                                  | (((uint32_t)ADI_FIR_CFG_BURSTEN)) |  (((uint32_t)ADI_FIR_CFG_SMQ_LIUPS_EN)) \
								  |  (((uint32_t)ADI_FIR_CFG_BURST16_DIS))

/* FIR_CTL1 Register macros */
#define FIR_FIRCTL1_EN_ENABLE                     (BITM_FIR_CTL1_EN)
#define FIR_FIRCTL1_DMAEN_ENABLE                  (BITM_FIR_CTL1_DMAEN)

/* Additional Bit Masks,Bit Positions required in ACM Mode for Software Control Registers */
#define BITM_FIR_DMA_OVERALL_STATUS (BITM_FIR_DMASTAT_CPLD | BITM_FIR_DMASTAT_CLD | BITM_FIR_DMASTAT_DLD | BITM_FIR_DMASTAT_PPGS | BITM_FIR_DMASTAT_WRBK)
#define GET_TASK_STATUS(x) ((((ADI_FIR_TASK_INFO *)(((ADI_FIR_TCB_INFO *)(x))->nFirSoftwareControl1))->eTaskState))
#define GET_INTERRUPT_COUNT(x) ((((ADI_FIR_TASK_INFO *)(((ADI_FIR_TCB_INFO *)(x))->nFirSoftwareControl1))->nInterruptCount))
#define GET_CHANNEL_COUNT(x) ((((ADI_FIR_TASK_INFO *)(((ADI_FIR_TCB_INFO *)(x))->nFirSoftwareControl1))->nNumChannels))
#define GET_LAST_TCB(x) ((((ADI_FIR_TASK_INFO *)(((ADI_FIR_TCB_INFO *)(x))->nFirSoftwareControl1))->pLastTCB))
#define GET_DEVICE_HANDLE(x) ((((ADI_FIR_TASK_INFO *)(((ADI_FIR_TCB_INFO *)(x))->nFirSoftwareControl1))->pDevice))
#define GET_LAST_CHANNEL_CALLBACK_ENABLE(x) ((((ADI_FIR_TASK_INFO *)(((ADI_FIR_TCB_INFO *)(x))->nFirSoftwareControl1))->bLastChannelCallback))

/*Task Info Structure */
typedef struct
{
    /* Interrupt Count */
	volatile uint32_t       nInterruptCount;
	/* Number Of Channels */
	uint32_t       nNumChannels;
	/* Pointer to Last TCB */
	void*          pLastTCB;
    /* State of the Task */
	ADI_FIR_TASK_STATE eTaskState;
	/* Flag to indicate that Last Channel Callback is needed */
	bool bLastChannelCallback;
	/* Device Handle */
	void * pDevice;
} ADI_FIR_TASK_INFO;

/* TCB Info Structure */
typedef struct
{
    /* Chain Pointer */
    void*          pNextTCB;

    /* Software Control Registers */
	void*         nFirSoftwareControl1; /* This field of the First TCB in the Chain will point to the Task Info */
    void*         nFirSoftwareControl2; /* This field is not used */

    /* Secondary Global Control Register */
	uint32_t       nFirSecondaryControl1;

    /* Coefficient Buffer */
    uint32_t       nCoeffCount;
    int32_t        nCoeffModify;
    void*          pCoeffIndex;

    /* Output Circular Buffer */
    void*          pOutputBase;
    uint32_t       nOutputCount;
    int32_t        nOutputModify;
    void* volatile pOutputIndex;

    /* Input Circular Buffer */
    void*          pInputBase;
    uint32_t       nInputCount;
    int32_t        nInputModify;
    void* volatile pInputIndex;

    /* Channel Control Register */
    uint32_t       nFirControl2;
} ADI_FIR_TCB_INFO;

#if defined(ADI_USE_DEBUG_DATA)
/* It is advised not to use debug Data while running infinite loop programs as the indexes for these
 * structures/data may go out of bound and cause unexpected results in the programs
 */
typedef struct
{
	int FirIntCnt[2];
	int SWIntCnt[2];
	int QueueTracker[30];
	int QueueTrackerIndex;
}ADI_FIR_DEBUG_DATA;

#endif /* defined(ADI_USE_DEBUG_DATA) */

/* Device Info structure */
typedef struct
{
	ADI_FIR_DEV_STATE              eDevState;
	ADI_CALLBACK                   pfFIRCallback[ADI_FIR_NUM_OF_CALLBACKS];
	void *                         pFIRCBParam[ADI_FIR_NUM_OF_CALLBACK_PARAMS];
	ADI_FIR_TCB_INFO               *pLastTCBinQueue;                                /* This pointer points to the end TCB of the Accelerator TCB Chain. */
    ADI_FIR_TCB_INFO               *pActiveTaskTCB;                                 /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
    volatile uint32_t              nIntCntOfCurrentTask;                            /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
    ADI_FIR_TypeDef *const         pFIRRegs;                                        /* Base Address of FIR instance registers */
    uint32_t                       nFirDoneIntId;                                   /* FIR DMA Done SEC Interrupt ID */
#if defined(ADI_USE_DEBUG_DATA)
    volatile ADI_FIR_DEBUG_DATA    *pFirDebugData;                                  /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
}ADI_FIR_DEV_INFO;


#endif /* __ADI_FIR_DEF_ACM_V3_H__ */


