/*********************************************************************************

Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_fir_legacy_v3.c
*
* @brief     This is the primary source file for the FIR Driver implementation in legacy mode for ADSP-SC83x processor.
*
*            Contains the implementation of the APIs for the FIR accelerator in legacy mode 
*
*/

#ifndef _ADI_FIR_LEGACY_V3_C_
#define _ADI_FIR_LEGACY_V3_C_
/*==========  I N C L U D E  ==========*/
#include <stdlib.h>
#include <string.h>
#include <adi_osal.h>
#include <services/int/adi_int.h>
#include <assert.h>
#include <sys/platform.h>
#include <drivers/fir/adi_fir.h>
#include "adi_fir_def_v3.h"
#include <sys/adi_core.h>
#include <services/int/adi_sec.h>
#include <adi_sys_addr.h>


/** @addtogroup FIR_Driver
 *  @{
 
 FIR Accelerator Driver follows a queuing programming model. Applications can create tasks for the FIR accelerator using the adi_fir_CreateTask() and queue the same as and when required using the adi_fir_QueueTask(). The driver maintains a processing queue of the tasks given/queued by the 
 application for the accelerator and schedules the same to the accelerator one after another. The applications can register a user defined callback using adi_fir_RegisterCallback().  The driver uses the callback to notify the application  on completion of the tasks/channels in a task. 
 The driver also maintains the status of all the tasks the application has queued. The application can also query the status of the task as required using the adi_fir_GetFirTaskStatus().Once the task which was queued is completed by the accelerator, the task is removed from the processing queue. 
 The driver maintains the information of all the tasks which are completed/created. Hence the application can reuse the tasks which were created by simply queuing it again using adi_fir_QueueTask(). The accelerator can be configured to operate in Legacy Mode or Auto Configuration Mode(ACM) using static configuration.
 In Legacy Mode, the driver maintains a software queue(Linked List) of the tasks queued by the application and schedules the same to the accelerator one after another. In Auto Configuration Mode(ACM), the driver utilizes the Halt feature in hardware to implement hardware queuing mechanism. 
 There are additional channel customizable features available in Auto Configuration Mode(ACM) like interrupts/triggers and more. Please refer to ADI_FIR_CHANNEL_INFO for more details. 
 The programming model of the driver is the same for both Legacy and Auto Configuration Mode(ACM) of operation.

 The Driver also has debug data structures(which contains number of interrupts raised) which can be used by the application for debug. The definitions/use of these debug data structures
 are enabled by the macro ADI_USE_DEBUG_DATA which can be added in the application project. It is advised not to use debug Data while running infinite loop programs as the indexes for these
 structures/data may go out of bound and cause unexpected results in the programs
 */

/*==========  L O C A L    F U N C    D E F S  ==========*/

/* Interrupt Handler for the FIR Device */
static void FirInterruptHandler (uint32_t iid, void* pParam);

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_FIR_RESULT ResetAccelerator(ADI_FIR_DEV_INFO *  pDevice);

/* Resets the given Task Structure */
static ADI_FIR_RESULT ResetTaskStructure (ADI_FIR_TASK_INFO *  pTask);

/* Starts the Queue Processing by writing to Chain Pointer and Control Register */
static ADI_FIR_RESULT StartQueue (ADI_FIR_DEV_INFO *  pDevice);

/* Translates the Given TCB address by adding appropriate offsets */
static uint32_t TranslateAddr (uint32_t  MemAddr);

static uint32_t RevertTranslatedAddr (uint32_t  MemAddr);

/* Validate the given device handle */
static ADI_FIR_RESULT ValidateDeviceHandle (ADI_FIR_DEV_HANDLE hDevice);

/* Validate the given Task Handle */
static ADI_FIR_RESULT ValidateTaskHandle (ADI_FIR_TASK_HANDLE hTask);

/*==========  D E B U G  S T R U C T U R E S  ==========*/

#if defined(ADI_USE_DEBUG_DATA)
/* It is advised not to use debug Data while running infinite loop programs as the indexes for these
 * structures/data may go out of bound and cause unexpected results in the programs
 */

ADI_FIR_DEBUG_DATA gFirDebugData[] =
{
	{
	    {0,0},
	    {0,0},
	    {0},
	    0
	}
	,{
		{0,0},
		{0,0},
		{0},
		0
	}
};


#endif /* ADI_USE_DEBUG_DATA */

/*==========  D A T A  ==========*/

/*static*/ ADI_FIR_QUEUE_INFO gFirQueueInfo[] =
{
    {
        NULL,                                                     /* Pointer to Active Task in the Queue */
        NULL,                                                     /* Pointer to Last Added Task in Queue */
		NULL                                                      /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_FIR_DEBUG_DATA *)&gFirDebugData[0]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
    }
	,{
	    NULL,                                                     /* Pointer to Active Task in the Queue */
	    NULL,                                                     /* Pointer to Last Added Task in Queue */
        NULL                                                      /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_FIR_DEBUG_DATA *)&gFirDebugData[1]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
};

/*static*/ ADI_FIR_DEV_INFO gFirDeviceInfo[] =
{
    {
        ADI_FIR_DEV_STATE_NOTINIT,                        /* FIR device state */
    	(ADI_FIR_QUEUE_INFO *)&gFirQueueInfo[0],          /* FIR Queue Info */
    	{NULL},                                           /* Pointer to FIR Callback functions */
    	{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_FIR_TypeDef *)ADI_FIR0_BASE,                  /* Pointer to Base Address of FIR instance registers */
#if (ADI_FIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_FIR0_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_FIR0_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_FIR_CFG_SEC_INT == 1) */
    }
	,{
	    ADI_FIR_DEV_STATE_NOTINIT,                        /* FIR device state */
	    (ADI_FIR_QUEUE_INFO *)&gFirQueueInfo[1],          /* FIR Queue Info */
	    {NULL},                                           /* Pointer to FIR Callback functions */
	    {NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_FIR_TypeDef *)ADI_FIR1_BASE,                  /* Pointer to Base Address of FIR instance registers */
#if (ADI_FIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_FIR1_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_FIR1_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_FIR_CFG_SEC_INT == 1) */
	}
};

#define FIR_MUTEX_LOCK(X)
#define FIR_MUTEX_UNLOCK(X)

/*==========  C O D E  ==========*/

/*!
 * @brief       adi_fir_Open API opens a FIR device.
 *
 * @details     Opens a specified FIR device and returns the handle to the device. It also installs the interrupt handler and configures the interrupt.
 *
 * @param [in]  nDeviceNum     The FIR device number to be opened.
 *
 * @param [out] phFirHandle    The pointer to a location where the handle to the opened device is written.
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS          If successfully opened the FIR device.
 *
 *
 * @sa          adi_fir_Close().
 */
ADI_FIR_RESULT adi_fir_Open (uint8_t             nDeviceNum,
							 ADI_FIR_DEV_HANDLE *phFirHandle)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
	
#if defined (ADI_DEBUG)
    assert(nDeviceNum <= (FIR_NUM_DEVICES - 1u));
    assert(phFirHandle != NULL);
#endif

    {
		/* Enter Critical Region */
		adi_osal_EnterCriticalRegion();
		/* Lock the Mutex */
		FIR_MUTEX_LOCK(&gFirMutex);

		/* Get the pointer to device */
		pDevice = &gFirDeviceInfo[nDeviceNum];

		/* Reset FIR_CTL1 and FIR_CHNPTR Registers */
		ResetAccelerator(pDevice);

		/* Install the FIR interrupt handler */
		adi_int_InstallHandler(pDevice->nFirDoneIntId, &FirInterruptHandler, pDevice, true);

	#if ((ADI_FIR_CFG_SEC_INT == 1))
		/* Configure the FIR Interrupt as Edge Sensitive at SEC */
		adi_sec_EnableEdgeSense(ADI_IID_TO_SEC_INT(pDevice->nFirDoneIntId),true);
	#endif /* ((ADI_FIR_CFG_SEC_INT == 1)) */

	#if ((ADI_FIR_CFG_SEC_INT == 0))
		/* Configure the FIR Direct Interrupt as Edge Sensitive */
		xthal_interrupt_sens_set(ADI_IID_TO_CORE_INT(pDevice->nFirDoneIntId), 0);
	#endif /* ((ADI_FIR_CFG_SEC_INT == 0)) */
	
		/* Initialize the Device State */
		pDevice->eDevState = ADI_FIR_DEV_STATE_IDLE;

		/* Write the Handle to the location given by the application */
		*phFirHandle = pDevice;

		/* Unlock the Mutex */
		FIR_MUTEX_UNLOCK(&gFirMutex);
		/* Exit Critical Region */
		adi_osal_ExitCriticalRegion();
    }
    /* Return the status back to the application */
    return (eResult);
}

/*!
 * @brief       adi_fir_Close API closes a given FIR device.
 *
 * @details     Closes the given FIR device and resets the FIR Queue.
 *
 * @param [in]  hFirHandle         FIR Device handle
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully closed the FIR device.
 *
 *  - #ADI_FIR_RESULT_FAILED                  If unable to successfully close the device.
 *
 * @sa          adi_fir_Open().
 */
ADI_FIR_RESULT adi_fir_Close (ADI_FIR_DEV_HANDLE hFirHandle)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    FIR_MUTEX_LOCK(&gFirMutex);

    /* Get the device handle */
    pDevice = (ADI_FIR_DEV_INFO*)hFirHandle;

    /* Reset FIR_CTL1 and FIR_CHNPTR Registers */
    ResetAccelerator(pDevice);

    /* Uninitialize the Device State and Queue Info*/
    pDevice->eDevState = ADI_FIR_DEV_STATE_NOTINIT;
    pDevice->pQueue->pActiveTask = NULL;
    pDevice->pQueue->pLastTaskAdded = NULL;

    /* Uninstall FIR interrupt handler */
    if( adi_int_UninstallHandler (pDevice->nFirDoneIntId) != ADI_INT_SUCCESS)
    {
        eResult = ADI_FIR_RESULT_FAILED;
    }

    /* Unlock the Mutex */
    FIR_MUTEX_UNLOCK(&gFirMutex);
    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the status */
    return (eResult);
}


/*!
 * @brief       adi_fir_CreateTask API creates a Task and the related TCB chain from the supplied Channel Information.
 *
 * @details     This API will
 *              - Create a FIR task for given FIR device,
 *              - Create a TCB chain from given Channel list
 *
 * @param [in]  hFirHandle        The FIR device handle
 *
 * @param [in]  pChannelList      Array of Channel information using which TCB chain will be created
 *
 * @param [in]  nNumChannels      Number of Channels(TCBs) in this task
 *
 * @param [in]  pMemory           Memory to store the TCB chain.
 *
 * @param [in]  nMemSize          Size of the memory provided (calculated on the basis of number of channels)\n
 *                                Memory Required = SIZE_OF_TASK_INFO + (NumberOfChannels*SIZE_OF_TCB_INFO)\n
 *                                = 28 + NumberOfChannels*52 Bytes(Legacy Mode)\n
 *                                = 24 + NumberOfChannels*64 Bytes(Auto Configuration Mode)\n
 *                                Please use macro FIR_MEM_SIZE(NO_OF_CHANNELS) which takes Number of Channels as argument for assigning memory to FIR tasks.
 *
 * @param [out]  phFirTask         Address to store the FIR task handle
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully created a Task for the FIR device.
 *
 * @sa          adi_fir_QueueTask()
 */
ADI_FIR_RESULT adi_fir_CreateTask (ADI_FIR_DEV_HANDLE     hFirHandle,
                                   ADI_FIR_CHANNEL_INFO  *pChannelList,
                                   uint32_t               nNumChannels,
                                   void                  *pMemory,
                                   uint32_t               nMemSize,
                                   ADI_FIR_TASK_HANDLE   *phFirTask)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
    ADI_FIR_TASK_INFO *pTask;
    uint32_t FirCtl2;
    ADI_FIR_TCB_INFO * pCurrentTCB;
	ADI_FIR_TCB_INFO * pTempTCB;
    uint32_t count;
    uint32_t count1 = 0u;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);

    /* Check whether pointers passed are not NULL */
    assert((pChannelList != NULL) &&  (pMemory != NULL) && (phFirTask != NULL));

    /* Check if the memory supplied is sufficient */
    assert(nMemSize >=(ADI_FIR_TASK_INFO_SIZE + (nNumChannels * ADI_FIR_TCB_INFO_SIZE)));

    /* Check if nNumChannels is less than Max. No of Channels */
    assert(nNumChannels <= FIR_MAX_NUM_CHANNELS);

    /* Check if the No of Taps for each channel is less than Max. No of Taps*/
    for(count1 = 0u; count1< nNumChannels; count1++)
    {
        assert((pChannelList[count1].nTapLength) <= FIR_MAX_NUM_TAPS_PER_CHANNEL);
    }
#endif /* ADI_DEBUG */
    /* Get the device handle */
    pDevice = (ADI_FIR_DEV_INFO*)hFirHandle;

    /* Make the Task Head as the pMemory Supplied*/
    pTask = (ADI_FIR_TASK_INFO*)pMemory;

    /* Write the Task parameters */
    pTask->eCoreId = adi_core_id();
    pTask->pDevice = (ADI_FIR_DEV_HANDLE)pDevice;
    pTask->nFIRCTL1 =((nNumChannels - 1u)<<BITP_FIR_CTL1_CH) | (FIR_DEF_FIRCTL1);
    pTask->nNumChannels = nNumChannels;
    pTask->pNext = NULL;
    pTask->pTCBChain = (ADI_FIR_TCB_INFO *)((ADI_FIR_TASK_INFO *)pMemory + 1);

    /*Create a TCB Chain */
    pCurrentTCB = pTask->pTCBChain;
    for(count = 0u; count< nNumChannels; count++)
    {
    	/*Create FIRCTL2 register */
    	FirCtl2 = (pChannelList[count].nTapLength - 1u) | ((pChannelList[count].nWindowSize -1u)<<BITP_FIR_CTL2_WINDOW);
    	if(pChannelList[count].eSampling != ADI_FIR_SAMPLING_SINGLE_RATE)
    	{
    		FirCtl2 |= BITM_FIR_CTL2_SRCEN;
    		FirCtl2 |= (pChannelList[count].eSampling == ADI_FIR_SAMPLING_INTERPOLATION) ? (((pChannelList[count].nSamplingRatio) -1u)<<BITP_FIR_CTL2_RATIO) | BITM_FIR_CTL2_UPSAMP : (((pChannelList[count].nSamplingRatio) -1u)<<BITP_FIR_CTL2_RATIO);
    	}
    	else
    	{
    		/* To avoid MISRA Warning */
    	}

    	/* Write to the nFirControl2 field */
    	pCurrentTCB->nFirControl2 = FirCtl2;

    	/*Copy the Coefficient, Input and Output Parameters */
        pCurrentTCB->nCoeffCount    = (uint32_t)pChannelList[count].nCoefficientCount;
        pCurrentTCB->nCoeffModify   = (int32_t)pChannelList[count].nCoefficientModify;
        pCurrentTCB->pCoeffIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pCoefficientIndex);

        pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffBase);
        pCurrentTCB->nOutputCount   = (uint32_t)pChannelList[count].nOutputBuffCount;
        pCurrentTCB->nOutputModify  = (int32_t)pChannelList[count].nOutputBuffModify;
        pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffIndex);

        pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffBase);
        pCurrentTCB->nInputCount    = (uint32_t)pChannelList[count].nInputBuffCount;
        pCurrentTCB->nInputModify   = (int32_t)pChannelList[count].nInputBuffModify;
        pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffIndex);

        /* Update Next Pointer of TCB */
        if(count == (nNumChannels - 1u))
        {
        	pCurrentTCB->pNextTCB = NULL;
        }
        else
        {
		      pTempTCB = pCurrentTCB + 1;
        	  pCurrentTCB->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTempTCB->nFirControl2));
        }

        pCurrentTCB = pCurrentTCB + 1;

    }

    /* Mark the Task State as None */
    pTask->eTaskState = ADI_FIR_TASK_STATE_NONE;

    /* Write the Task Handle to the location supplied */
    *phFirTask = pTask;

    /* Return the Status */
    return (eResult);
}

/*!
 * @brief       adi_fir_QueueTask API queues the given Task to FIR Task Queue if the FIR accelerator is already running. If not, it allocates
 *              the given task to the FIR accelerator.
 *
 * @details     The adi_fir_QueueTask API queues the given task if the accelerator is already running.If not, it allocates
 *              the given task to the accelerator. It asserts to check whether the task is already running or queued or if the device is not opened.
 *              In Legacy Mode, a software queue is maintained for the accelerator device. In Auto Configuration Mode, the API makes use of the
 *              Halt feature to implement hardware queuing.
 *
 * @param [in]  hFirTask   FIR task handle.
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully Queued the given task.
 *
 *  - #ADI_FIR_RESULT_INVALID_OPERATION       If the device is not opened.
 *
 * @sa          adi_fir_CreateTask().
 */
ADI_FIR_RESULT adi_fir_QueueTask (ADI_FIR_TASK_HANDLE hFirTask)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_TASK_INFO *pTask = (ADI_FIR_TASK_INFO*)hFirTask;
	ADI_FIR_DEV_STATE eDevState;
    ADI_FIR_DEV_INFO *pDevice;
#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
#endif /* ADI_USE_DEBUG_DATA */
#if (ADI_CACHE_MANAGEMENT == 1)
   ADI_FIR_TCB_INFO* pTCBChain;
   uint8_t nNumChannels = (uint8_t)0;
   uint8_t Count = (uint8_t)0;
   void *_start = NULL;
   void *_end = NULL;
   void  *temp;
   int32_t nCoeffModify = 0;
#endif /* (ADI_CACHE_MANAGEMENT == 1) */
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
    /* Check if the Task is running or queued */
    assert((pTask->eTaskState != ADI_FIR_TASK_STATE_QUEUED) && (pTask->eTaskState != ADI_FIR_TASK_STATE_RUNNING));
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    FIR_MUTEX_LOCK(&gFirMutex);

    /* Get the Device Handle */
    pDevice = (ADI_FIR_DEV_INFO *)pTask->pDevice;

    /* Get the Device State */
    eDevState = pDevice->eDevState;

#if (ADI_CACHE_MANAGEMENT == 1)
    pTCBChain = pTask->pTCBChain;
    nNumChannels = (uint8_t)pTask->nNumChannels;

    /* Input Buffer-Flush,Output Buffer-Invalidate, Coefficient Buffer-Flush */
    for(Count=0u;Count<nNumChannels;Count++)
    {
       /* Flush Input Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pInputBase));
       _end = (float *)_start + (pTCBChain->nInputCount);
       flush_data_buffer(_start,_end,0);

       /* Invalidate Output Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pOutputBase));
       _end = (float *)_start + (pTCBChain->nOutputCount);
       flush_data_buffer(_start,_end,1);

       /* Flush Coefficient Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pCoeffIndex));
       nCoeffModify = pTCBChain->nCoeffModify;
       if(nCoeffModify < 0)
       {
           _end = (float *)_start - (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffCount));
       }
       else
       {
           _end = (float *)_start + (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffCount));
       }
       flush_data_buffer(_start,_end,0);

       if(Count!= (nNumChannels-1u))
       {
		   /* Update the TCB Pointer to point to the next TCB */
		   temp = pTCBChain->pNextTCB;
		   pTCBChain = (void *)((uint32_t *)RevertTranslatedAddr((uint32_t)(temp)) - ((ADI_FIR_TCB_INFO_SIZE/4u)-1u));
       }

    }

    /* Flush the TCB Chain */
    _start = pTask->pTCBChain;
    _end = ((ADI_FIR_TCB_INFO *)_start) + nNumChannels;
    flush_data_buffer(_start,_end,0);

#endif /* (ADI_CACHE_MANAGEMENT == 1) */
    switch(eDevState)
    {
		case ADI_FIR_DEV_STATE_IDLE :
			/* Update Queue */
			pDevice->pQueue->pActiveTask = pTask;
			pDevice->pQueue->pLastTaskAdded = pTask;
			/*Update Task State */
			pTask->eTaskState = ADI_FIR_TASK_STATE_QUEUED;
			/* Start Accelerator */
			StartQueue(pDevice);
			/*Update Device State */
			pDevice->eDevState = ADI_FIR_DEV_STATE_INUSE;
			/* Update Task State */
			pTask->eTaskState = ADI_FIR_TASK_STATE_RUNNING;
			break;

		case ADI_FIR_DEV_STATE_INUSE :
/*-------------------FOR_DEBUG------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
		eCoreID = adi_core_id();
		if(eCoreID == ADI_CORE_SHARC0)
		{
			pDevice->pQueue->pFirDebugData->QueueTracker[pDevice->pQueue->pFirDebugData->QueueTrackerIndex++] = (int)1;
		}
#if __NUM_SHARC_CORES__ > 1
	else if(eCoreID == ADI_CORE_SHARC1)
		{
		pDevice->pQueue->pFirDebugData->QueueTracker[pDevice->pQueue->pFirDebugData->QueueTrackerIndex++] = (int)2;
		}
#endif /*  __NUM_SHARC_CORES__ */
	else
	    {
            /* MISRA Compliance */
	    }
#endif /* ADI_USE_DEBUG_DATA */
/*-------------------FOR_DEBUG------------------------------*/
			/* Update Queue */
			pDevice->pQueue->pLastTaskAdded->pNext = pTask;
			pDevice->pQueue->pLastTaskAdded = pTask;
			/*Update Task State */
			pTask->eTaskState = ADI_FIR_TASK_STATE_QUEUED;
			break;

		case ADI_FIR_DEV_STATE_NOTINIT:
			eResult = ADI_FIR_RESULT_INVALID_OPERATION;
			break;

		default :
			break;
    }

    /* Unlock the Mutex */
    FIR_MUTEX_UNLOCK(&gFirMutex);
    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the result */
    return (eResult);
}

/*!
 * @brief       adi_fir_DeleteTask API deletes the given Task.
 *
 * @details     When a task is created, Driver keeps its information intact even after FIR accelerator has completed it so that application can reschedule it by just queuing it again.
 *              adi_fir_DeleteTask API deletes the given Task and application will not be able to reuse it afterwards.
 *              Also if the task is in queue and its state is not running, delete task removes the task from the accelerator task queue.
 *              If the task state is running, ADI_FIR_RESULT_TASK_STATE_RUNNING will be returned and the task will not be aborted.
 *              This API is only available in Legacy Mode of Operation.
 *
 * @param [in]  hFirTask   FIR task handle.
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully deleted the given Task.
 *
 *  - #ADI_FIR_RESULT_TASK_STATE_RUNNING      If the Task State is Running.
 *
 * @sa          adi_fir_QueueTask()
 */
ADI_FIR_RESULT adi_fir_DeleteTask (ADI_FIR_TASK_HANDLE hFirTask)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_TASK_INFO *pTask = (ADI_FIR_TASK_INFO*)hFirTask;
    ADI_FIR_TASK_INFO *pPrevTask;
	ADI_FIR_TASK_STATE eTaskState;
	ADI_FIR_DEV_INFO * pDevice;

#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    FIR_MUTEX_LOCK(&gFirMutex);

    /* Get the Task Handle */
    pTask = (ADI_FIR_TASK_INFO*) hFirTask;

    /* Get the Task Status */
    eTaskState = pTask->eTaskState;

    if(pTask->eTaskState == ADI_FIR_TASK_STATE_QUEUED)
    {

        pDevice = (ADI_FIR_DEV_INFO *)pTask->pDevice;
			
		/* Find the previous Task in the Queue and update Queue */
        pPrevTask = pDevice->pQueue->pActiveTask;
        while(pPrevTask->pNext != pTask)
        {
        	pPrevTask = pPrevTask->pNext;
        }

        pPrevTask->pNext = pTask->pNext;

        /* Reset the Task Structure of the Task to be deleted and Update Task Status */
        ResetTaskStructure(pTask);

    }
    else if(pTask->eTaskState == ADI_FIR_TASK_STATE_RUNNING)
    {
        /* Write the Result */
       	eResult = ADI_FIR_RESULT_TASK_STATE_RUNNING;
    }
    else
    {
        /* Means eTaskState == ADI_FIR_TASK_STATE_NONE || eTaskState == ADI_FIR_TASK_STATE_COMPLETED */

        /* Reset the Task Structure */
        ResetTaskStructure(pTask);
    }

    /* Unlock the Mutex */
    FIR_MUTEX_UNLOCK(&gFirMutex);
	/* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    return (eResult);
}

/*!
 * @brief       The adi_fir_GetFirTaskStatus API will return the status of given task.
 *
 * @param[in]   hFirTask      FIR task handle
 *
 * @param[out]  TaskStatus   Address to store the task status
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully returned the status of the given Task in the given location.
 *
 */
ADI_FIR_RESULT adi_fir_GetFirTaskStatus (ADI_FIR_TASK_HANDLE hFirTask, ADI_FIR_TASK_STATE *TaskStatus)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;

    ADI_FIR_TASK_INFO* pTask = (ADI_FIR_TASK_INFO*) hFirTask;

#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
    /* Check if the address passed is NULL */
    assert(TaskStatus != NULL);
#endif

    /* Lock the Mutex */
	FIR_MUTEX_LOCK(&gFirMutex);

    /* Get the Task structure */
    pTask = (ADI_FIR_TASK_INFO*) hFirTask;

    /* Write the Task Status to the given location */
    *TaskStatus = pTask->eTaskState;

    /* Unlock the Mutex */
    FIR_MUTEX_UNLOCK(&gFirMutex);

    /* Return status */
    return (eResult);
}

/*!
 * @brief       The adi_fir_RegisterCallback API registers an application defined Callback.
 *
 * @details     In legacy Mode, the callback occurs after completion of each task and the event passed in the callback is ADI_FIR_EVENT_ALL_CHANNEL_DONE.
 *              In Auto Configuration Mode, the callback can be selected for each channel of a task using the bCallbackEnable
 *              parameter of the ADI_FIR_CHANNEL_INFO.In Auto Configuration Mode,ADI_FIR_EVENT_CHANNEL_DONE event is passed in Callback.
 *              In Legacy Mode, the Callback argument passed by the driver to the application(pArg) will be the task handle of the completed task.
 *              In Auto Configuration Mode, the callback argument passed by the driver to the application(pArg) will be the task handle to which the completed channel belongs to.
 *
 *
 * @param[in]   hFirHandle     FIR Device Handle
 *
 * @param[in]   pfCallback     Application Supplied Callback function which is called to Notify Task as Completed.
 *
 * @param[in]   pCBParam       Application Supplied Callback parameter which will be passed back in the Callback Function.
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully registered the application defined Callback function.
 *
 */
ADI_FIR_RESULT adi_fir_RegisterCallback (ADI_FIR_DEV_HANDLE hFirHandle,
                                         ADI_CALLBACK       pfCallback,
		                                 void *             pCBParam)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);
#endif
    /* Get the Device Handle */
	pDevice = (ADI_FIR_DEV_INFO*) hFirHandle;

	/* Register the Callback function and Callback parameter */
        pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX] = pfCallback;
        pDevice->pFIRCBParam[ADI_FIR_MASTER_CALLBACK_INDEX]   = pCBParam;

    /* Return status */
    return (eResult);
}


/*!
 * @brief       The adi_fir_FloatingPointEnable API configures the output Floating Point Rounding Mode for the the given task.
 *
 * @details     The adi_fir_FloatingPointEnable API configures the output Floating Point Rounding Mode for the the given task.
 *              If the Task State is running, the API returns  ADI_FIR_RESULT_INVALID_STATE.This API is only available in Legacy Mode.
 *              In Auto Configuration Mode, Output Rounding Mode is customizable for each channel using the eRoundingMode field of
 *              ADI_FIR_CHANNEL_INFO.
 *
 * @param [in]  hFirTask         FIR task handle.
 *
 * @param [in]  eRoundingMode    Output Rounding Mode.
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully configured the Output Rounding Mode.
 *
 *  - #ADI_FIR_RESULT_INVALID_STATE           If Task State is running.
 *
 * @sa          adi_fir_FixedPointEnable().
 */
ADI_FIR_RESULT adi_fir_FloatingPointEnable (ADI_FIR_TASK_HANDLE         hFirTask,
                                            ADI_FIR_FLOAT_ROUNDING_MODE eRoundingMode)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_TASK_INFO *pTask;
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
#endif
    /* Get the device handle */
	pTask = (ADI_FIR_TASK_INFO*)hFirTask;

	if(pTask->eTaskState == ADI_FIR_TASK_STATE_RUNNING)
	{
		eResult = ADI_FIR_RESULT_INVALID_STATE;
	}
	else
	{
		pTask->nFIRCTL1 |= ((uint32_t)eRoundingMode) << BITP_FIR_CTL1_RND;
	}

	/* Return the status */
	return (eResult);
}

/*!
 * @brief       The adi_fir_FixedPointEnable API configures the Task to use either Unsigned or Signed Integer for Input.
 *
 * @details     The adi_fir_FixedPointEnable API configures the Task to use either Unsigned or Signed Integer for Input.
 *              If the Task State is running, the API returns  ADI_FIR_RESULT_INVALID_STATE.This API is only available in Legacy Mode.
 *              In Auto Configuration Mode, fixed point operations can be enabled/disabled for each channel using the bFixedPoint
 *              field of ADI_FIR_CHANNEL_INFO.
 *
 * @param [in]  hFirTask         FIR task handle.
 *
 * @param [in]  eInputFormat     Input Format.
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully configured the Input format.
 *
 *  - #ADI_FIR_RESULT_INVALID_STATE           If Task State is running.
 *
 * @sa          adi_fir_FloatingPointEnable().
 */
ADI_FIR_RESULT adi_fir_FixedPointEnable (ADI_FIR_TASK_HANDLE        hFirTask,
                                         ADI_FIR_FIXED_INPUT_FORMAT eInputFormat)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
    ADI_FIR_TASK_INFO *pTask;
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
#endif
    /* Get the Task handle */
    pTask = (ADI_FIR_TASK_INFO*)hFirTask;

	if(pTask->eTaskState == ADI_FIR_TASK_STATE_RUNNING)
    {
       	eResult = ADI_FIR_RESULT_INVALID_STATE;
    }
    else
    {
        pTask->nFIRCTL1 |= (((uint32_t)eInputFormat) << BITP_FIR_CTL1_TC) | BITM_FIR_CTL1_FXD;
    }

    /* Return the status */
    return (eResult);
}

/*!
 * @brief       The adi_fir_UpdateTask API updates the Circular Buffer Parameters of the task.
 *
 * @details     The adi_fir_UpdateTask API updates the Circular Buffer Paramters of the task.When a task has to be reused , the Input and Output Circular Buffer Parameters can be updated before Queuing the task again.
 *              Note that in Auto Configuration Mode, this API must be called before reusing the task as the hardware doesn't update the Task Buffer Indexes. In legacy Mode, this API is optional as the Task Buffer Indexes are updated by the
 *              Hardware after its complete
 *              If the Task State is running or Queued, the API returns ADI_FIR_RESULT_INVALID_OPERATION.
 *
 * @param [in]  hFirTask               FIR task handle.
 *
 * @param [in]  pChannelParamsList     Array of Channel Buffer Parameters.
 *
 * @param [in]  nNumChannels           Number of Channels of the task.
 *
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully configured the Input format.
 *
 *  - #ADI_FIR_RESULT_INVALID_OPERATION       If Task State is not ADI_FIR_TASK_STATE_COMPLETED.
 *
 * @sa          adi_fir_FloatingPointEnable().
 */
ADI_FIR_RESULT adi_fir_UpdateTask(ADI_FIR_TASK_HANDLE             hFirTask,
		                          ADI_FIR_CHANNEL_BUFFER_INFO     *pChannelParamsList,
								  uint32_t                        nNumChannels)
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_TASK_INFO *pTask = NULL;
    ADI_FIR_TCB_INFO *pCurrentTCB = NULL;
    uint32_t count = 0u;
#if defined (ADI_DEBUG)
    /* Validate the Task handle*/
    assert(ValidateTaskHandle(hFirTask) == ADI_FIR_RESULT_SUCCESS);
    /* Check whether pointers passed are not NULL */
    assert((pChannelParamsList != NULL));
#endif
    pTask = (ADI_FIR_TASK_INFO*) hFirTask;

    if(((pTask->nNumChannels) != nNumChannels) || ((pTask->eTaskState) != ADI_FIR_TASK_STATE_COMPLETED))
    {
    	eResult = ADI_FIR_RESULT_INVALID_OPERATION;
    }
    else
    {
    	/* Intitalize the TCB pointer */
    	pCurrentTCB = pTask->pTCBChain;

    	/* Update the Circular Buffer Parameters in a loop */
    	for(count = 0u; count< nNumChannels; count++)
    	{
    		pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffBase);
    		pCurrentTCB->nOutputCount   = (uint32_t)pChannelParamsList[count].nOutputBuffCount;
    		pCurrentTCB->nOutputModify  = (int32_t)pChannelParamsList[count].nOutputBuffModify;
    		pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffIndex);

    		pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffBase);
    		pCurrentTCB->nInputCount    = (uint32_t)pChannelParamsList[count].nInputBuffCount;
    		pCurrentTCB->nInputModify   = (int32_t)pChannelParamsList[count].nInputBuffModify;
    		pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffIndex);

    		/* Update the TCB Pointer */
    		pCurrentTCB = pCurrentTCB + 1;
    	}

    }

    /* Return Result */
    return (eResult);

}

#if (defined(__ADSPSHARCFX__))
/*!
 * @brief       The adi_fir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct.
 *
 * @details     The adi_fir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct.
 *
 * @param [in]  hFirHandle         FIR Device handle.
 *
 * @param [in]  bSecInt    True - Non-direct Interrupt; False - Direct Interrupt
 * @return      Status
 *
 *  - #ADI_FIR_RESULT_SUCCESS                 If successfully configured the Output Rounding Mode.
 *
 *  - #ADI_FIR_RESULT_FAILED           If Task State is running.
 *
 * @sa          adi_fir_FixedPointEnable().
 */
ADI_FIR_RESULT adi_fir_ConfigSecInt (ADI_FIR_DEV_HANDLE     hFirHandle,
                                                           bool bSecInt )
{
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_SUCCESS;
    ADI_FIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
     assert(ValidateDeviceHandle(hFirHandle) == ADI_FIR_RESULT_SUCCESS);
#endif
     uint32_t sid, cid, iid;

     ADI_INT_ID_STATUS res;

     /* Get the Device Handle */
 	pDevice = (ADI_FIR_DEV_INFO*) hFirHandle;

     iid = pDevice->nFirDoneIntId;

     if( ADI_IID_IS_CORE_INT(iid))
     {
    	 cid = iid;
    	 res = get_sid(iid, &sid);
     }
	 else
	 {
         sid = iid;
         res = get_cid(iid, &cid);
	 }

     if (res == ADI_INT_ID_SUCCESS)
     {
    	 if( bSecInt == true)
    	 {
    		 /* Install Non-Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(sid, FirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_FIR_RESULT_FAILED;
    		 }

    	 }
    	 else
    	 {
    		 /* Install Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(cid, FirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_FIR_RESULT_FAILED;
    		 }
    	 }
     }
     else
     {
    	 return ADI_FIR_RESULT_FAILED;
     }

     return ADI_FIR_RESULT_SUCCESS;
}
#endif /* defined(__ADSPSHARCFX__) */

/* FirInterrupt Handler */
static void FirInterruptHandler (uint32_t iid, void* pParam)
{
	ADI_FIR_DEV_INFO *pDevice = (ADI_FIR_DEV_INFO*)pParam;
	ADI_FIR_TASK_INFO *pTask;
    uint32_t nDMAStat;
    ADI_FIR_TASK_INFO * pTemp;
    ADI_SEC_RESULT eResult;
/*-------------------FOR_DEBUG--------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
    eCoreID = adi_core_id();
    if(eCoreID == ADI_CORE_SHARC0)
    {
    	pDevice->pQueue->pFirDebugData->FirIntCnt[0]++;
    }
#if __NUM_SHARC_CORES__ > 1
    else if(eCoreID == ADI_CORE_SHARC1)
    {
    	pDevice->pQueue->pFirDebugData->FirIntCnt[1]++;
    }
    else
    {
        /* MISRA Compliance */
    }
#endif /* __NUM_SHARC_CORES__ > 1 */
#endif /* ADI_USE_DEBUG_DATA */
/*------------------FOR_DEBUG----------------------------------*/

    /* Lock the Mutex */
	FIR_MUTEX_LOCK(&gFirMutex);

	/* Get the Device handle */
    pDevice = (ADI_FIR_DEV_INFO*)pParam;

    /* Get the Handle of the Task that is completed */
    pTask = pDevice->pQueue->pActiveTask;

    /* Get DMA Status. Clear All Channels Done Interrupt Status */
    nDMAStat = (pDevice->pFIRRegs->DMASTAT);

    /* Mark the Task as Complete */
    pTask->eTaskState = ADI_FIR_TASK_STATE_COMPLETED;

    /* Copy the Task Handle in TaskCompleted field of Queue Info */
    pDevice->pQueue->pTaskCompleted = pTask;

    /* Reset Accelerator */
    ResetAccelerator(pDevice);

    /* If there are no pending Tasks,Make pActiveTask = pLastTaskAdded = NULL, Device State = Idle */

    if(pTask->pNext == NULL)
    {
        pDevice->pQueue->pActiveTask = NULL;
        pDevice->pQueue->pLastTaskAdded = NULL;
        pDevice->eDevState = ADI_FIR_DEV_STATE_IDLE;
    }
    else
    {
    	/* Update Queue */
    	pDevice->pQueue->pActiveTask = pTask->pNext;

        /* Start Queue */
        StartQueue(pDevice);

    	/* Update Task State */
        pDevice->pQueue->pActiveTask->eTaskState = ADI_FIR_TASK_STATE_RUNNING;
		
        /* Dequeue the Task that is completed */
		pTask->pNext = NULL;
    }

    /* Unlock the Mutex */
    FIR_MUTEX_UNLOCK(&gFirMutex);

    /* Call the Application defined Callback depending on the Driver operating Mode */
	{
		/* Call the Master Callback(Dual Core Mode). In single Core Mode, there is only One Callback.*/
        if(pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX] != NULL)
        {
            pDevice->pfFIRCallback[ADI_FIR_MASTER_CALLBACK_INDEX](pDevice->pFIRCBParam[ADI_FIR_MASTER_CALLBACK_INDEX],(uint32_t)ADI_FIR_EVENT_ALL_CHANNEL_DONE,(void *)pTask);
        }

	}

}

/* Validate the Device Handle */
static ADI_FIR_RESULT ValidateDeviceHandle (ADI_FIR_DEV_HANDLE hDevice)
{
    unsigned int x;
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_INVALID_HANDLE;
    ADI_FIR_DEV_INFO* pDevice = (ADI_FIR_DEV_INFO*)hDevice;

    for (x = 0u; x < FIR_NUM_DEVICES; x++)
    {
        if (pDevice == &gFirDeviceInfo[x])
        {

            eResult = ADI_FIR_RESULT_SUCCESS;

        }
    }

    return (eResult);
}

/* Validate the Task Handle */
static ADI_FIR_RESULT ValidateTaskHandle(ADI_FIR_TASK_HANDLE hTask)
{
    unsigned int x;
	ADI_FIR_TASK_INFO *pTask = (ADI_FIR_TASK_INFO *)hTask;
    ADI_FIR_RESULT eResult = ADI_FIR_RESULT_INVALID_HANDLE;
    for (x = 0u; x < FIR_NUM_DEVICES; x++)
    {
        if (pTask->pDevice == (ADI_FIR_DEV_HANDLE)&gFirDeviceInfo[x])
        {

            eResult = ADI_FIR_RESULT_SUCCESS;

        }
    }
    return (eResult);
}

/* Translates the supplied memory address for DMA operation based on Core ID. */
static uint32_t TranslateAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */
#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_internal_to_system_addr(MemAddr, 1);
#endif

    return (MemAddr >> 2u);
}

/*
 * Reverts the supplied memory address for DMA operation based on Core ID.
 *
 * Parameters
 *  - MemAddr   Memory address to work on
 *
 * Returns
 *  - Translated Memory address for DMA access.
 *
 */
static uint32_t RevertTranslatedAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */

    MemAddr = MemAddr << 2u;

#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_system_to_internal_addr(MemAddr);
#endif

    return MemAddr;
}


/* Starts the Queue Processing by writing to Chain Pointer and Control Register */
static ADI_FIR_RESULT StartQueue (ADI_FIR_DEV_INFO *  pDevice)
{
    /* Schedule Next Task */
	/* Start Accelerator */
	(pDevice->pFIRRegs->CHNPTR)= (uint32_t)TranslateAddr((uint32_t)(&pDevice->pQueue->pActiveTask->pTCBChain->nFirControl2));
	(pDevice->pFIRRegs->CTL1) = pDevice->pQueue->pActiveTask->nFIRCTL1 | FIR_FIRCTL1_EN_ENABLE | FIR_FIRCTL1_DMAEN_ENABLE;
	return ADI_FIR_RESULT_SUCCESS;
}

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_FIR_RESULT ResetAccelerator(ADI_FIR_DEV_INFO *  pDevice)
{

	(pDevice->pFIRRegs->CTL1) = 0u;
	(pDevice->pFIRRegs->CHNPTR)= 0u;

	return ADI_FIR_RESULT_SUCCESS;
}

/* Resets the given Task Structure */
static ADI_FIR_RESULT ResetTaskStructure (ADI_FIR_TASK_INFO *  pTask)
{

	pTask->pDevice = NULL;
	pTask->pNext = NULL;
	pTask->pTCBChain = NULL;
	pTask->nFIRCTL1 = 0u;
    pTask->nNumChannels = 0u;

    if(pTask->eTaskState != ADI_FIR_TASK_STATE_NONE)
    {
    	pTask->eTaskState  = ADI_FIR_TASK_STATE_NONE;
    }

    return ADI_FIR_RESULT_SUCCESS;
}

/*@}*/

#endif /*_ADI_FIR_LEGACY_V3_C_*/
