/*******************************************************************************

Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you
agree to the terms of the associated Analog Devices License Agreement.

*******************************************************************************/

/*!
* @file      adi_iir_acm_v3.c
*
* @brief     IIR Accelerator Driver Auto Configuration Mode source file for ADSP-SC83x processors.
*
* @details
*            This is the primary source file for the IIR Accelerator Auto Configuration Mode implementation for ADSP-SC83x processors.
*/

#ifndef __ADI_IIR_ACM_V3_C__
#define __ADI_IIR_ACM_V3_C__
/*==========  I N C L U D E  ==========*/
#include <stdlib.h>
#include <string.h>
#include <adi_osal.h>
#include <services/int/adi_int.h>
#include <assert.h>
#include <sys/platform.h>
#include <drivers/iir/adi_iir.h>
#include "adi_iir_def_v3.h"
#include <sys/adi_core.h>
#include <services/int/adi_sec.h>

/*==========  L O C A L    F U N C    D E F S  ==========*/

/* Interrupt Handler for the IIR Device */
static void IirInterruptHandler (uint32_t iid, void* pParam);

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_IIR_RESULT ResetAccelerator(ADI_IIR_DEV_INFO *pDevice);

/* Disables and Enables Accelerator */
static ADI_IIR_RESULT DisableandEnableAccelerator(ADI_IIR_DEV_INFO *pDevice);

/* Translates the Given TCB address by adding appropriate offsets */
static uint32_t TranslateAddr (uint32_t  MemAddr);

static uint32_t RevertTranslatedAddr (uint32_t  MemAddr);

/* Validate the given device handle */
static ADI_IIR_RESULT ValidateDeviceHandle (ADI_IIR_DEV_HANDLE hDevice);

/* Validate the given Task Handle */
static ADI_IIR_RESULT ValidateTaskHandle (ADI_IIR_TASK_HANDLE hTask);

/*==========  D E B U G  S T R U C T U R E S  ==========*/

#if defined(ADI_USE_DEBUG_DATA)
/* It is advised not to use debug Data while running infinite loop programs as the indexes for these
 * structures/data may go out of bound and cause unexpected results in the programs
 */

ADI_IIR_DEBUG_DATA gIirDebugData[] =
{
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#if (IIR_NUM_DEVICES > 1u)
	,{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 1u) */
#if (IIR_NUM_DEVICES > 2u)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 2u) */
#if (IIR_NUM_DEVICES > 3u)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 3u) */
#if (IIR_NUM_DEVICES > 4u)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 4u) */
#if (IIR_NUM_DEVICES > 5u)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 5u) */
#if (IIR_NUM_DEVICES > 6u)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 6u) */
#if (IIR_NUM_DEVICES > 7u)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 7u) */
};

#endif /* ADI_USE_DEBUG_DATA */

/*==========  D A T A  ==========*/

static ADI_IIR_DEV_INFO gIirDeviceInfo[] =
{
    {
        ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
    	NULL,                                           /* Pointer to IIR Callback function */
    	NULL,                                           /* Pointer to Callback Data Parameter */
		NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
		NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
		0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR0_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR0_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR0_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
       ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[0]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
    }
#if (IIR_NUM_DEVICES > 1u)
    ,{
        ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
     	NULL,                                           /* Pointer to IIR Callback function */
       	NULL,                                           /* Pointer to Callback Data Parameter */
    	NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
    	NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
    	0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
    	(ADI_IIR_CFG0_TypeDef *)ADI_IIR1_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR1_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR1_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[1]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
    }
#endif /* (IIR_NUM_DEVICES > 1u) */
#if (IIR_NUM_DEVICES > 2u)
    ,
	{
	    ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
	    NULL,                                           /* Pointer to IIR Callback function */
	    NULL,                                           /* Pointer to Callback Data Parameter */
	    NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
	    NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
	    0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
	    (ADI_IIR_CFG0_TypeDef *)ADI_IIR2_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR2_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR2_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[2]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 2u) */
#if (IIR_NUM_DEVICES > 3u)
    ,
	{
	    ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
	    NULL,                                           /* Pointer to IIR Callback function */
	    NULL,                                           /* Pointer to Callback Data Parameter */
	    NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
	    NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
	    0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
	    (ADI_IIR_CFG0_TypeDef *)ADI_IIR3_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR3_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR3_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[3]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 3u) */
#if (IIR_NUM_DEVICES > 4u)
    ,
	{
	    ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
	    NULL,                                           /* Pointer to IIR Callback function */
	    NULL,                                           /* Pointer to Callback Data Parameter */
	    NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
	    NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
	    0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
	    (ADI_IIR_CFG0_TypeDef *)ADI_IIR4_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR4_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR4_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[4]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 4u) */
#if (IIR_NUM_DEVICES > 5u)
    ,
	{
	    ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
	    NULL,                                           /* Pointer to IIR Callback function */
	    NULL,                                           /* Pointer to Callback Data Parameter */
	    NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
	    NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
	    0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
	    (ADI_IIR_CFG0_TypeDef *)ADI_IIR5_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR5_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR5_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[5]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 5u) */
#if (IIR_NUM_DEVICES > 6u)
    ,
	{
	    ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
	    NULL,                                           /* Pointer to IIR Callback function */
	    NULL,                                           /* Pointer to Callback Data Parameter */
	    NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
	    NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
	    0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
	    (ADI_IIR_CFG0_TypeDef *)ADI_IIR6_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR6_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR6_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[6]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 6u) */
#if (IIR_NUM_DEVICES > 7u)
    ,
	{
	    ADI_IIR_DEV_STATE_NOTINIT,                      /* IIR device state */
	    NULL,                                           /* Pointer to IIR Callback function */
	    NULL,                                           /* Pointer to Callback Data Parameter */
	    NULL,                                           /* This pointer points to the end TCB of the Accelerator TCB Chain. */
	    NULL,                                           /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
	    0u,                                              /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
	    (ADI_IIR_CFG0_TypeDef *)ADI_IIR7_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR7_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR7_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[7]        /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 7u) */
};

/*==========  C O D E  ==========*/



ADI_IIR_RESULT adi_iir_Open (uint8_t             nDeviceNum,
							 ADI_IIR_DEV_HANDLE *phIirHandle)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;

#if defined (ADI_DEBUG)
    assert(nDeviceNum <= (IIR_NUM_DEVICES - 1u));
    assert(phIirHandle != NULL);
#endif

    {

		/* Get the pointer to device */
		pDevice = &gIirDeviceInfo[nDeviceNum];

		/* Reset IIR_CTL1 and IIR_CHNPTR Registers */
		ResetAccelerator(pDevice);

		/* Configure the Global Control Register using static configuration */
		(pDevice->pIIRRegs->CTL1) = IIR_DEF_IIRCTL1;

		/* Install the IIR interrupt handler */
		adi_int_InstallHandler(pDevice->nIirDoneIntId, &IirInterruptHandler, pDevice, true);

#if ((ADI_IIR_CFG_SEC_INT == 1))
		/* Configure the IIR Interrupt as Edge Sensitive at SEC */
		adi_sec_EnableEdgeSense(ADI_IID_TO_SEC_INT(pDevice->nIirDoneIntId),true);
#endif /* ((ADI_IIR_CFG_SEC_INT == 1)) */

#if ((ADI_IIR_CFG_SEC_INT == 0))
		/* Configure the IIR Direct Interrupt as Edge Sensitive */
		xthal_interrupt_sens_set(ADI_IID_TO_CORE_INT(pDevice->nIirDoneIntId), 0);
#endif /* ((ADI_IIR_CFG_SEC_INT == 0)) */

		/* Initialize the Device State */
		pDevice->eDevState = ADI_IIR_DEV_STATE_INIT;

		/* Write the Handle to the location given by the application */
		*phIirHandle = pDevice;
    }
    /* Return the status back to the application */
    return (eResult);
}


ADI_IIR_RESULT adi_iir_Close (ADI_IIR_DEV_HANDLE hIirHandle)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Get the device handle */
    pDevice = (ADI_IIR_DEV_INFO*)hIirHandle;

    /* Reset IIR_CTL1 and IIR_CHNPTR Registers */
    ResetAccelerator(pDevice);

    /* Uninitialize the Device State and Queue Info*/
    pDevice->eDevState = ADI_IIR_DEV_STATE_NOTINIT;
    pDevice->pLastTCBinQueue = NULL;
    pDevice->pActiveTaskTCB  = NULL;
    pDevice->nIntCntOfCurrentTask = 0u;
    pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX] = NULL;
    pDevice->pIIRCBParam[ADI_IIR_MASTER_CALLBACK_INDEX] = NULL;

    /* Uninstall IIR interrupt handler */
    if( adi_int_UninstallHandler (pDevice->nIirDoneIntId) != ADI_INT_SUCCESS)
    {
        eResult = ADI_IIR_RESULT_FAILED;
    }

    /* Return the status */
    return (eResult);
}


ADI_IIR_RESULT adi_iir_CreateTask (ADI_IIR_DEV_HANDLE     hIirHandle,
                                   ADI_IIR_CHANNEL_INFO  *pChannelList,
                                   uint32_t               nNumChannels,
                                   void                  *pMemory,
                                   uint32_t               nMemSize,
                                   ADI_IIR_TASK_HANDLE   *phIirTask)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
    ADI_IIR_TCB_INFO *pTCBChain;
    uint32_t IirCtl2;
    uint32_t IirCtl1;
    ADI_IIR_TCB_INFO * pCurrentTCB;
	ADI_IIR_TCB_INFO * pTempTCB;
    uint32_t count;
    uint32_t IntCount;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);

    /* Check whether pointers passed are not NULL */
    assert((pChannelList != NULL) &&  (pMemory != NULL) && (phIirTask != NULL));

    /* Check if the memory supplied is sufficient */
    assert(nMemSize >=(ADI_IIR_TASK_INFO_SIZE + (nNumChannels * ADI_IIR_TCB_INFO_SIZE)));
#endif
    /* Initialize Interrupt Count variable. This variable will be written to the Task Info Structure in the end  */
    IntCount = 0u;
#if defined (ADI_DEBUG)
    /* Check if the No of Biquads for each channel is less than Max. No of Biquads */
    for(count = 0u; count< nNumChannels; count++)
    {
        assert((pChannelList[count].nBiquads) <= IIR_MAX_NUM_BIQUADS_PER_CHANNEL);
    }
#endif
    /* Get the device handle */
    pDevice = (ADI_IIR_DEV_INFO*)hIirHandle;

    /* Make the Task Head as the pMemory Supplied*/
    pTCBChain = (ADI_IIR_TCB_INFO*)pMemory;

    /*Create a TCB Chain */
    pCurrentTCB = pTCBChain;

    for(count = 0u; count< nNumChannels; count++)
    {
    	/*Create IIRCTL2 register */
    	IirCtl2 = (pChannelList[count].nBiquads - 1u) | ((pChannelList[count].nWindowSize -1u)<<BITP_IIR_CTL2_WINDOW);

        if((pChannelList[count].bCallbackEnable == false) && (count != (nNumChannels - 1u)))
        {
        	IirCtl2 |= BITM_IIR_CTL2_IMASK;
        }
        else
        {
 		   /* Update the Number of Tasks that Contain Interrupt Enabled in the Chain.*/
 			IntCount++;
        }
        if(pChannelList[count].bTriggerEnable == false)
        {
        	IirCtl2 |= BITM_IIR_CTL2_TMASK;
        }

        if(pChannelList[count].bTriggerWaitEnable == true)
        {
        	IirCtl2 |= BITM_IIR_CTL2_TWAIT;
        }

    	/* Write to the nIirControl2 field */
    	pCurrentTCB->nIirControl2 = IirCtl2;

    	/*Create Secondary Global Control register */
        IirCtl1 = ((uint32_t)pChannelList[count].eRoundingMode) << BITP_IIR_CTL1_RND;
        if(pChannelList[count].bFortyBitSelect == true)
        {
        	IirCtl1 |= BITM_IIR_CTL1_FORTYBIT;
        }
        if(pChannelList[count].bSaveBiquadState == true)
        {
        	IirCtl1 |= BITM_IIR_CTL1_SS;
        }

        /* Write to the Secondary Global Control Register Field */
        pCurrentTCB->nIirSecondaryControl1 = IirCtl1;

    	/*Copy the Coefficient, Input and Output Parameters */
        pCurrentTCB->nCoeffCount   = (uint32_t)pChannelList[count].nCoefficientLength;
        pCurrentTCB->nCoeffModify   = (int32_t)pChannelList[count].nCoefficientModify;
        pCurrentTCB->pCoeffIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pCoefficientIndex);

        pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffBase);
        pCurrentTCB->nOutputCount  = (uint32_t)pChannelList[count].nOutputBuffLength;
        pCurrentTCB->nOutputModify  = (int32_t)pChannelList[count].nOutputBuffModify;
        pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffIndex);

        pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffBase);
        pCurrentTCB->nInputCount   = (uint32_t)pChannelList[count].nInputBuffLength;
        pCurrentTCB->nInputModify   = (int32_t)pChannelList[count].nInputBuffModify;
        pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffIndex);

        /* Update Next Pointer of TCB and other parameters of the Last TCB */
        if(count == (nNumChannels - 1u))
        {
        	pCurrentTCB->pNextTCB = NULL;

			/* Make the Software Control1 field of the First TCB point task Info Structure */
			pTCBChain->nIirSoftwareControl1 = pCurrentTCB+1;

			/* Write the Interrupt Count filed of the Task Info structure */
			GET_INTERRUPT_COUNT((ADI_IIR_TCB_INFO *)pTCBChain) = IntCount;

			/* Write the Number of Channels field of the Task Info  structure */
			GET_CHANNEL_COUNT((ADI_IIR_TCB_INFO *)pTCBChain) = nNumChannels;

			/* Write to the pLastChannel field of the task info Structure */
			GET_LAST_TCB(pTCBChain) = pCurrentTCB;

			/* Update Task Status in Task Info Structure */
			GET_TASK_STATUS(pTCBChain) = ADI_IIR_TASK_STATE_NONE;

			/* Write the Device Handle in the Task Info Structure */
			GET_DEVICE_HANDLE(pTCBChain) = (void *)hIirHandle;

			/* Update the flag which indicates whether application wants Callback for the Last Channel or no */
			GET_LAST_CHANNEL_CALLBACK_ENABLE(pTCBChain)=  pChannelList[count].bCallbackEnable;

        }
        else
        {
		      pTempTCB = pCurrentTCB + 1;
        	  pCurrentTCB->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTempTCB->nIirControl2));
        }

        pCurrentTCB = pCurrentTCB + 1;

    }

    /* Write the Task Handle to the location supplied */
    *phIirTask = pTCBChain;

    /* Return the Status */
    return (eResult);
}


ADI_IIR_RESULT adi_iir_QueueTask (ADI_IIR_TASK_HANDLE hIirTask)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TCB_INFO *pTask = (ADI_IIR_TCB_INFO*)hIirTask;
	ADI_IIR_DEV_STATE eDevState;
    ADI_IIR_DEV_INFO *pDevice;
    uint32_t nLastChannelSCTL1;
#if (ADI_CACHE_MANAGEMENT == 1)
   ADI_IIR_TCB_INFO* pTCBChain;
   uint32_t nNumChannels = (uint32_t)0;
   uint32_t Count = (uint32_t)0;
   void *_start = NULL;
   void *_end = NULL;
   void  *temp;
   int32_t nCoeffModify = 0;
#endif /* (ADI_CACHE_MANAGEMENT == 1) */
#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
#endif /* ADI_USE_DEBUG_DATA */
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);

    /* Check if the Task is running or queued */
    assert((GET_TASK_STATUS(pTask) != ADI_IIR_TASK_STATE_QUEUED) && (GET_TASK_STATUS(pTask) != ADI_IIR_TASK_STATE_RUNNING));
#endif
    /* Additional Check in ACM Mode */
    pDevice = (ADI_IIR_DEV_INFO *)GET_DEVICE_HANDLE(pTask);
#if defined (ADI_DEBUG)
    assert(pDevice->eDevState == ADI_IIR_DEV_STATE_INIT);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

#if (ADI_CACHE_MANAGEMENT == 1)
    pTCBChain = (ADI_IIR_TCB_INFO *)hIirTask;
    nNumChannels = GET_CHANNEL_COUNT((ADI_IIR_TCB_INFO *)pTCBChain);

    /* Input Buffer-Flush,Output Buffer-Invalidate, (Coefficient Buffer-Flush&Invalidate because the DMA writes back the states) */
    for(Count=0u;Count<nNumChannels;Count++)
    {

       /* Flush Input Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pInputBase));
       _end = (float *)_start + (pTCBChain->nInputCount);
       flush_data_buffer(_start,_end,0);

       /* Invalidate Output Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pOutputBase));
       _end = (float *)_start + (pTCBChain->nOutputCount);
       flush_data_buffer(_start,_end,1);

       /* Flush&Invalidate Coefficient Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pCoeffIndex));
       nCoeffModify = pTCBChain->nCoeffModify;
       if(nCoeffModify < 0)
       {
           _end = (float *)_start - (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffCount));
       }
       else
       {
    	   _end = (float *)_start + (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffCount));
       }
       flush_data_buffer(_start,_end,1);

       if(Count!= (nNumChannels-1u))
       {
		   /* Update the TCB Pointer to point to the next TCB */
		   temp = pTCBChain->pNextTCB;
		   pTCBChain = (void *)((uint32_t *)RevertTranslatedAddr((uint32_t)(temp)) - ((ADI_IIR_TCB_INFO_SIZE/4u)-1u));
       }

    }

    /* Flush the TCB Chain */
    _start = (ADI_IIR_TCB_INFO *)hIirTask;
    _end = ((ADI_IIR_TCB_INFO *)_start) + nNumChannels;
    flush_data_buffer(_start,_end,0);

#endif /* (ADI_CACHE_MANAGEMENT == 1) */

    /* Halt Accelerator */
    (pDevice->pIIRRegs->CTL1) |= BITM_IIR_CTL1_HALT;

    /* Check if the Accelerator is Idle or Running */
    if(((pDevice->pIIRRegs->DMASTAT) & BITM_IIR_DMA_OVERALL_STATUS) != 0u) 
    {
/*-------------------FOR_DEBUG------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
		eCoreID = adi_core_id();
		if(eCoreID == ADI_CORE_SHARC0)
		{
			pDevice->pIirDebugData->QueueTracker[pDevice->pIirDebugData->QueueTrackerIndex++] = 1;
		}
#if __NUM_SHARC_CORES__ > 1
	else if(eCoreID == ADI_CORE_SHARC1)
		{
			pDevice->pIirDebugData->QueueTracker[pDevice->pIirDebugData->QueueTrackerIndex++] = 2;
		}
	    else
		{
            /* For MISRA Compliance */
		}
#endif /*  __NUM_SHARC_CORES__ */
#endif /* ADI_USE_DEBUG_DATA */
/*-------------------FOR_DEBUG------------------------------*/
        /* Running */
#if defined (ADI_DEBUG)
    	assert(pDevice->pLastTCBinQueue != NULL);
#endif
    	/* Update Queue */
    	pDevice->pLastTCBinQueue->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTask->nIirControl2));
    	/* If the previous TCB is in L2, the above update of linking the previous TCB to the Current TCB has to flushed */
    	flush_data_buffer((&(pDevice->pLastTCBinQueue->pNextTCB)),(&(pDevice->pLastTCBinQueue->pNextTCB)) + 1,0);
    	pDevice->pLastTCBinQueue = (ADI_IIR_TCB_INFO *)GET_LAST_TCB(pTask);

    	/* Check if the CP is NULL. If yes, write to CP. If accelerator is processing the last channel of the Task. */
    	if((pDevice->pIIRRegs->CHNPTR) == (uint32_t)NULL)
    	{
    	    (pDevice->pIIRRegs->CHNPTR) = (uint32_t)TranslateAddr((uint32_t)(&pTask->nIirControl2));
    	}

    	/* Update the Task status */
    	GET_TASK_STATUS(pTask) = ADI_IIR_TASK_STATE_QUEUED;
    }
    else
    {
        /* Idle */

    	/* Check if all the interrupts have occurred. If not update the Queue as interrupts are still pending */
    	if(pDevice->pLastTCBinQueue != NULL)
    	{
    		/* Update Queue */
        	pDevice->pLastTCBinQueue->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTask->nIirControl2));
        	/* If the previous TCB is in L2, the above update of linking the previous TCB to the Current TCB has to flushed */
        	flush_data_buffer((&(pDevice->pLastTCBinQueue->pNextTCB)),(&(pDevice->pLastTCBinQueue->pNextTCB)) + 1,0);
        	pDevice->pLastTCBinQueue = (ADI_IIR_TCB_INFO *)GET_LAST_TCB(pTask);
        	/* Update the Task status */
        	GET_TASK_STATUS(pTask) = ADI_IIR_TASK_STATE_QUEUED;
    	}
    	else
    	{
    		/* Update Queue */
    		pDevice->pLastTCBinQueue = (ADI_IIR_TCB_INFO *)GET_LAST_TCB(pTask);
    		pDevice->pActiveTaskTCB = pTask;

        	/* Update the Task status */
    		GET_TASK_STATUS(pTask) = ADI_IIR_TASK_STATE_RUNNING;

        	/* Load the Interrupt Count of the Current Task */
        	pDevice->nIntCntOfCurrentTask = GET_INTERRUPT_COUNT(pTask);
    	}
    	/* Write to CP, Disable and enable Accelerator */
	    (pDevice->pIIRRegs->CHNPTR) = (uint32_t)TranslateAddr((uint32_t)(&pTask->nIirControl2));
	    DisableandEnableAccelerator(pDevice);

    }

    /* Resume Accelerator */
    (pDevice->pIIRRegs->CTL1) &= ~BITM_IIR_CTL1_HALT;

    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the result */
    return (eResult);
}


ADI_IIR_RESULT adi_iir_GetIirTaskStatus (ADI_IIR_TASK_HANDLE hIirTask, ADI_IIR_TASK_STATE *TaskStatus)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TCB_INFO *pTask;
    uint32_t nLastChannelSCTL1;
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
    /* Check if the address passed is NULL */
    assert(TaskStatus != NULL);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Get the Task structure */
    pTask = (ADI_IIR_TCB_INFO*) hIirTask;

    /* Write the Task Status to the given location */
    *TaskStatus = (ADI_IIR_TASK_STATE)GET_TASK_STATUS(pTask);

    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return status */
    return (eResult);
}


ADI_IIR_RESULT adi_iir_RegisterCallback (ADI_IIR_DEV_HANDLE hIirHandle,
                                         ADI_CALLBACK       pfCallback,
		                                 void *             pCBParam)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Get the Device Handle */
	pDevice = (ADI_IIR_DEV_INFO*) hIirHandle;

	/* Register the Callback function and Callback parameter */
        pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX] = pfCallback;
        pDevice->pIIRCBParam[ADI_IIR_MASTER_CALLBACK_INDEX]   = pCBParam;

    /* Return status */
    return (eResult);
}


ADI_IIR_RESULT adi_iir_UpdateTask(ADI_IIR_TASK_HANDLE hIirTask,
		                          ADI_IIR_CHANNEL_BUFFER_INFO  *pChannelParamsList,
								  uint32_t               nNumChannels)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TCB_INFO *pTask = (ADI_IIR_TCB_INFO*)hIirTask;
    ADI_IIR_TCB_INFO * pCurrentTCB = NULL;
    uint32_t count = 0u;
#if defined (ADI_DEBUG)
    /* Validate the Task handle*/
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
    /* Check whether pointers passed are not NULL */
    assert((pChannelParamsList != NULL));
#endif
    if( (GET_CHANNEL_COUNT((ADI_IIR_TCB_INFO *)pTask) != nNumChannels) || (GET_TASK_STATUS((ADI_IIR_TCB_INFO *)pTask) != ADI_IIR_TASK_STATE_COMPLETED) )
    {
    	eResult = ADI_IIR_RESULT_INVALID_OPERATION;
    }
    else
    {
    	pCurrentTCB = pTask;

    	/* Update the Circular Buffer Parameters in a loop */
    	for(count = 0u; count< nNumChannels; count++)
    	{
    		pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffBase);
    		pCurrentTCB->nOutputCount   = (uint32_t)pChannelParamsList[count].nOutputBuffLength;
    		pCurrentTCB->nOutputModify  = (int32_t)pChannelParamsList[count].nOutputBuffModify;
    		pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffIndex);

    		pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffBase);
    		pCurrentTCB->nInputCount    = (uint32_t)pChannelParamsList[count].nInputBuffLength;
    		pCurrentTCB->nInputModify   = (int32_t)pChannelParamsList[count].nInputBuffModify;
    		pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffIndex);

    		pCurrentTCB = pCurrentTCB + 1;
    	}

    }

    /* Return Result */
    return (eResult);

}

#if (defined(__ADSPSHARCFX__))
/*!
 * @brief       The adi_iir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct.
 *
 * @details     The adi_iir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct..
 *
 * @param [in]  hIirHandle         IIR Device handle.
 *
 * @param [in]  bSecInt    True - Non-direct Interrupt; False - Direct Interrupt
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully configured the Output Rounding Mode.
 *
 *  - #ADI_IIR_RESULT_FAILED           If Task State is running.
 *
 * @sa          adi_iir_FixedPointEnable().
 */
ADI_IIR_RESULT adi_iir_ConfigSecInt (ADI_IIR_DEV_HANDLE     hIirHandle,
                                                           bool bSecInt )
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
     assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);
#endif
     uint32_t sid, cid, iid;

     ADI_INT_ID_STATUS res;

     /* Get the Device Handle */
 	pDevice = (ADI_IIR_DEV_INFO*) hIirHandle;

     iid = pDevice->nIirDoneIntId;

     if( ADI_IID_IS_CORE_INT(iid))
     {
    	 cid = iid;
    	 res = get_sid(iid, &sid);
     }
	 else
	 {
         sid = iid;
         res = get_cid(iid, &cid);
	 }

     if (res == ADI_INT_ID_SUCCESS)
     {
    	 if( bSecInt == true)
    	 {
    		 /* Install Non-Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(sid, IirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_IIR_RESULT_FAILED;
    		 }

    	 }
    	 else
    	 {
    		 /* Install Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(cid, IirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_IIR_RESULT_FAILED;
    		 }
    	 }
     }
     else
     {
    	 return ADI_IIR_RESULT_FAILED;
     }

     return ADI_IIR_RESULT_SUCCESS;
}
#endif /* defined(__ADSPSHARCFX__) */

/* IirInterrupt Handler */
static void IirInterruptHandler (uint32_t iid, void* pParam)
{
	ADI_IIR_DEV_INFO *pDevice = (ADI_IIR_DEV_INFO*)pParam;
	ADI_IIR_TCB_INFO *pTask;
    uint32_t nDMAStat;

/*-------------------FOR_DEBUG--------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
	ADI_CORE_ID eCoreID;
	eCoreID = adi_core_id();
	if(eCoreID == ADI_CORE_SHARC0)
	{
		pDevice->pIirDebugData->IirIntCnt[0]++;
	}
#if __NUM_SHARC_CORES__ > 1
	else if(eCoreID == ADI_CORE_SHARC1)
	{
		pDevice->pIirDebugData->IirIntCnt[1]++;
	}
	else
	{
        /* For MISRA Compliance */
	}
#endif /* __NUM_SHARC_CORES__ > 1 */
#endif /* ADI_USE_DEBUG_DATA */
/*------------------FOR_DEBUG----------------------------------*/


    /* Get the Handle of the Task that is completed */
    pTask = pDevice->pActiveTaskTCB;

    /* Get DMA Status. Clear All Channels Done Interrupt Status */
    nDMAStat = (pDevice->pIIRRegs->DMASTAT);

    /* Decrement Count */
    (pDevice->nIntCntOfCurrentTask--);

    /* Check if the Task is Complete (Count == 0) */
     if(pDevice->nIntCntOfCurrentTask == 0u)
     {
     	/* Update Status of the Completed Task */
    	GET_TASK_STATUS(pTask) = ADI_IIR_TASK_STATE_COMPLETED;

     	/* Check if there are more tasks in Queue */
         if(GET_LAST_TCB(pTask) == pDevice->pLastTCBinQueue)
         {
         	/* Reset Queue Pointers */
             pDevice->pActiveTaskTCB = NULL;
             pDevice->pLastTCBinQueue = NULL;

         }
         else
         {
 			/* Update pActiveTaskTCB of Device Info */
 			void  *temp = (((ADI_IIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB);
 			pDevice->pActiveTaskTCB = (ADI_IIR_TCB_INFO *)(((uint32_t *)RevertTranslatedAddr((uint32_t)temp)) - ((ADI_IIR_TCB_INFO_SIZE/4u)-1u));

     	    /* Update Task Status of the next Task */
     	    GET_TASK_STATUS(pDevice->pActiveTaskTCB) = ADI_IIR_TASK_STATE_RUNNING;

         	/* Update Interrupt Count in Device Info */
         	pDevice->nIntCntOfCurrentTask = GET_INTERRUPT_COUNT(pDevice->pActiveTaskTCB);

     	    /* Dequeue the Completed Task */
     	    ((ADI_IIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB = NULL;
            /* If the TCB chain which is completed is in L2, then the above update of dequeing the completed TCB chain must be flushed */
     	   flush_data_buffer(&(((ADI_IIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB),(&(((ADI_IIR_TCB_INFO *)GET_LAST_TCB(pTask))->pNextTCB)) + 1,0);
         }
     	/*Check whether the Last Channel Callback is needed by the application or not */
     	if(GET_LAST_CHANNEL_CALLBACK_ENABLE(pTask) == true)
     	{
             /* Call Callback */
         	if(pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX] != NULL)
         	{
         	    pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX](pDevice->pIIRCBParam[ADI_IIR_MASTER_CALLBACK_INDEX],(uint32_t)ADI_IIR_EVENT_CHANNEL_DONE,(void *)pTask);
         	}
     	}
     }
     else
     {
         /* Call Callback */
     	if(pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX] != NULL)
     	{
     	    pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX](pDevice->pIIRCBParam[ADI_IIR_MASTER_CALLBACK_INDEX],(uint32_t)ADI_IIR_EVENT_CHANNEL_DONE,(void *)pTask);
     	}

     }

}


/* Validate the Device Handle */
static ADI_IIR_RESULT ValidateDeviceHandle (ADI_IIR_DEV_HANDLE hDevice)
{
    unsigned int x;
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_INVALID_HANDLE;
    ADI_IIR_DEV_INFO* pDevice = (ADI_IIR_DEV_INFO*)hDevice;

    for (x = 0u; x < IIR_NUM_DEVICES; x++)
    {
        if (pDevice == &gIirDeviceInfo[x])
        {

            eResult = ADI_IIR_RESULT_SUCCESS;

        }
    }

    return (eResult);
}

/* Validate the Task Handle */
static ADI_IIR_RESULT ValidateTaskHandle(ADI_IIR_TASK_HANDLE hTask)
{
    unsigned int x;
    ADI_IIR_TCB_INFO *pTask = (ADI_IIR_TCB_INFO *)hTask;
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_INVALID_HANDLE;
    for (x = 0u; x < IIR_NUM_DEVICES; x++)
    {
        if (GET_DEVICE_HANDLE(pTask) == ((ADI_IIR_DEV_HANDLE)&gIirDeviceInfo[x]))
        {
            eResult = ADI_IIR_RESULT_SUCCESS;
        }
    }
    return (eResult);
}


/* Translates the supplied memory address for DMA operation based on Core ID. */
static uint32_t TranslateAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */
#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_internal_to_system_addr(MemAddr, 1);
#endif

    return (MemAddr >> 2u);
}

/*
 * Reverts the supplied memory address for DMA operation based on Core ID.
 *
 * Parameters
 *  - MemAddr   Memory address to work on
 *
 * Returns
 *  - Translated Memory address for DMA access.
 *
 */
static uint32_t RevertTranslatedAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */

    MemAddr = MemAddr << 2u;

#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_system_to_internal_addr(MemAddr);
#endif

    return MemAddr;
}

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_IIR_RESULT ResetAccelerator(ADI_IIR_DEV_INFO *pDevice)
{
    (pDevice->pIIRRegs->CTL1) = 0u;
	(pDevice->pIIRRegs->CHNPTR) = 0u;
	
	return ADI_IIR_RESULT_SUCCESS;
}

static ADI_IIR_RESULT DisableandEnableAccelerator(ADI_IIR_DEV_INFO *pDevice)
{
    (pDevice->pIIRRegs->CTL1) &= ~(BITM_IIR_CTL1_EN | BITM_IIR_CTL1_DMAEN); /* can be optimized( dmaen clear/set may not be required) */
    (pDevice->pIIRRegs->CTL1) |= (BITM_IIR_CTL1_EN | BITM_IIR_CTL1_DMAEN);

	return ADI_IIR_RESULT_SUCCESS;
}

#endif /*__ADI_IIR_ACM_V3_C__*/


