/*!
 *****************************************************************************
 * @file:    adi_iir_def_acm_v3.h
 * @brief:   IIR Device Driver definition for ACM Mode
 * @version: $Revision: 66664 $
 * @date:    $Date: 2021-08-02 09:50:29 -0400 (Mon, 02 Aug 2021) $
 *****************************************************************************
Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.
This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_iir_def_acm_v3.h
     Description  :   Static function and structure definition needed by the device driver
                      for managing the IIR  device for ADSP-SC83x processors.
================================================================================ */

#ifndef __ADI_IIR_DEF_ACM_V3_H__
#define __ADI_IIR_DEF_ACM_V3_H__




/*==========  D E F I N E S  ==========*/


#define IIR_MAX_NUM_BIQUADS_PER_CHANNEL   (64u)

#define ADI_IIR_NUM_OF_CALLBACKS       (1u)
#define ADI_IIR_NUM_OF_CALLBACK_PARAMS (1u)
#define ADI_IIR_MASTER_CALLBACK_INDEX  (0u)

#define ADI_IIR_CFG_BURSTEN        (1ul << BITP_IIR_CTL1_BURSTEN)

#define ADI_IIR_CFG_SMQ_LIUPS_EN    (1ul << BITP_IIR_CTL1_SMQ_LIUPS_EN)

/* Default IIRCTL1 Configuration */
#define IIR_DEF_IIRCTL1     (  (((uint32_t)ADI_IIR_CFG_ACCELERATOR_MODE)  << BITP_IIR_CTL1_ACM)      \
							 | (((uint32_t)ADI_IIR_CFG_BURSTEN)) | (((uint32_t)ADI_IIR_CFG_SMQ_LIUPS_EN)) )

/* IIR_CTL1 Register macros */
#define IIR_IIRCTL1_EN_ENABLE                     (BITM_IIR_CTL1_EN)
#define IIR_IIRCTL1_DMAEN_ENABLE                  (BITM_IIR_CTL1_DMAEN)

/* Additional Bit Masks,Bit Positions required in ACM Mode for Software Control Registers */
#define BITM_IIR_DMA_OVERALL_STATUS (BITM_IIR_DMASTAT_CPLD | BITM_IIR_DMASTAT_CDKLD | BITM_IIR_DMASTAT_PPGS | BITM_IIR_DMASTAT_WRBK | BITM_IIR_DMASTAT_SVDK)
#define GET_TASK_STATUS(x) ((((ADI_IIR_TASK_INFO *)(((ADI_IIR_TCB_INFO *)(x))->nIirSoftwareControl1))->eTaskState))
#define GET_INTERRUPT_COUNT(x) ((((ADI_IIR_TASK_INFO *)(((ADI_IIR_TCB_INFO *)(x))->nIirSoftwareControl1))->nInterruptCount))
#define GET_CHANNEL_COUNT(x) ((((ADI_IIR_TASK_INFO *)(((ADI_IIR_TCB_INFO *)(x))->nIirSoftwareControl1))->nNumChannels))
#define GET_LAST_TCB(x) ((((ADI_IIR_TASK_INFO *)(((ADI_IIR_TCB_INFO *)(x))->nIirSoftwareControl1))->pLastTCB))
#define GET_DEVICE_HANDLE(x) ((((ADI_IIR_TASK_INFO *)(((ADI_IIR_TCB_INFO *)(x))->nIirSoftwareControl1))->pDevice))
#define GET_LAST_CHANNEL_CALLBACK_ENABLE(x) ((((ADI_IIR_TASK_INFO *)(((ADI_IIR_TCB_INFO *)(x))->nIirSoftwareControl1))->bLastChannelCallback))

/*Task Info Structure */
typedef struct
{
    /* Interrupt Count */
	volatile uint32_t       nInterruptCount;
	/* Number Of Channels */
	uint32_t       nNumChannels;
	/* Pointer to Last TCB */
	void*          pLastTCB;
    /* State of the Task */
	ADI_IIR_TASK_STATE eTaskState;
	/* Flag to indicate that Last Channel Callback is needed */
	bool bLastChannelCallback;
	/* Device Handle */
	void * pDevice;
} ADI_IIR_TASK_INFO;

/* TCB Info Structure */
typedef struct
{
    /* Chain Pointer */
    void*          pNextTCB;

    /* Software Control Registers */
	void*         nIirSoftwareControl1; /* This field of the First TCB in the Chain will point to the Task Info */
    void*         nIirSoftwareControl2; /*  This field is not used  */

    /* Secondary Global Control Register */
	uint32_t       nIirSecondaryControl1;

    /* Coefficient Buffer */
    uint32_t       nCoeffCount;
    int32_t        nCoeffModify;
    void*          pCoeffIndex;

    /* Output Circular Buffer */
    void*          pOutputBase;
    uint32_t       nOutputCount;
    int32_t        nOutputModify;
    void* volatile pOutputIndex;

    /* Input Circular Buffer */
    void*          pInputBase;
    uint32_t       nInputCount;
    int32_t        nInputModify;
    void* volatile pInputIndex;

    /* Channel Control Register */
    uint32_t       nIirControl2;
} ADI_IIR_TCB_INFO;

#if defined(ADI_USE_DEBUG_DATA)
/* It is advised not to use debug Data while running infinite loop programs as the indexes for these
 * structures/data may go out of bound and cause unexpected results in the programs
 */
typedef struct
{
	int IirIntCnt[2];
	int SWIntCnt[2];
	int QueueTracker[30];
	int QueueTrackerIndex;
}ADI_IIR_DEBUG_DATA;

#endif /* defined(ADI_USE_DEBUG_DATA) */

/* Device Info structure */
typedef struct
{
	ADI_IIR_DEV_STATE         eDevState;
	ADI_CALLBACK              pfIIRCallback[ADI_IIR_NUM_OF_CALLBACKS];
	void *                    pIIRCBParam[ADI_IIR_NUM_OF_CALLBACK_PARAMS];
	ADI_IIR_TCB_INFO          *pLastTCBinQueue; /* This pointer points to the end TCB of the Accelerator TCB Chain. */
    ADI_IIR_TCB_INFO          *pActiveTaskTCB; /* This pointer points to the First TCB of the Current Active Task(A Task is just a TCB Chain supplied by the application)*/
    volatile uint32_t         nIntCntOfCurrentTask; /* This will keep track of the number of TCBs for which Interrupt has occurred in the Current Task */
    ADI_IIR_CFG0_TypeDef *const    pIIRRegs;           /* Base Address of IIR instance registers */
	uint32_t                  nIirDoneIntId;          /* SEC IIR DMA Done Interrupt ID */
#if defined(ADI_USE_DEBUG_DATA)
    volatile ADI_IIR_DEBUG_DATA    *pIirDebugData; /* Pointer to Debug Data Structure */
#endif /* ADI_USE_DEBUG_DATA */
}ADI_IIR_DEV_INFO;


#endif /* __ADI_IIR_DEF_ACM_V3_H__ */
