/*!
 *****************************************************************************
 * @file:    adi_iir_def_legacy_v3.h
 * @brief:   IIR Device Driver definition for Legacy Mode
 * @version: $Revision: 66664 $
 * @date:    $Date: 2021-08-02 09:50:29 -0400 (Mon, 02 Aug 2021) $
 *****************************************************************************
Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.
This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_iir_def_legacy_v3.h
     Description  :   Static function and structure definition needed by the device driver
                      for managing the IIR  device for ADSP-SC83x processors.
================================================================================ */

#ifndef __ADI_IIR_DEF_LEGACY_V3_H__
#define __ADI_IIR_DEF_LEGACY_V3_H__



/*==========  D E F I N E S  ==========*/


#define IIR_MAX_NUM_CHANNELS              (32u)
#define IIR_MAX_NUM_BIQUADS_PER_CHANNEL   (64u)
#define IIR_MAX_TOTAL_BIQUADS             (288u)

#define ADI_IIR_NUM_OF_CALLBACKS       (1u)
#define ADI_IIR_NUM_OF_CALLBACK_PARAMS (1u)
#define ADI_IIR_MASTER_CALLBACK_INDEX  (0u)

#define ADI_IIR_CFG_BURSTEN        (1ul << BITP_IIR_CTL1_BURSTEN)

#define ADI_IIR_CFG_L_TIMASK_EN    (1ul << BITP_IIR_CTL1_L_TIMASK_EN)
#define ADI_IIR_CFG_L_TWAIT_EN     (1ul << BITP_IIR_CTL1_L_TWAIT_EN)

/* Default IIRCTL1 Configuration */
#define IIR_DEF_IIRCTL1     (  (((uint32_t)ADI_IIR_CFG_ROUNDING_MODE)                  << BITP_IIR_CTL1_RND)      \
                             | (((uint32_t)ADI_IIR_CFG_SAVE_BIQUAD_STATE)              << BITP_IIR_CTL1_SS)       \
							 | (((uint32_t)ADI_IIR_CFG_FORTYBIT_FLOATING_POINT_SELECT) << BITP_IIR_CTL1_FORTYBIT) \
							 | (((uint32_t)ADI_IIR_CFG_BURSTEN))                                                  \
							 | (((uint32_t)ADI_IIR_CFG_L_TIMASK_EN))                                              \
							 | (((uint32_t)ADI_IIR_CFG_L_TWAIT_EN))                                               \
							 | (((uint32_t)ADI_IIR_CFG_SLEW_EN) << BITP_IIR_CTL1_SLEW_EN)                                                  \
                            )

/* IIR_CTL1 Register macros */
#define IIR_IIRCTL1_EN_ENABLE                     (BITM_IIR_CTL1_EN)
#define IIR_IIRCTL1_DMAEN_ENABLE                  (BITM_IIR_CTL1_DMAEN)


/* TCB Info Structure */
typedef struct
{
    /* Chain Pointer */
    void*          pNextTCB;

    /* Coefficient Buffer */
    uint32_t       nCoeffLength;
    int32_t        nCoeffModify;
    void*          pCoeffIndex;

#if (ADI_IIR_CFG_SLEW_EN == 1)
    /* Slewing Enabled */
    void*          pTargetCoeffIndex;
    uint32_t       nAlpha;
    uint32_t       nNn;
#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */
    /* Output Circular Buffer */
    void*          pOutputBase;
    uint32_t       nOutputLength;
    int32_t        nOutputModify;
    void* volatile pOutputIndex;

    /* Input Circular Buffer */
    void*          pInputBase;
    uint32_t       nInputLength;
    int32_t        nInputModify;
    void* volatile pInputIndex;

    /* Registers */
    uint32_t       nIirControl2;
} ADI_IIR_TCB_INFO;

/* Task Info structure */
typedef struct ADI_IIR_TASK_INFO_T
{
	struct ADI_IIR_TASK_INFO_T     *pNext;
	ADI_IIR_TCB_INFO               *pTCBChain;
	uint32_t                       nNumChannels;
	uint32_t                       nIIRCTL1;
	ADI_IIR_TASK_STATE             eTaskState;
	ADI_IIR_DEV_HANDLE             *pDevice;
	ADI_CORE_ID                    eCoreId;
	/* The Reserved bytes are added to make sure that Task Info and TCB Chain are on two different cache lines */
	uint8_t                        RESERVED[ADI_CACHE_LINE_LENGTH];
}ADI_IIR_TASK_INFO;

#if defined(ADI_USE_DEBUG_DATA)

typedef struct
{
	int IirIntCnt[2];
	int SWIntCnt[2];
	int QueueTracker[30];
	int QueueTrackerIndex;
}ADI_IIR_DEBUG_DATA;

#endif /* defined(ADI_USE_DEBUG_DATA) */

/* Queue Info Structure */
typedef struct
{
	ADI_IIR_TASK_INFO *pActiveTask;
 	ADI_IIR_TASK_INFO *pLastTaskAdded;
    ADI_IIR_TASK_INFO *pTaskCompleted;
#if defined(ADI_USE_DEBUG_DATA)
    volatile ADI_IIR_DEBUG_DATA *pIirDebugData;
#endif/* ADI_USE_DEBUG_DATA */
}ADI_IIR_QUEUE_INFO;

/* Device Info structure */
typedef struct
{
	ADI_IIR_DEV_STATE         eDevState;
	ADI_IIR_QUEUE_INFO        *pQueue;
	ADI_CALLBACK              pfIIRCallback[ADI_IIR_NUM_OF_CALLBACKS];
	void *                    pIIRCBParam[ADI_IIR_NUM_OF_CALLBACK_PARAMS];
	ADI_IIR_CFG0_TypeDef *const    pIIRRegs;           /* Base Address of IIR instance registers */
	uint32_t                  nIirDoneIntId;          /* SEC IIR DMA Done Interrupt ID */
}ADI_IIR_DEV_INFO;


#endif /* __ADI_IIR_DEF_LEGACY_V3_H__ */
