/*******************************************************************************

Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you
agree to the terms of the associated Analog Devices License Agreement.

*******************************************************************************/

/*!
* @file      adi_iir_legacy_v3.c
*
* @brief     IIR Accelerator Driver Legacy Mode source file for ADSP-SC83x processors.
*
* @details
*            This is the primary source file for the IIR Accelerator Legacy Mode implementation for ADSP-SC83x processors.
*/

#ifndef __ADI_IIR_LEGACY_V3_C__
#define __ADI_IIR_LEGACY_V3_C__
/*==========  I N C L U D E  ==========*/
#include <stdlib.h>
#include <string.h>
#include <adi_osal.h>
#include <services/int/adi_int.h>
#include <assert.h>
#include <sys/platform.h>
#include <drivers/iir/adi_iir.h>
#include "adi_iir_def_v3.h"
#include <sys/adi_core.h>
#include <services/int/adi_sec.h>


/** @addtogroup IIR_Driver
 *  @{
 
 The IIR Accelerator Driver follows a queuing programming model. Applications can create tasks for the IIR accelerator using the adi_iir_CreateTask() and queue the same as and when required using the adi_iir_QueueTask(). The driver maintains a processing queue of the tasks given/queued by the
 application for the accelerator and schedules the same to the accelerator one after another. The applications can register a user defined callback using adi_iir_RegisterCallback().  The driver uses the callback to notify the application  on completion of the tasks/channels in a task. 
 The driver also maintains the status of all the tasks the application has queued. The application can also query the status of the task as required using the adi_iir_GetIirTaskStatus().Once the task which was queued is completed by the accelerator, the task is removed from the processing queue. 
 The driver maintains the information of all the tasks which are completed/created. Hence the application can reuse the tasks which were created by simply queuing it again using adi_iir_QueueTask(). The accelerator can be configured to operate in Legacy Mode or Auto Configuration Mode(ACM) using static configuration.
 In Legacy Mode, the driver maintains a software queue(Linked List) of the tasks queued by the application and schedules the same to the accelerator one after another. In Auto Configuration Mode(ACM), the driver utilizes the Halt feature in hardware to implement hardware queuing mechanism. 
 There are additional channel customizable features available in Auto Configuration Mode(ACM) like interrupts/triggers/save state and more. Please refer to ADI_IIR_CHANNEL_INFO for more details. 
 The programming model of the driver is the same for both Legacy and Auto Configuration Mode(ACM) of operation.
 
 The Driver also has debug data structures(which contains number of interrupts raised) which can be used by the application for debug. The definitions/use of these debug data structures
 are enabled by the macro ADI_USE_DEBUG_DATA which can be added in the application project. It is advised not to use debug Data while running infinite loop programs as the indexes for these
 structures/data may go out of bound and cause unexpected results in the programs.

 @}*/

/*==========  L O C A L    F U N C    D E F S  ==========*/
/* Interrupt Handler for the IIR Device */
static void IirInterruptHandler (uint32_t iid, void* pParam);

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_IIR_RESULT ResetAccelerator(ADI_IIR_DEV_INFO *  pDevice);

/* Resets the given Task Structure */
static ADI_IIR_RESULT ResetTaskStructure (ADI_IIR_TASK_INFO *  pTask);

/* Starts the Queue Processing by writing to Chain Pointer and Control Register */
static ADI_IIR_RESULT StartQueue (ADI_IIR_DEV_INFO *  pDevice);

/* Translates the Given TCB address by adding appropriate offsets */
static uint32_t TranslateAddr (uint32_t  MemAddr);

static uint32_t RevertTranslatedAddr (uint32_t  MemAddr);

/* Validate the given device handle */
static ADI_IIR_RESULT ValidateDeviceHandle (ADI_IIR_DEV_HANDLE hDevice);

/* Validate the given Task Handle */
static ADI_IIR_RESULT ValidateTaskHandle (ADI_IIR_TASK_HANDLE hTask);

/*==========  D E B U G  S T R U C T U R E S  ==========*/

#if defined(ADI_USE_DEBUG_DATA)

ADI_IIR_DEBUG_DATA gIirDebugData[] =
{
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#if (IIR_NUM_DEVICES > 1)
	,{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 1) */
#if (IIR_NUM_DEVICES > 2)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 2) */
#if (IIR_NUM_DEVICES > 3)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 3) */
#if (IIR_NUM_DEVICES > 4)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 4) */
#if (IIR_NUM_DEVICES > 5)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 5) */
#if (IIR_NUM_DEVICES > 6)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 6) */
#if (IIR_NUM_DEVICES > 7)
	,
	{
		{0,0},
		{0,0},
		{0},
		0
	}
#endif /* (IIR_NUM_DEVICES > 7) */
};

#endif /* ADI_USE_DEBUG_DATA */

/*==========  D A T A  ==========*/

/*static*/ ADI_IIR_QUEUE_INFO gIirQueueInfo[] =
{
    {
        NULL,                                             /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
        ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[0]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
    }
#if (IIR_NUM_DEVICES > 1)
	,{
	    NULL,                                             /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[1]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 1) */
#if (IIR_NUM_DEVICES > 2)
    ,
	{
	    NULL,                                              /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[2]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 2) */
#if (IIR_NUM_DEVICES > 3)
    ,
	{
	    NULL,                                              /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[3]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 3) */
#if (IIR_NUM_DEVICES > 4)
    ,
	{
	    NULL,                                              /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[4]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 4) */
#if (IIR_NUM_DEVICES > 5)
    ,
	{
	    NULL,                                              /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[5]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 5) */
#if (IIR_NUM_DEVICES > 6)
    ,
	{
	    NULL,                                              /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[6]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 6) */
#if (IIR_NUM_DEVICES > 7)
    ,
	{
	    NULL,                                              /* Pointer to Active Task in the Queue. In Non-Queuing Mode, this pointer is used to keep track of the assigned task to accelerator */
        NULL,                                              /* Pointer to Last Added Task in Queue. This pointer is not used in Non-Queuing Mode */
		NULL                                              /* Pointer to Task Completed */
#if defined(ADI_USE_DEBUG_DATA)
	    ,(volatile ADI_IIR_DEBUG_DATA *)&gIirDebugData[7]          /* Pointer to the Debug Data of the Queue */
#endif/* ADI_USE_DEBUG_DATA */
	}
#endif /* (IIR_NUM_DEVICES > 7) */
};

/*static*/ ADI_IIR_DEV_INFO gIirDeviceInfo[] =
{
    {
        ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
    	(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[0],             /* IIR Queue Info */
    	{NULL},                                           /* Pointer to IIR Callback functions */
    	{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR0_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR0_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR0_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */
    }
#if (IIR_NUM_DEVICES > 1)
	,{
	    ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
	 	(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[1],             /* IIR Queue Info */
	   	{NULL},                                           /* Pointer to IIR Callback functions */
	   	{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR1_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR1_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR1_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

	}
#endif /* (IIR_NUM_DEVICES > 1) */
#if (IIR_NUM_DEVICES > 2)
    ,
	{
		ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
		(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[2],             /* IIR Queue Info */
		{NULL},                                           /* Pointer to IIR Callback functions */
		{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR2_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR2_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR2_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

	}
#endif /* (IIR_NUM_DEVICES > 2) */
#if (IIR_NUM_DEVICES > 3)
    ,
	{
		ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
		(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[3],             /* IIR Queue Info */
		{NULL},                                           /* Pointer to IIR Callback functions */
		{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR3_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR3_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR3_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */

	}
#endif /* (IIR_NUM_DEVICES > 3u) */
#if (IIR_NUM_DEVICES > 4)
    ,
	{
		ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
		(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[4],             /* IIR Queue Info */
		{NULL},                                           /* Pointer to IIR Callback functions */
		{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR4_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR4_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR4_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */
	}
#endif /* (IIR_NUM_DEVICES > 4) */
#if (IIR_NUM_DEVICES > 5)
    ,
	{
		ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
		(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[5],             /* IIR Queue Info */
		{NULL},                                           /* Pointer to IIR Callback functions */
		{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR5_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR5_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR5_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */
	}
#endif /* (IIR_NUM_DEVICES > 5) */
#if (IIR_NUM_DEVICES > 6)
    ,
	{
		ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
		(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[6],             /* IIR Queue Info */
		{NULL},                                           /* Pointer to IIR Callback functions */
		{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR6_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR6_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR6_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */
	}
#endif /* (IIR_NUM_DEVICES > 6) */
#if (IIR_NUM_DEVICES > 7)
    ,
	{
		ADI_IIR_DEV_STATE_NOTINIT,                        /* IIR device state */
		(ADI_IIR_QUEUE_INFO *)&gIirQueueInfo[7],             /* IIR Queue Info */
		{NULL},                                           /* Pointer to IIR Callback functions */
		{NULL},                                           /* Pointer to Callback Data Parameter */
		(ADI_IIR_CFG0_TypeDef *)ADI_IIR7_BASE,                        /* Pointer to Base Address of IIR instance registers */

#if (ADI_IIR_CFG_SEC_INT == 1)
		(uint32_t)INTR_SEC_IIR7_DMA                           /* DMA Done SEC Interrupt ID */
#else
		(uint32_t)INTR_IIR7_DMA                        /* DMA Done Direct Interrupt ID */
#endif /* (ADI_IIR_CFG_SEC_INT == 1) */
	}
#endif /* (IIR_NUM_DEVICES > 7) */
};

#define IIR_MUTEX_LOCK(X)
#define IIR_MUTEX_UNLOCK(X)


/*==========  C O D E  ==========*/

/*!
 * @brief       adi_iir_Open API opens a IIR device.
 *
 * @details     Opens a specified IIR device and returns the handle to the device. It also installs the interrupt handler and configures the interrupt.
 *
 * @param [in]  nDeviceNum     The IIR device number to be opened.
 *
 * @param [out] phIirHandle    The pointer to a location where the handle to the opened device is written.
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully opened the IIR device.
 *
 *
 * @sa          adi_iir_Close().
 */
ADI_IIR_RESULT adi_iir_Open (uint8_t             nDeviceNum,
							 ADI_IIR_DEV_HANDLE *phIirHandle)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;

#if defined (ADI_DEBUG)
    assert(nDeviceNum <= (uint8_t)(IIR_NUM_DEVICES - 1));
    assert(phIirHandle != NULL);
#endif

    {

		/* Enter Critical Region */
		adi_osal_EnterCriticalRegion();
		/* Lock the Mutex */
		IIR_MUTEX_LOCK(&gIirMutex);

		/* Get the pointer to device */
		pDevice = &gIirDeviceInfo[nDeviceNum];

		/* Reset IIR_CTL1 and IIR_CHNPTR Registers */
		ResetAccelerator(pDevice);

		/* Install the IIR interrupt handler */
		adi_int_InstallHandler(pDevice->nIirDoneIntId, &IirInterruptHandler, pDevice, true);

#if ((ADI_IIR_CFG_SEC_INT == 1))
		/* Configure the IIR Interrupt as Edge Sensitive at SEC */
		adi_sec_EnableEdgeSense(ADI_IID_TO_SEC_INT(pDevice->nIirDoneIntId),true);
#endif /* ((ADI_IIR_CFG_SEC_INT == 1)) */

#if ((ADI_IIR_CFG_SEC_INT == 0))
		/* Configure the IIR Direct Interrupt as Edge Sensitive */
		xthal_interrupt_sens_set(ADI_IID_TO_CORE_INT(pDevice->nIirDoneIntId), 0);
#endif /* ((ADI_IIR_CFG_SEC_INT == 0)) */

		/* Initialize the Device State */
		pDevice->eDevState = ADI_IIR_DEV_STATE_IDLE;

		/* Write the Handle to the location given by the application */
		*phIirHandle = pDevice;

		/* Unlock the Mutex */
		IIR_MUTEX_UNLOCK(&gIirMutex);
		/* Exit Critical Region */
		adi_osal_ExitCriticalRegion();
    }
    /* Return the status back to the application */
    return (eResult);
}

/*!
 * @brief       adi_iir_Close API closes a given IIR device.
 *
 * @details     Closes the given IIR device and resets the IIR queue. It also uninstalls the interrup handler.
 *
 * @param [in]  hIirHandle         IIR Device handle
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully closed the IIR device.
 *
 *  - #ADI_IIR_RESULT_FAILED                  If unable to successfully close the device.
 *
 * @sa          adi_iir_Open().
 */
ADI_IIR_RESULT adi_iir_Close (ADI_IIR_DEV_HANDLE hIirHandle)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    IIR_MUTEX_LOCK(&gIirMutex);

    /* Get the device handle */
    pDevice = (ADI_IIR_DEV_INFO*)hIirHandle;

    /* Reset IIR_CTL1 and IIR_CHNPTR Registers */
    ResetAccelerator(pDevice);

    /* Uninitialize the Device State and Queue Info*/
    pDevice->eDevState = ADI_IIR_DEV_STATE_NOTINIT;
    pDevice->pQueue->pActiveTask = NULL;
    pDevice->pQueue->pLastTaskAdded = NULL;

    /* Uninstall IIR interrupt handler */
    if( adi_int_UninstallHandler (pDevice->nIirDoneIntId) != ADI_INT_SUCCESS)
    {
        eResult = ADI_IIR_RESULT_FAILED;
    }

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);
    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the status */
    return (eResult);
}


/*!
 * @brief       adi_iir_CreateTask API creates a Task and the related TCB chain from the supplied Channel Information.
 *
 * @details     This API will
 *              - Create a IIR task for given IIR device,
 *              - Create a TCB chain from given Channel list
 *
 * @param [in]  hIirHandle        The IIR device handle
 *
 * @param [in]  pChannelList      Array of Channel information using which TCB chain will be created
 *
 * @param [in]  nNumChannels      Number of Channels(TCBs) in this task
 *
 * @param [in]  pMemory           Memory to store the TCB chain.
 *
 * @param [in]  nMemSize          Size of the memory provided (calculated on the basis of number of channels)\n
 *                                Memory Required = SIZE_OF_TASK_INFO + NumberOfChannels*(SIZE_OF_TCB_INFO)\n
 *                                = 28 + NumberOfChannels*52 bytes(Legacy Mode)\n
 *                                = 24 + NumberOfChannels*64 bytes(Auto Configuration Mode)\n
 *                                Please use macro IIR_MEM_SIZE(NO_OF_CHANNELS) which takes Number of Channels as argument for assigning memory to IIR tasks.
 *
 * @param [out]  phIirTask        Address to store the IIR task handle
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS     If successfully created a Task for the IIR device.
 *
 * @sa          adi_iir_QueueTask()
 */
ADI_IIR_RESULT adi_iir_CreateTask (ADI_IIR_DEV_HANDLE     hIirHandle,
                                   ADI_IIR_CHANNEL_INFO  *pChannelList,
                                   uint32_t               nNumChannels,
                                   void                  *pMemory,
                                   uint32_t               nMemSize,
                                   ADI_IIR_TASK_HANDLE   *phIirTask)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
    ADI_IIR_TASK_INFO *pTask;
    uint32_t IirCtl2;
    ADI_IIR_TCB_INFO * pCurrentTCB;
	ADI_IIR_TCB_INFO * pTempTCB;
    uint32_t count;
    uint32_t count1;
#if defined (ADI_DEBUG)
    uint32_t TotalBiquadCount = 0u;
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);

    /* Check whether pointers passed are not NULL */
    assert((pChannelList != NULL) &&  (pMemory != NULL) && (phIirTask != NULL));

    /* Check if the memory supplied is sufficient */
    assert(nMemSize >=(ADI_IIR_TASK_INFO_SIZE + (nNumChannels * ADI_IIR_TCB_INFO_SIZE)));

    /* Check if nNumChannels is less than Max. No of Channels */
    assert(nNumChannels <= IIR_MAX_NUM_CHANNELS);

    /* Check if the No of Biquads for each channel is less than Max. No of Biquads. Also Total Biquad Count must be less than or equal to 288 */
    for(count1 = 0u; count1< nNumChannels; count1++)
    {
        assert((pChannelList[count1].nBiquads) <= IIR_MAX_NUM_BIQUADS_PER_CHANNEL);
        TotalBiquadCount = TotalBiquadCount + pChannelList[count1].nBiquads;
        assert((TotalBiquadCount) <= IIR_MAX_TOTAL_BIQUADS);
    }

#if (ADI_IIR_CFG_SLEW_EN == 1)
    for(count1 = 0u; count1< nNumChannels; count1++)
    {
        /* Check if bits [31:24] are zero */
        assert(((pChannelList[count1].nNn) & 0xFF000000) == 0x00000000);

        /* Check if slewing frequency is a multiple of window size in linear alpha mode */
        if(pChannelList[count1].bLinearAlpha == true)
        {
        	assert(((pChannelList[count1].nNn) % (pChannelList[count1].nWindowSize)) == 0x00000000);
        }

        /* Check if window size is a multiple of  slewing frequency in constant alpha mode */
        if(pChannelList[count1].bLinearAlpha == false)
        {
        	assert(((pChannelList[count1].nWindowSize) % ((pChannelList[count1].nNn) & BITM_IIR_SLEW_CNFG_SLEW_FREQ)) == 0x00000000);
        }
    }



#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */

#endif /* ADI_DEBUG */
    /* Get the device handle */
    pDevice = (ADI_IIR_DEV_INFO*)hIirHandle;

    /* Make the Task Head as the pMemory Supplied*/
    pTask = (ADI_IIR_TASK_INFO*)pMemory;

    /* Write the Task parameters */
    pTask->eCoreId = adi_core_id();
    pTask->pDevice = (ADI_IIR_DEV_HANDLE)pDevice;
    pTask->nIIRCTL1 =((nNumChannels - 1u)<<BITP_IIR_CTL1_CH) | (IIR_DEF_IIRCTL1);
    pTask->nNumChannels = nNumChannels;
    pTask->pNext = NULL;
    pTask->pTCBChain = (ADI_IIR_TCB_INFO *)((ADI_IIR_TASK_INFO *)pMemory + 1);

    /*Create a TCB Chain */
    pCurrentTCB = pTask->pTCBChain;
    for(count = 0u; count< nNumChannels; count++)
    {
    	/*Create IIRCTL2 register */
    	IirCtl2 = (pChannelList[count].nBiquads - 1u) | ((pChannelList[count].nWindowSize -1u)<<BITP_IIR_CTL2_WINDOW);

    	/* For ADSP-SC594, Enable interrupt for the last channel. This is done to keep the abstractions for legacy and ACM similar across all platforms(ADSP-SC589/ADSP-SC573/ADSP-21569/ADSP-SC594) i.e
    	 * in legacy mode, we get interrupt after completion of a task and in ACM mode, interrupt is customizableacross channels */
        if(count != (nNumChannels - 1u))
        {
        	IirCtl2 |= BITM_IIR_CTL2_IMASK;
        }

        /* Set TMASK and IMASK */
        if(pChannelList[count].bTriggerEnable == false)
        {
        	IirCtl2 |= BITM_IIR_CTL2_TMASK;
        }

        if(pChannelList[count].bTriggerWaitEnable == true)
        {
        	IirCtl2 |= BITM_IIR_CTL2_TWAIT;
        }

#if (ADI_IIR_CFG_SLEW_EN == 1)
       if(pChannelList[count].bLinearAlpha == true)
        {
        	IirCtl2 |= BITM_IIR_CTL2_SLEW_LALPHA_MODE;
        }

       if(pChannelList[count].bLinearAlpha == false)
        {
    	   if(pChannelList[count].bSaturationEnable == true)
    	   {
               IirCtl2 |= BITM_IIR_CTL2_SLEW_SAT_EN;
    	   }
        }

#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */



    	/* Write to the nIirControl2 field */
    	pCurrentTCB->nIirControl2 = IirCtl2;

    	/*Copy the Coefficient, Input and Output Parameters */
        pCurrentTCB->nCoeffLength   = (uint32_t)pChannelList[count].nCoefficientLength;
        pCurrentTCB->nCoeffModify   = (int32_t)pChannelList[count].nCoefficientModify;
        pCurrentTCB->pCoeffIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pCoefficientIndex);
#if (ADI_IIR_CFG_SLEW_EN == 1)
        /* Update the slewing parameters */
        pCurrentTCB->pTargetCoeffIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pTargetCoefficientIndex);
        pCurrentTCB->nAlpha               = (uint32_t)pChannelList[count].nAlpha;

        /* Note that actual programmed value of 'n' should be 'n-1' */
        pCurrentTCB->nNn                  = ((uint32_t)pChannelList[count].nNn - 1u);
#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */
        pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffBase);
        pCurrentTCB->nOutputLength  = (uint32_t)pChannelList[count].nOutputBuffLength;
        pCurrentTCB->nOutputModify  = (int32_t)pChannelList[count].nOutputBuffModify;
        pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelList[count].pOutputBuffIndex);

        pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffBase);
        pCurrentTCB->nInputLength   = (uint32_t)pChannelList[count].nInputBuffLength;
        pCurrentTCB->nInputModify   = (int32_t)pChannelList[count].nInputBuffModify;
        pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelList[count].pInputBuffIndex);

        /* Update Next Pointer of TCB */
        if(count == (nNumChannels - 1u))
        {
        	pCurrentTCB->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTask->pTCBChain->nIirControl2));
        }
        else
        {
		      pTempTCB = pCurrentTCB + 1;
        	  pCurrentTCB->pNextTCB = (void *)TranslateAddr((uint32_t)(&pTempTCB->nIirControl2));
        }

        pCurrentTCB = pCurrentTCB + 1;

    }

    /* Mark the Task State as None */
    pTask->eTaskState = ADI_IIR_TASK_STATE_NONE;

    /* Write the Task Handle to the location supplied */
    *phIirTask = pTask;

    /* Return the Status */
    return (eResult);
}

#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
/*!
 * @brief       adi_iir_QueueTask API queues the given Task to IIR Task Queue if the IIR accelerator is already running. If not, it allocates
 *              the given task to the IIR accelerator.
 *
 * @details     The adi_iir_QueueTask API queues the given task if the accelerator is already running.If not, it allocates
 *              the given task to the accelerator. It asserts to check whether the task is already running or queued or if the device is not opened.
 *              In Legacy Mode, a software queue is maintained for the accelerator device. In Auto Configuration Mode, the API makes use of the
 *              Halt feature to implement hardware queuing.
 *
 * @param [in]  hIirTask   IIR task handle.
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully Queued the given task.
 *
 *  - #ADI_IIR_RESULT_INVALID_OPERATION       If the device is not opened.In ACM Mode, this result is not returned.
 *
 * @sa          adi_iir_CreateTask().
 */
ADI_IIR_RESULT adi_iir_QueueTask (ADI_IIR_TASK_HANDLE hIirTask)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TASK_INFO *pTask = (ADI_IIR_TASK_INFO*)hIirTask;
	ADI_IIR_DEV_STATE eDevState;
    ADI_IIR_DEV_INFO *pDevice;

#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
#endif /* ADI_USE_DEBUG_DATA */
#if (ADI_CACHE_MANAGEMENT == 1)
   ADI_IIR_TCB_INFO* pTCBChain;
   uint8_t nNumChannels = (uint8_t)0;
   uint8_t Count = (uint8_t)0;
   void *_start = NULL;
   void *_end = NULL;
   void  *temp;
   int32_t nCoeffModify = 0;
#endif /* (ADI_CACHE_MANAGEMENT == 1) */
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
    /* Check if the Task is running or queued */
    assert((pTask->eTaskState != ADI_IIR_TASK_STATE_QUEUED) && (pTask->eTaskState != ADI_IIR_TASK_STATE_RUNNING));
#endif

    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    IIR_MUTEX_LOCK(&gIirMutex);

    /* Get the Device Handle */
    pDevice = (ADI_IIR_DEV_INFO *)pTask->pDevice;

    /* Get the Device State */
    eDevState = pDevice->eDevState;
#if (ADI_CACHE_MANAGEMENT == 1)
    pTCBChain = pTask->pTCBChain;
    nNumChannels = (uint8_t)pTask->nNumChannels;

    /* Input Buffer-Flush,Output Buffer-Invalidate, (Coefficient Buffer-Flush&Invalidate because the DMA writes the States back) */
    for(Count=0u;Count<nNumChannels;Count++)
    {

       /* Flush Input Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pInputBase));
       _end = (float *)_start + (pTCBChain->nInputLength);
       flush_data_buffer(_start,_end,0);

       /* Invalidate Output Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pOutputBase));
       _end = (float *)_start + (pTCBChain->nOutputLength);
       flush_data_buffer(_start,_end,1);

       /* Flush&Invalidate Coefficient Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pCoeffIndex));
       nCoeffModify = pTCBChain->nCoeffModify;
       if(nCoeffModify < 0)
       {
           _end = (float *)_start - (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffLength));
       }
       else
       {
    	   _end = (float *)_start + (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffLength));
       }
       flush_data_buffer(_start,_end,1);

       if(Count!= (nNumChannels-1u))
       {
		   /* Update the TCB Pointer to point to the next TCB */
		   temp = pTCBChain->pNextTCB;
		   pTCBChain = (void *)((uint32_t *)RevertTranslatedAddr((uint32_t)(temp)) - ((ADI_IIR_TCB_INFO_SIZE/4u)-1u));
       }

    }

    /* Flush the TCB Chain */
    _start = pTask->pTCBChain;
    _end = ((ADI_IIR_TCB_INFO *)_start) + nNumChannels;
    flush_data_buffer(_start,_end,0);

#endif /* (ADI_CACHE_MANAGEMENT == 1) */
    switch(eDevState)
    {
		case ADI_IIR_DEV_STATE_IDLE :
			/* Update Queue */
			pDevice->pQueue->pActiveTask = pTask;
			pDevice->pQueue->pLastTaskAdded = pTask;
			/*Update Task State */
			pTask->eTaskState = ADI_IIR_TASK_STATE_QUEUED;
			/* Start Accelerator */
			StartQueue(pDevice);
			/*Update Device State */
			pDevice->eDevState = ADI_IIR_DEV_STATE_INUSE;
			/* Update Task State */
			pTask->eTaskState = ADI_IIR_TASK_STATE_RUNNING;
			break;

		case ADI_IIR_DEV_STATE_INUSE :
/*-------------------FOR_DEBUG------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
	    eCoreID = adi_core_id();
        if(eCoreID == ADI_CORE_SHARC0)
        {
	        pDevice->pQueue->pIirDebugData->QueueTracker[pDevice->pQueue->pIirDebugData->QueueTrackerIndex++] = 1;
        }
#if __NUM_SHARC_CORES__ > 1
        else if(eCoreID == ADI_CORE_SHARC1)
        {
        	pDevice->pQueue->pIirDebugData->QueueTracker[pDevice->pQueue->pIirDebugData->QueueTrackerIndex++] = 2;
        }
#endif /*  __NUM_SHARC_CORES__ */
	    else
	    {
            /* MISRA Compliance */
	    }
#endif /* ADI_USE_DEBUG_DATA */
/*-------------------FOR_DEBUG------------------------------*/
			/* Update Queue */
			pDevice->pQueue->pLastTaskAdded->pNext = pTask;
			pDevice->pQueue->pLastTaskAdded = pTask;
			/*Update Task State */
			pTask->eTaskState = ADI_IIR_TASK_STATE_QUEUED;
			break;

		case ADI_IIR_DEV_STATE_NOTINIT:
			eResult = ADI_IIR_RESULT_INVALID_OPERATION;
			break;

		default :
			break;
    }

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);
    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the result */
    return (eResult);
}

#endif /* ADI_IIR_CFG_NOQUEUE_MODE == 0 */

#if (ADI_IIR_CFG_NOQUEUE_MODE == 1)
/*!
 * @brief       adi_iir_StartTask API assigns the task to the accelerator and starts it.
 *
 * @details     adi_iir_StartTask API assigns the task to the accelerator and starts it. In Non-Queuing Mode, only one task can be assigned to the accelerator and it can be run for multiple iterations without
 *              needing to reload the coefficients for every iteration. This API returns ADI_IIR_RESULT_INVALID_OPERATION if there is already some other task assigned to the accelerator or if the same task is already running on the acclerator.
 *              When the task is completed, a callback if registered is called and the task state moves to ADI_IIR_TASK_STATE_ASSIGNED. The next iteration of the same task can be started by calling this same API. To unassign the task
 *              from the accelerator, adi_iir_UnassignTask API is used.
 * @param [in]  hIirTask   IIR task handle.
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully Queued the given task.
 *
 *  - #ADI_IIR_RESULT_INVALID_OPERATION       If the device is not opened OR if there is already some other task assigned to the acclerator OR if the same task is already running.
 *
 * @sa          adi_iir_CreateTask().
 */
ADI_IIR_RESULT adi_iir_StartTask (ADI_IIR_TASK_HANDLE hIirTask)
{

    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TASK_INFO *pTask = (ADI_IIR_TASK_INFO*)hIirTask;
	ADI_IIR_DEV_STATE eDevState;
    ADI_IIR_DEV_INFO *pDevice;

#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
#endif /* ADI_USE_DEBUG_DATA */
#if (ADI_CACHE_MANAGEMENT == 1)
   ADI_IIR_TCB_INFO* pTCBChain;
   uint8_t nNumChannels = (uint8_t)0;
   uint8_t Count = (uint8_t)0;
   void *_start = NULL;
   void *_end = NULL;
   void  *temp;
   int32_t nCoeffModify = 0;
#endif /* (ADI_CACHE_MANAGEMENT == 1) */
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
    /* Check if the Task is running  */
    assert(pTask->eTaskState != ADI_IIR_TASK_STATE_RUNNING);
#endif

    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    IIR_MUTEX_LOCK(&gIirMutex);

    /* Get the Device Handle */
    pDevice = (ADI_IIR_DEV_INFO *)pTask->pDevice;

    /* Get the Device State */
    eDevState = pDevice->eDevState;
#if (ADI_CACHE_MANAGEMENT == 1)
    pTCBChain = pTask->pTCBChain;
    nNumChannels = (uint8_t)pTask->nNumChannels;

    /* Input Buffer-Flush,Output Buffer-Invalidate, (Coefficient Buffer-Flush&Invalidate because the DMA writes the States back) */
    for(Count=0u;Count<nNumChannels;Count++)
    {

       /* Flush Input Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pInputBase));
       _end = (float *)_start + (pTCBChain->nInputLength);
       flush_data_buffer(_start,_end,0);

       /* Invalidate Output Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pOutputBase));
       _end = (float *)_start + (pTCBChain->nOutputLength);
       flush_data_buffer(_start,_end,1);

       /* Flush&Invalidate Coefficient Buffer */
       _start = (void *)RevertTranslatedAddr(((uint32_t)pTCBChain->pCoeffIndex));
       nCoeffModify = pTCBChain->nCoeffModify;
       if(nCoeffModify < 0)
       {
           _end = (float *)_start - (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffLength));
       }
       else
       {
    	   _end = (float *)_start + (((uint32_t)(abs(nCoeffModify)))*(pTCBChain->nCoeffLength));
       }
       flush_data_buffer(_start,_end,1);

       if(Count!= (nNumChannels-1u))
       {
		   /* Update the TCB Pointer to point to the next TCB */
		   temp = pTCBChain->pNextTCB;
		   pTCBChain = (void *)((uint32_t *)RevertTranslatedAddr((uint32_t)(temp)) - ((ADI_IIR_TCB_INFO_SIZE/4u)-1u));
       }

    }

    /* Flush the TCB Chain */
    _start = pTask->pTCBChain;
    _end = ((ADI_IIR_TCB_INFO *)_start) + nNumChannels;
    flush_data_buffer(_start,_end,0);

#endif /* (ADI_CACHE_MANAGEMENT == 1) */

    switch(eDevState)
    {
		case ADI_IIR_DEV_STATE_IDLE :
		    /* Return ADI_RESULT_INVALID_OPERATION if the task is already running or if there is already some other task assigned to the accelerator */
		    if((pTask->eTaskState == ADI_IIR_TASK_STATE_RUNNING) || ((pDevice->pQueue->pActiveTask != NULL) && (pDevice->pQueue->pActiveTask != pTask)))
		    {
		    	eResult = ADI_IIR_RESULT_INVALID_OPERATION;
		    }
		    else
		    {
		    	/* It means that there is either no task assigned or the same task is assigned and is completed */

		    	/* No task assigned */
		    	if(pDevice->pQueue->pActiveTask == NULL)
		    	{
                    /* Update the ActiveTask pointer */
		    		pDevice->pQueue->pActiveTask = pTask;

					/*Update Task State */
					pTask->eTaskState = ADI_IIR_TASK_STATE_ASSIGNED;

					/* Start Accelerator */
					StartQueue(pDevice);

					/*Update Device State */
					pDevice->eDevState = ADI_IIR_DEV_STATE_INUSE;

					/* Update Task State */
					pTask->eTaskState = ADI_IIR_TASK_STATE_RUNNING;
					break;
		    	}
		    	else
				{
		    		/* It means the previous iteration of the task has completed.We only need to set IIR_CTL1.DMAEN to start the task */
					/* Start Accelerator */
					pDevice->pIIRRegs->CTL1 |= IIR_IIRCTL1_DMAEN_ENABLE;

					/*Update Device State */
					pDevice->eDevState = ADI_IIR_DEV_STATE_INUSE;

					/* Update Task State */
					pTask->eTaskState = ADI_IIR_TASK_STATE_RUNNING;
					break;
				}

		    }

		case ADI_IIR_DEV_STATE_INUSE :
		    /* Return ADI_RESULT_INVALID_OPERATION as there is some task running already */
		    eResult = ADI_IIR_RESULT_INVALID_OPERATION;
			break;

		case ADI_IIR_DEV_STATE_NOTINIT:
			eResult = ADI_IIR_RESULT_INVALID_OPERATION;
			break;

		default :
			break;
    }

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);
    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the result */
    return (eResult);
}

/*!
 * @brief       adi_iir_UnassignTask API un-assigns the task from the accelerator device.
 *
 * @details     adi_iir_StartTask API un-assigns the task from the accelerator device. In Non-Queuing Mode, only one task can be assigned to the accelerator and it can be run for multiple iterations without
 *              needing to reload the coefficients for every iteration. This API returns ADI_IIR_RESULT_INVALID_OPERATION if the device is already running a task or if there is no task assigned to the device.
 *              When the task is completed, a callback if registered is called and the task state moves to ADI_IIR_TASK_STATE_ASSIGNED. The next iteration of the same task can be started by calling this same API. To unassign the task
 *              from the accelerator, adi_iir_UnassignTask API is used.
 * @param [in]  hIirTask   IIR task handle.
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully Queued the given task.
 *
 *  - #ADI_IIR_RESULT_INVALID_OPERATION       If the device is already running a task OR if there is no task assigned to the device.
 *
 * @sa          adi_iir_StartTask().
 */
ADI_IIR_RESULT adi_iir_UnassignTask (ADI_IIR_DEV_HANDLE hIirHandle)
{

    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TASK_INFO *pTask;
	ADI_IIR_DEV_STATE eDevState;
    ADI_IIR_DEV_INFO *pDevice;

#if defined (ADI_DEBUG)
    /* Validate the Device handle */
    assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);
#endif

    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    IIR_MUTEX_LOCK(&gIirMutex);

    /* Get the Device Handle */
    pDevice = (ADI_IIR_DEV_INFO*)hIirHandle;

    /* Get the Device State */
    eDevState = pDevice->eDevState;

    /* Get the task handle which is assigned to the device */
    pTask = pDevice->pQueue->pActiveTask;

    /* Return ADI_RESULT_INVALID_OPERATION if there is no task assigned to the device OR if the device is not opened OR if there is already some task running on the device */
    if((pTask == NULL) || (eDevState != ADI_IIR_DEV_STATE_IDLE))
    {
    	eResult = ADI_IIR_RESULT_INVALID_OPERATION;
    }
    else
    {
    	/* Reset Accelerator */
    	ResetAccelerator(pDevice);

        /* Make the ActiveTask pointer as NULL */
        pDevice->pQueue->pActiveTask = NULL;

    	/* Update the task state */
        pTask->eTaskState = ADI_IIR_TASK_STATE_NONE;

    }

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);
    /* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    /* Return the result */
    return (eResult);
}
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 1) */

/*!
 * @brief       adi_iir_DeleteTask API deletes the given Task.
 *
 * @details     When a task is created, Driver keeps its information intact even after IIR accelerator has completed it so that application can reschedule it by just queuing it again.
 *              adi_iir_DeleteTask API deletes the given Task and application will not be able to reuse it afterwards.
 *              Also if the task is in queue and its state is not running, delete task removes the task from the accelerator task queue.
 *              If the task state is running, ADI_IIR_RESULT_TASK_STATE_RUNNING will be returned and the task will not be aborted.
 *              This API is only available in Legacy Mode of Operation.
 *              In Non-Queuing Mode, if the task is assigned to accelerator or running, the API returns  ADI_IIR_RESULT_INVALID_OPERATION.
 *
 * @param [in]  hIirTask   IIR task handle.
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully deleted the given Task.
 *
 *  - #ADI_IIR_RESULT_TASK_STATE_RUNNING      If the Task State is Running.
 *
 * @sa          adi_iir_QueueTask()
 */
ADI_IIR_RESULT adi_iir_DeleteTask (ADI_IIR_TASK_HANDLE hIirTask)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TASK_INFO *pTask = (ADI_IIR_TASK_INFO*) hIirTask;
    ADI_IIR_TASK_INFO *pPrevTask;
	ADI_IIR_TASK_STATE eTaskState;
	ADI_IIR_DEV_INFO * pDevice;

#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Enter Critical Region */
    adi_osal_EnterCriticalRegion();

    /* Lock the Mutex */
    IIR_MUTEX_LOCK(&gIirMutex);

    /* Get the Task Handle */
    pTask = (ADI_IIR_TASK_INFO*) hIirTask;

    /* Get the Task Status */
    eTaskState = pTask->eTaskState;

#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
    if(pTask->eTaskState == ADI_IIR_TASK_STATE_QUEUED)
    {

        pDevice = (ADI_IIR_DEV_INFO *)pTask->pDevice;
			
		/* Find the previous Task in the Queue and update Queue */
        pPrevTask = pDevice->pQueue->pActiveTask;
        while(pPrevTask->pNext != pTask)
        {
        	pPrevTask = pPrevTask->pNext;
        }

        pPrevTask->pNext = pTask->pNext;

        /* Reset the Task Structure of the Task to be deleted and Update Task Status */
        ResetTaskStructure(pTask);

    }
    else if(pTask->eTaskState == ADI_IIR_TASK_STATE_RUNNING)
    {
        /* Write the Result */
       	eResult = ADI_IIR_RESULT_TASK_STATE_RUNNING;
    }
    else
    {
        /* Means eTaskState == ADI_IIR_TASK_STATE_NONE || eTaskState == ADI_IIR_TASK_STATE_COMPLETED */

        /* Reset the Task Structure */
        ResetTaskStructure(pTask);
    }
#else

   if(pTask->eTaskState != ADI_IIR_TASK_STATE_NONE)
   {
	   eResult = ADI_IIR_RESULT_INVALID_OPERATION;
   }
   else
   {

       /* Reset the Task Structure */
       ResetTaskStructure(pTask);
   }
#endif /* #if (ADI_IIR_CFG_NOQUEUE_MODE == 0) */

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);
	/* Exit Critical Region */
    adi_osal_ExitCriticalRegion();

    return (eResult);
}

/*!
 * @brief       The adi_iir_GetIirTaskStatus API will return the status of given task.
 *
 * @param[in]   hIirTask      IIR task handle
 *
 * @param[out]  TaskStatus   Address to store the task status
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully returned the status of the given Task in the given location.
 *
 */
ADI_IIR_RESULT adi_iir_GetIirTaskStatus (ADI_IIR_TASK_HANDLE hIirTask, ADI_IIR_TASK_STATE *TaskStatus)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;

    ADI_IIR_TASK_INFO* pTask = (ADI_IIR_TASK_INFO*) hIirTask;

#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
    /* Check if the address passed is NULL */
    assert(TaskStatus != NULL);
#endif

    /* Lock the Mutex */
	IIR_MUTEX_LOCK(&gIirMutex);

    /* Get the Task structure */
    pTask = (ADI_IIR_TASK_INFO*) hIirTask;

    /* Write the Task Status to the given location */
    *TaskStatus = pTask->eTaskState;

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);

    /* Return status */
    return (eResult);
}

/*!
 * @brief       The adi_iir_RegisterCallback API registers an application defined Callback.
 *
 * @details     In legacy Mode, the callback occurs after completion of each task and the event passed in the callback is ADI_IIR_EVENT_ALL_CHANNEL_DONE.
 *              In Auto Configuration Mode, the callback can be selected for each channel of a task using the bCallbackEnable
 *              parameter of the ADI_IIR_CHANNEL_INFO.In Auto Configuration Mode,ADI_IIR_EVENT_CHANNEL_DONE event is passed in Callback.
 *              In Legacy Mode, the Callback argument passed by the driver to the application(pArg) will be the task handle of the completed task.
 *              In Auto Configuration Mode, the callback argument passed by the driver to the application(pArg) will be the task handle to which the completed channel belongs to.
 *
 * @param[in]   hIirHandle     Channel Handle
 *
 * @param[in]   pfCallback     Application Supplied Callback function which is called to Notify Task as Completed.
 *
 * @param[in]   pCBParam       Application Supplied Callback parameter which will be passed back in the Callback Function.
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS  If successfully registered the application defined Callback function.
 *
 */
ADI_IIR_RESULT adi_iir_RegisterCallback (ADI_IIR_DEV_HANDLE hIirHandle,
                                         ADI_CALLBACK       pfCallback,
		                                 void *             pCBParam)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
    assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Get the Device Handle */
	pDevice = (ADI_IIR_DEV_INFO*) hIirHandle;

	/* Register the Callback function and Callback parameter */
        pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX] = pfCallback;
        pDevice->pIIRCBParam[ADI_IIR_MASTER_CALLBACK_INDEX]   = pCBParam;

    /* Return status */
    return (eResult);
}


/*!
 * @brief       The adi_iir_FloatingPointEnable API configures the output Floating Point Rounding Mode for the the given task.
 *
 * @details     The adi_iir_FloatingPointEnable API configures the output Floating Point Rounding Mode for the the given task.
 *              If the Task State is running, the API returns  ADI_IIR_RESULT_INVALID_STATE.This API is only available in Legacy Mode.
 *              In Auto Configuration Mode, Output Rounding Mode is customizable for each channel using the eRoundingMode field of
 *              ADI_IIR_CHANNEL_INFO.
 *              In Non-Queuing Mode, if the task is assigned to accelerator or running, the API returns  ADI_IIR_RESULT_INVALID_OPERATION.
 *
 * @param [in]  hIirTask         IIR task handle.
 *
 * @param [in]  eRoundingMode    Output Rounding Mode.
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully configured the Output Rounding Mode.
 *
 *  - #ADI_IIR_RESULT_INVALID_STATE           If Task State is running.
 *
 * @sa          adi_iir_SaveStateEnable().
 */
ADI_IIR_RESULT adi_iir_FloatingPointEnable (ADI_IIR_TASK_HANDLE         hIirTask,
                                            ADI_IIR_FLOAT_ROUNDING_MODE eRoundingMode)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TASK_INFO *pTask;
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Get the device handle */
	pTask = (ADI_IIR_TASK_INFO*)hIirTask;

#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
	if(pTask->eTaskState == ADI_IIR_TASK_STATE_RUNNING)
	{
		eResult = ADI_IIR_RESULT_INVALID_STATE;
	}
	else
	{
		pTask->nIIRCTL1 |= ((uint32_t)eRoundingMode) << BITP_IIR_CTL1_RND;
	}
#else
	if(pTask->eTaskState != ADI_IIR_TASK_STATE_NONE)
	{
		eResult = ADI_IIR_RESULT_INVALID_STATE;
	}
	else
	{
		pTask->nIIRCTL1 |= ((uint32_t)eRoundingMode) << BITP_IIR_CTL1_RND;
	}
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 0) */

	/* Return the status */
	return (eResult);
}


/*!
 * @brief       The adi_iir_SaveStateEnable API enables/disables the Save Biquad State feature for the given Task.
 *
 * @details     The adi_iir_SaveStateEnable API enables/disables the Save Biquad State feature for the given Task.
 *              If the Task State is running, the API returns  ADI_IIR_RESULT_INVALID_STATE.
 *              This API is only available in Legacy Mode. In Auto Configuration Mode, Save State enable/disable is
 *              customizable for each channel of a task using the bSaveBiquadState field of ADI_IIR_CHANNEL_INFO.
 *              In Non-Queuing Mode, we can enable/disable save state of the task only when it is not assigned to accelerator.
 *              In Non-Queuing Mode, if the task is assigned to accelerator or running, the API returns  ADI_IIR_RESULT_INVALID_OPERATION.
 *
 * @param [in]  hIirTask         IIR task handle.
 *
 * @param [in]  bFlag            True  - Enable Save State.
 *                               False - Disable Save State.
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully disabled/enabled Save State for a given Task.
 *
 *  - #ADI_IIR_RESULT_INVALID_STATE           If Task State is running.
 *
 * @sa          adi_iir_FloatingPointEnable().
 */
ADI_IIR_RESULT adi_iir_SaveStateEnable (ADI_IIR_TASK_HANDLE        hIirTask,
		                                const bool                  bFlag)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
    ADI_IIR_TASK_INFO *pTask;
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Get the Task handle */
    pTask = (ADI_IIR_TASK_INFO*)hIirTask;

#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
	if(pTask->eTaskState == ADI_IIR_TASK_STATE_RUNNING)
    {
       	eResult = ADI_IIR_RESULT_INVALID_STATE;
    }
    else
    {
    	if(bFlag)
    	{
    		 pTask->nIIRCTL1 |= BITM_IIR_CTL1_SS;
    	}
    	else
    	{
    		pTask->nIIRCTL1 &= ~BITM_IIR_CTL1_SS;
    	}

    }
#else
	if(pTask->eTaskState != ADI_IIR_TASK_STATE_NONE)
    {
       	eResult = ADI_IIR_RESULT_INVALID_STATE;
    }
    else
    {
    	if(bFlag)
    	{
    		 pTask->nIIRCTL1 |= BITM_IIR_CTL1_SS;
    	}
    	else
    	{
    		pTask->nIIRCTL1 &= ~BITM_IIR_CTL1_SS;
    	}

    }
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 0) */
    /* Return the status */
    return (eResult);
}
/*!
 * @brief       The adi_iir_FortyBitSelect API selects between 32-bit IEEE floating-point format or
 *              40-bit IEEE floating-point format .
 *
 * @details     The adi_iir_FortyBitSelect API selects between 32-bit IEEE floating-point format or
 *              40-bit IEEE floating-point format.This API is only available in Legacy Mode.
 *              In Auto Configuration Mode,Forty Bit Select Enable/Disable is customizable for each channel of a task using the bFortyBitSelect
 *              field of ADI_IIR_CHANNEL_INFO.
 *              If the Task State is running, the API returns  ADI_IIR_RESULT_INVALID_STATE.
 *              In Non-Queuing Mode, if the task is assigned to accelerator or running, the API returns  ADI_IIR_RESULT_INVALID_OPERATION.
 *
 * @param [in]  hIirTask         IIR task handle.
 *
 * @param [in]  bFlag            True  - 40-bit IEEE floating-point format.
 *                               False - 32-bit IEEE floating-point format.
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully configured the Input format.
 *
 *  - #ADI_IIR_RESULT_INVALID_STATE           If Task State is running.
 *
 * @sa          adi_iir_FloatingPointEnable().
 */
ADI_IIR_RESULT adi_iir_FortyBitSelect (ADI_IIR_TASK_HANDLE        hIirTask,
		                                const bool                  bFlag)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
    ADI_IIR_TASK_INFO *pTask;
#if defined (ADI_DEBUG)
    /* Validate the Task handle */
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
#endif
    /* Get the Task handle */
    pTask = (ADI_IIR_TASK_INFO*)hIirTask;

#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
	if(pTask->eTaskState == ADI_IIR_TASK_STATE_RUNNING)
    {
       	eResult = ADI_IIR_RESULT_INVALID_STATE;
    }
    else
    {
    	if(bFlag)
    	{
    		 pTask->nIIRCTL1 |= BITM_IIR_CTL1_FORTYBIT;
    	}
    	else
    	{
    		pTask->nIIRCTL1 &= ~BITM_IIR_CTL1_FORTYBIT;
    	}

    }
#else
	if(pTask->eTaskState != ADI_IIR_TASK_STATE_NONE)
    {
       	eResult = ADI_IIR_RESULT_INVALID_STATE;
    }
    else
    {
    	if(bFlag)
    	{
    		 pTask->nIIRCTL1 |= BITM_IIR_CTL1_FORTYBIT;
    	}
    	else
    	{
    		pTask->nIIRCTL1 &= ~BITM_IIR_CTL1_FORTYBIT;
    	}

    }
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 0) */

    /* Return the status */
    return (eResult);
}

/*!
 * @brief       The adi_iir_UpdateTask API updates the Circular Buffer Paramters of the task.
 *
 * @details     The adi_iir_UpdateTask API updates the Circular Buffer Parameters of the task.When a task has to be reused , the Input and Output circular Buffer Parameters can be updated before Queuing the task again.
 *              Note that in Auto Configuration Mode, this API must be called before reusing the task as the hardware doesn't update the Task Buffer Indexes(only in ADSP-21569). In legacy Mode, this API is optional before reusing a task as the Task Buffer Indexes are updated by the
 *              Hardware after its completion.This API can also be used to update the slewing parameters of the task. Note that the task should not be assigned to the accelerator or running while updating the slewing parameters.
 *              If the Task State is running or Queued, the API returns  ADI_IIR_RESULT_INVALID_OPERATION.
 *              In No-Queuing Mode, if the task is assigned to accelerator or running, the API returns  ADI_IIR_RESULT_INVALID_OPERATION.
 *
 * @param [in]  hIirTask               IIR task handle.
 *
 * @param [in]  pChannelParamsList     Array of Channel Buffer Parameters.
 *
 * @param [in]  nNumChannels           Number of Channels of the task.
 *
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully configured the Input format.
 *
 *  - #ADI_IIR_RESULT_INVALID_OPERATION           If Task State is not ADI_IIR_TASK_STATE_COMPLETED.
 *
 * @sa          adi_iir_FloatingPointEnable().
 */
ADI_IIR_RESULT adi_iir_UpdateTask(ADI_IIR_TASK_HANDLE             hIirTask,
		                          ADI_IIR_CHANNEL_BUFFER_INFO     *pChannelParamsList,
							      uint32_t                        nNumChannels)
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_TASK_INFO *pTask = NULL;
    ADI_IIR_TCB_INFO *pCurrentTCB = NULL;
    uint32_t count = 0u;
#if (ADI_IIR_CFG_SLEW_EN == 1)
    uint32_t count1;
    uint32_t IirCtl2;
    uint32_t nWindowSize;
#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */

#if defined (ADI_DEBUG)
    /* Validate the Task handle*/
    assert(ValidateTaskHandle(hIirTask) == ADI_IIR_RESULT_SUCCESS);
    /* Check whether pointers passed are not NULL */
    assert((pChannelParamsList != NULL));

#if (ADI_IIR_CFG_SLEW_EN == 1)

    pTask = (ADI_IIR_TASK_INFO*) hIirTask;

	/* Intitalize the TCB pointer */
	pCurrentTCB = pTask->pTCBChain;

    for(count1 = 0u; count1< nNumChannels; count1++)
    {
		/* Read the IirCtl2 from TCB */
		IirCtl2 = pCurrentTCB->nIirControl2;

		/* Extract the window size from IirCtl2 */
		nWindowSize = ((IirCtl2 & BITM_IIR_CTL2_WINDOW) >> BITP_IIR_CTL2_WINDOW) + 1u;

        /* Check if bits [31:24] are zero */
        assert(((pChannelParamsList[count1].nNn) & 0xFF000000) == 0x00000000);

        /* Check if slewing frequency is a multiple of window size in linear alpha mode */
        if(pChannelParamsList[count1].bLinearAlpha == true)
        {
        	assert(((pChannelParamsList[count1].nNn) % (nWindowSize)) == 0x00000000);
        }

        /* Check if window size is a multiple of  slewing frequency in constant alpha mode */
        if(pChannelParamsList[count1].bLinearAlpha == false)
        {
        	assert(((nWindowSize) % ((pChannelParamsList[count1].nNn) & BITM_IIR_SLEW_CNFG_SLEW_FREQ)) == 0x00000000);
        }

		/* Update the TCB Pointer */
    	pCurrentTCB = pCurrentTCB + 1;

    }

#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */

#endif
    pTask = (ADI_IIR_TASK_INFO*) hIirTask;

#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)

    if(((pTask->nNumChannels) != nNumChannels) || ((pTask->eTaskState) != ADI_IIR_TASK_STATE_COMPLETED))
    {
    	eResult = ADI_IIR_RESULT_INVALID_OPERATION;
    }
    else
    {
    	/* Intitalize the TCB pointer */
    	pCurrentTCB = pTask->pTCBChain;

    	/* Update the Circular Buffer Parameters in a loop */
    	for(count = 0u; count< nNumChannels; count++)
    	{
    		pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffBase);
    		pCurrentTCB->nOutputLength  = (uint32_t)pChannelParamsList[count].nOutputBuffLength;
    		pCurrentTCB->nOutputModify  = (int32_t)pChannelParamsList[count].nOutputBuffModify;
    		pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffIndex);

    		pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffBase);
    		pCurrentTCB->nInputLength   = (uint32_t)pChannelParamsList[count].nInputBuffLength;
    		pCurrentTCB->nInputModify   = (int32_t)pChannelParamsList[count].nInputBuffModify;
    		pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffIndex);

    		/* Update the TCB Pointer */
    		pCurrentTCB = pCurrentTCB + 1;
    	}

    }
#else
    if(((pTask->nNumChannels) != nNumChannels) || ((pTask->eTaskState) != ADI_IIR_TASK_STATE_NONE))
    {
    	eResult = ADI_IIR_RESULT_INVALID_OPERATION;
    }
    else
    {
    	/* Intitalize the TCB pointer */
    	pCurrentTCB = pTask->pTCBChain;

    	/* Update the Circular Buffer Parameters in a loop */
    	for(count = 0u; count< nNumChannels; count++)
    	{
#if (ADI_IIR_CFG_SLEW_EN == 1)

    		/* Read the IirCtl2 from TCB */
    		IirCtl2 = pCurrentTCB->nIirControl2;

            if(pChannelParamsList[count].bLinearAlpha == true)
            {
        	    IirCtl2 |= BITM_IIR_CTL2_SLEW_LALPHA_MODE;
            }

            if(pChannelParamsList[count].bLinearAlpha == false)
            {
    	       if(pChannelParamsList[count].bSaturationEnable == true)
    	       {
                   IirCtl2 |= BITM_IIR_CTL2_SLEW_SAT_EN;
    	       }
            }

            /* Write back the IirCtl2 to TCB */
            pCurrentTCB->nIirControl2 = IirCtl2;

#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */
    		pCurrentTCB->pOutputBase    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffBase);
    		pCurrentTCB->nOutputLength  = (uint32_t)pChannelParamsList[count].nOutputBuffLength;
    		pCurrentTCB->nOutputModify  = (int32_t)pChannelParamsList[count].nOutputBuffModify;
    		pCurrentTCB->pOutputIndex   = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pOutputBuffIndex);

    		pCurrentTCB->pInputBase     = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffBase);
    		pCurrentTCB->nInputLength   = (uint32_t)pChannelParamsList[count].nInputBuffLength;
    		pCurrentTCB->nInputModify   = (int32_t)pChannelParamsList[count].nInputBuffModify;
    		pCurrentTCB->pInputIndex    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pInputBuffIndex);

#if (ADI_IIR_CFG_SLEW_EN == 1)
           /* Update the slewing parameters */
           pCurrentTCB->pTargetCoeffIndex    = (void *)TranslateAddr((uint32_t)pChannelParamsList[count].pTargetCoefficientIndex);
           pCurrentTCB->nAlpha               = (uint32_t)pChannelParamsList[count].nAlpha;

           /* Note that actual programmed value of 'n' should be 'n-1' */
           pCurrentTCB->nNn                  = ((uint32_t)pChannelParamsList[count].nNn - 1u);
#endif /* (ADI_IIR_CFG_SLEW_EN == 1) */

    		/* Update the TCB Pointer */
    		pCurrentTCB = pCurrentTCB + 1;
    	}

    }
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 0) */

    /* Return Result */
    return (eResult);

}

#if (defined(__ADSPSHARCFX__))
/*!
 * @brief       The adi_iir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct.
 *
 * @details     The adi_iir_ConfigSecInt API configures the interrupt to the accelerator as direct or non-direct.
 *
 * @param [in]  hIirHandle         IIR Device handle.
 *
 * @param [in]  bSecInt    True - Non-direct Interrupt; False - Direct Interrupt
 * @return      Status
 *
 *  - #ADI_IIR_RESULT_SUCCESS                 If successfully configured the Output Rounding Mode.
 *
 *  - #ADI_IIR_RESULT_FAILED           If Task State is running.
 *
 * @sa          adi_iir_FixedPointEnable().
 */
ADI_IIR_RESULT adi_iir_ConfigSecInt (ADI_IIR_DEV_HANDLE     hIirHandle,
                                                           bool bSecInt )
{
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_SUCCESS;
    ADI_IIR_DEV_INFO *pDevice;
#if defined (ADI_DEBUG)
    /* Validate the device handle */
     assert(ValidateDeviceHandle(hIirHandle) == ADI_IIR_RESULT_SUCCESS);
#endif
     uint32_t sid, cid, iid;

     ADI_INT_ID_STATUS res;

     /* Get the Device Handle */
 	pDevice = (ADI_IIR_DEV_INFO*) hIirHandle;

     iid = pDevice->nIirDoneIntId;

     if( ADI_IID_IS_CORE_INT(iid))
     {
    	 cid = iid;
    	 res = get_sid(iid, &sid);
     }
	 else
	 {
         sid = iid;
         res = get_cid(iid, &cid);
	 }

     if (res == ADI_INT_ID_SUCCESS)
     {
    	 if( bSecInt == true)
    	 {
    		 /* Install Non-Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(sid, IirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_IIR_RESULT_FAILED;
    		 }

    	 }
    	 else
    	 {
    		 /* Install Direct_Interrupt Handler */
    		 if(adi_int_ConfigInt(cid, IirInterruptHandler, pDevice) != ADI_INT_SUCCESS)
    		 {
    			 return ADI_IIR_RESULT_FAILED;
    		 }
    	 }
     }
     else
     {
    	 return ADI_IIR_RESULT_FAILED;
     }

     return ADI_IIR_RESULT_SUCCESS;
}
#endif /* defined(__ADSPSHARCFX__) */

/* IirInterrupt Handler */
static void IirInterruptHandler (uint32_t iid, void* pParam)
{
#if (ADI_IIR_CFG_NOQUEUE_MODE == 0)
	ADI_IIR_DEV_INFO *pDevice = (ADI_IIR_DEV_INFO*)pParam;
	ADI_IIR_TASK_INFO *pTask;
	volatile uint32_t nTimeout;
    uint32_t nDMAStat;
    ADI_IIR_TASK_INFO * pTemp;
    ADI_SEC_RESULT eResult;

/*-------------------FOR_DEBUG--------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
    eCoreID = adi_core_id();
    if(eCoreID == ADI_CORE_SHARC0)
    {
    	pDevice->pQueue->pIirDebugData->IirIntCnt[0]++;
    }
#if __NUM_SHARC_CORES__ > 1
    else if(eCoreID == ADI_CORE_SHARC1)
    {
    	pDevice->pQueue->pIirDebugData->IirIntCnt[1]++;
    }
#endif /* __NUM_SHARC_CORES__ > 1 */
    else
    {
        /* MISRA Compliance */
    }
#endif /* ADI_USE_DEBUG_DATA */
/*------------------FOR_DEBUG----------------------------------*/

    /* Lock the Mutex */
	IIR_MUTEX_LOCK(&gIirMutex);

	/* Get the Device handle */
    pDevice = (ADI_IIR_DEV_INFO*)pParam;

    /* Get the Handle of the Task that is completed */
    pTask = pDevice->pQueue->pActiveTask;
    
    /* Get DMA Status. Clear All Channels Done Interrupt Status */
    nDMAStat = (pDevice->pIIRRegs->DMASTAT);

    /* Mark the Task as Complete */
    pTask->eTaskState = ADI_IIR_TASK_STATE_COMPLETED;

    /* Copy the Task Handle in TaskCompleted field of Queue Info */
    pDevice->pQueue->pTaskCompleted = pTask;

    /* Reset Accelerator */
    ResetAccelerator(pDevice);

    /* If there are no pending Tasks,Make pActiveTask = pLastTaskAdded = NULL, Device State = Idle */

    if(pTask->pNext == NULL)
    {
        pDevice->pQueue->pActiveTask = NULL;
        pDevice->pQueue->pLastTaskAdded = NULL;
        pDevice->eDevState = ADI_IIR_DEV_STATE_IDLE;
    }
    else
    {
    	/* Update Queue */
    	pDevice->pQueue->pActiveTask = pTask->pNext;

        /* Start Queue */
        StartQueue(pDevice);

    	/* Update Task State */
        pDevice->pQueue->pActiveTask->eTaskState = ADI_IIR_TASK_STATE_RUNNING;
		
        /* Dequeue the Task that is completed */
		pTask->pNext = NULL;
    }

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);

    /* Call the Application defined Callback depending on the Driver operating Mode */
	{
		/* Call the Master Callback(Dual Core Mode). In single Core Mode, there is only One Callback.*/
        if(pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX] != NULL)
        {
            pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX](pDevice->pIIRCBParam[ADI_IIR_MASTER_CALLBACK_INDEX],(uint32_t)ADI_IIR_EVENT_ALL_CHANNEL_DONE,(void *)pTask);
        }

	}
#else
	ADI_IIR_DEV_INFO *pDevice = (ADI_IIR_DEV_INFO*)pParam;
	ADI_IIR_TASK_INFO *pTask;
	volatile uint32_t nTimeout;
    uint32_t nDMAStat;
    ADI_IIR_TASK_INFO * pTemp;
    ADI_SEC_RESULT eResult;

/*-------------------FOR_DEBUG--------------------------------*/
#if defined(ADI_USE_DEBUG_DATA)
    ADI_CORE_ID eCoreID;
    eCoreID = adi_core_id();
    if(eCoreID == ADI_CORE_SHARC0)
    {
    	pDevice->pQueue->pIirDebugData->IirIntCnt[0]++;
    }
#if __NUM_SHARC_CORES__ > 1
    else if(eCoreID == ADI_CORE_SHARC1)
    {
    	pDevice->pQueue->pIirDebugData->IirIntCnt[1]++;
    }
#endif /* __NUM_SHARC_CORES__ > 1 */
    else
    {
        /* MISRA Compliance */
    }
#endif /* ADI_USE_DEBUG_DATA */
/*------------------FOR_DEBUG----------------------------------*/

    /* Lock the Mutex */
	IIR_MUTEX_LOCK(&gIirMutex);

	/* Get the Device handle */
    pDevice = (ADI_IIR_DEV_INFO*)pParam;

    /* Get the Handle of the Task that is completed */
    pTask = pDevice->pQueue->pActiveTask;

    /* Get DMA Status. Clear All Channels Done Interrupt Status */
    nDMAStat = (pDevice->pIIRRegs->DMASTAT);

    /* Only Clear IIR_CTL1.DMAEN bit */
    pDevice->pIIRRegs->CTL1 &= ~(IIR_IIRCTL1_DMAEN_ENABLE);

    /* Move the device state to IDLE */
    pDevice->eDevState = ADI_IIR_DEV_STATE_IDLE;

    /* Mark the Task back to Assigned from Running */
    pTask->eTaskState = ADI_IIR_TASK_STATE_ASSIGNED;

    /* Copy the Task Handle in TaskCompleted field of Queue Info */
    pDevice->pQueue->pTaskCompleted = pTask;

    /* Unlock the Mutex */
    IIR_MUTEX_UNLOCK(&gIirMutex);

    /* Call the Application defined Callback depending on the Driver operating Mode */
	{
		/* Call the Master Callback(Dual Core Mode). In single Core Mode, there is only One Callback.*/
        if(pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX] != NULL)
        {
            pDevice->pfIIRCallback[ADI_IIR_MASTER_CALLBACK_INDEX](pDevice->pIIRCBParam[ADI_IIR_MASTER_CALLBACK_INDEX],(uint32_t)ADI_IIR_EVENT_ALL_CHANNEL_DONE,(void *)pTask);
        }

	}
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 0) */
}

/* Validate the Device Handle */
static ADI_IIR_RESULT ValidateDeviceHandle (ADI_IIR_DEV_HANDLE hDevice)
{
    unsigned int x;
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_INVALID_HANDLE;
    ADI_IIR_DEV_INFO* pDevice = (ADI_IIR_DEV_INFO*)hDevice;

    for (x = 0u; x < (unsigned int)IIR_NUM_DEVICES; x++)
    {
        if (pDevice == &gIirDeviceInfo[x])
        {

            eResult = ADI_IIR_RESULT_SUCCESS;

        }
    }

    return (eResult);
}

/* Validate the Task Handle */
static ADI_IIR_RESULT ValidateTaskHandle(ADI_IIR_TASK_HANDLE hTask)
{
    unsigned int x;
	ADI_IIR_TASK_INFO *pTask = (ADI_IIR_TASK_INFO *)hTask;
    ADI_IIR_RESULT eResult = ADI_IIR_RESULT_INVALID_HANDLE;
    for (x = 0u; x < (unsigned int)IIR_NUM_DEVICES; x++)
    {
        if (pTask->pDevice == (ADI_IIR_DEV_HANDLE)&gIirDeviceInfo[x])
        {

            eResult = ADI_IIR_RESULT_SUCCESS;

        }
    }
    return (eResult);
}

/* Translates the supplied memory address for DMA operation based on Core ID. */
static uint32_t TranslateAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */
#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_internal_to_system_addr(MemAddr, 1);
#endif

    return (MemAddr >> 2u);
}

/*
 * Reverts the supplied memory address for DMA operation based on Core ID.
 *
 * Parameters
 *  - MemAddr   Memory address to work on
 *
 * Returns
 *  - Translated Memory address for DMA access.
 *
 */
static uint32_t RevertTranslatedAddr (uint32_t  MemAddr)
{
    /* The TCB addresses have been divided by 4. Adjust them so the common TranslateAddr routine works */

    MemAddr = MemAddr << 2u;

#if (defined(__ADSPSHARC__) && (__ADSPSHARC__ >= 0x200)) || (defined(__ADSPSHARCFX__))
    MemAddr = adi_rtl_system_to_internal_addr(MemAddr);
#endif

    return MemAddr;
}

/* Starts the Queue Processing by writing to Chain Pointer and Control Register */
static ADI_IIR_RESULT StartQueue (ADI_IIR_DEV_INFO *  pDevice)
{
    /* Schedule Next Task */
	/* Start Accelerator */
	(pDevice->pIIRRegs->CHNPTR) = (uint32_t)TranslateAddr((uint32_t)(&pDevice->pQueue->pActiveTask->pTCBChain->nIirControl2));
	(pDevice->pIIRRegs->CTL1) = pDevice->pQueue->pActiveTask->nIIRCTL1 | IIR_IIRCTL1_EN_ENABLE | IIR_IIRCTL1_DMAEN_ENABLE;
	return ADI_IIR_RESULT_SUCCESS;
}

/* Resets the Accelerator Chain Pointer and Global Control Registers */
static ADI_IIR_RESULT ResetAccelerator(ADI_IIR_DEV_INFO *  pDevice)
{

	pDevice->pIIRRegs->CTL1 = 0u;
	pDevice->pIIRRegs->CHNPTR = 0u;

	return ADI_IIR_RESULT_SUCCESS;
}

/* Resets the given Task Structure */
static ADI_IIR_RESULT ResetTaskStructure (ADI_IIR_TASK_INFO *  pTask)
{

	pTask->pDevice = NULL;
	pTask->pNext = NULL;
	pTask->pTCBChain = NULL;
	pTask->nIIRCTL1 = 0u;
    pTask->nNumChannels = 0u;

    if(pTask->eTaskState != ADI_IIR_TASK_STATE_NONE)
    {
    	pTask->eTaskState  = ADI_IIR_TASK_STATE_NONE;
    }

    return ADI_IIR_RESULT_SUCCESS;
}

#endif /*__ADI_IIR_LEGACY_V3_C__*/
