/*!
 *****************************************************************************
   @file:    adi_iir_config_SC8xx.h
   @brief:   Configuration options for IIR Driver.
  -----------------------------------------------------------------------------

 Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

 This software is proprietary.  By using this software you agree
 to the terms of the associated Analog Devices License Agreement.

*****************************************************************************/
/** @addtogroup IIR_Driver IIR Accelerator Driver
 *  @{
 */
 
 /** @addtogroup IIR_Driver_Static_Configuration IIR Accelerator Driver Static Configuration
 *  @{
 */
 
#ifndef ADI_IIR_CONFIG_SC8XX_H
#define ADI_IIR_CONFIG_SC8XX_H


/*! This macro enables/disables cache management by the driver.\n If the application
 *  wants to manage cache, this macro has to be set to 0.
*/
#define ADI_CACHE_MANAGEMENT      (0)

/*! Legacy Mode                      */
#define ADI_IIR_ACCELERATOR_MODE_LEGACY                                     (0u)
/*! Auto Configuration Mode          */
#define ADI_IIR_ACCELERATOR_MODE_ACM                                        (1u)


/*! This Macro allows to set the Accelerator Mode for the Tasks created.\n
    It can be configured to one of the following macros:
    - #ADI_IIR_ACCELERATOR_MODE_LEGACY
    - #ADI_IIR_ACCELERATOR_MODE_ACM
 */
#define ADI_IIR_CFG_ACCELERATOR_MODE    (ADI_IIR_ACCELERATOR_MODE_LEGACY)

/*! @cond PRIVATE */
#if  ADI_IIR_CFG_ACCELERATOR_MODE == ADI_IIR_ACCELERATOR_MODE_LEGACY
/*! @endcond */

/*! This Macro allows to set the driver mode of operation.\n
    It can be configured to one of the following macros:
    (0) -  Driver operates in Queuing Mode\n
    (1) -  Driver operates in Non-Queuing(Single Task) Mode\n
    In Non-Queuing Mode, a single task can be assigned to the accelerator for processing. The same task can be repeated for multiple iterations without
    reloading the coefficients for each iteration. Note that save state can be enabled or disabled as required.
 */
#define ADI_IIR_CFG_NOQUEUE_MODE    (0u)

/*! This Macro allows to enable disable coefficient slewing.\n
    It can be configured to one of the following macros:
    (0) -  Disable Coefficient Slewing\n
    (1) -  Enable Coefficient Slewing\n
    Note that in ADSP-SC83x, slewing is only enabled in Non Queuing Mode. Since after TCB processing, the slewed coefficients are not saved back to
    target coefficient memory when save state is enabled.
 */
#define ADI_IIR_CFG_SLEW_EN    (0)


/*! IEEE round to nearest (even)     */
#define ADI_IIR_RNDG_MODE_NEAREST_EVEN                                      (0u)

/*! Round to zero             */
#define ADI_IIR_RNDG_MODE_TO_ZERO                                           (1u)

/*! This Macro allows to set the Output Rounding Mode for the Tasks created.\n
    This macro only has effect in Legacy mode.\n
	In Auto Configuration Mode, the Output Rounding Mode can be set as part of ADI_IIR_CHANNEL_INFO.\n
    It can be configured to one of the following macros:
    - #ADI_IIR_RNDG_MODE_NEAREST_EVEN
    - #ADI_IIR_RNDG_MODE_TO_ZERO
 */
#define ADI_IIR_CFG_ROUNDING_MODE    (ADI_IIR_RNDG_MODE_NEAREST_EVEN)

/*! This macro allows to selects between 32-bit IEEE floating-point format or
    40-bit IEEE floating-point format .\n
    This macro only has effect in Legacy mode.\n
	In Auto Configuration Mode, the Forty Bit Enable/disable can be set as part of ADI_IIR_CHANNEL_INFO.\n
    It can be configured to one of the following macros:
    (1) - 40-bit IEEE floating-point\n
    (0) - 32-bit IEEE floating-point
 */
#define ADI_IIR_CFG_FORTYBIT_FLOATING_POINT_SELECT    (0u)

/*! This macro allows to enable/disables Biquad Save State feature for all the IIR Tasks created.\n
    This macro only has effect in Legacy mode.\n
	Auto Configuration Mode, the Save State Enable/disable can be set as part of ADI_IIR_CHANNEL_INFO.\n
    It can be configured to one of the following macros:
    (1) -  Enable Save Biquad State Feature\n
    (0) -  Disable Save Biquad State Feature
 */
#define ADI_IIR_CFG_SAVE_BIQUAD_STATE    (1u)

/*! @cond PRIVATE */
#endif /* ADI_IIR_CFG_ACCELERATOR_MODE == ADI_IIR_ACCELERATOR_MODE_LEGACY */
/*! @endcond */

#if defined(__ADSPSHARCFX__)
/*! This Macro configures whether the accelerator driver uses SEC interrupts or Direct Core Interrupt\n
    It can be configured to one of the following macros:
    - 0 : Direct Interrupt
    - 1 : SEC Interrupt
 */
#define ADI_IIR_CFG_SEC_INT    (1)
#endif /* defined(__ADSPSHARCFX__) */


/*
** Verify the macro configuration
*/

#if ((ADI_IIR_CFG_ACCELERATOR_MODE != ADI_IIR_ACCELERATOR_MODE_LEGACY) && \
     (ADI_IIR_CFG_ACCELERATOR_MODE != ADI_IIR_ACCELERATOR_MODE_ACM))
#error "ADI_IIR_CFG_ACCELERATOR_MODE macro is wrongly configured"
#endif /* ADI_IIR_CFG_ACCELERATOR_MODE verification */

#if  ADI_IIR_CFG_ACCELERATOR_MODE == ADI_IIR_ACCELERATOR_MODE_LEGACY

#if ((ADI_IIR_CFG_NOQUEUE_MODE != 0u)       && \
     (ADI_IIR_CFG_NOQUEUE_MODE != 1u))
#error "ADI_IIR_CFG_NOQUEUE_MODE macro wrongly configured"
#endif /* ADI_IIR_CFG_NOQUEUE_MODE verification */

#if (ADI_IIR_CFG_NOQUEUE_MODE == 1)
#if ((ADI_IIR_CFG_SLEW_EN != 0u)       && \
     (ADI_IIR_CFG_SLEW_EN != 1u))
#error "ADI_IIR_CFG_SLEW_EN macro wrongly configured"
#endif /* ADI_IIR_CFG_SLEW_EN verification */
#else
#if (ADI_IIR_CFG_SLEW_EN != 0u)
#error "ADI_IIR_CFG_SLEW_EN macro wrongly configured"
#endif /* ADI_IIR_CFG_SLEW_EN verification */
#endif /* (ADI_IIR_CFG_NOQUEUE_MODE == 1) */

#if ((ADI_IIR_CFG_ROUNDING_MODE != ADI_IIR_RNDG_MODE_NEAREST_EVEN)       && \
	 (ADI_IIR_CFG_ROUNDING_MODE != ADI_IIR_RNDG_MODE_TO_ZERO))
#error "ADI_IIR_CFG_ROUNDING_MODE macro wrongly configured"
#endif /* ADI_IIR_CFG_ROUNDING_MODE verification */

#if ((ADI_IIR_CFG_FORTYBIT_FLOATING_POINT_SELECT != 0u)       && \
     (ADI_IIR_CFG_FORTYBIT_FLOATING_POINT_SELECT != 1u))
#error "ADI_IIR_CFG_FORTYBIT_FLOATING_POINT_SELECT macro wrongly configured"
#endif /* ADI_IIR_CFG_FORTYBIT_FLOATING_POINT_SELECT verification */

#if ((ADI_IIR_CFG_SAVE_BIQUAD_STATE != 0u)       && \
     (ADI_IIR_CFG_SAVE_BIQUAD_STATE != 1u))
#error "ADI_IIR_CFG_SAVE_BIQUAD_STATE macro wrongly configured"
#endif /* ADI_IIR_CFG_SAVE_BIQUAD_STATE verification */

#endif /* ADI_IIR_CFG_ACCELERATOR_MODE == ADI_IIR_ACCELERATOR_MODE_LEGACY */


#if defined(__ADSPSHARCFX__)
#if  ((ADI_IIR_CFG_SEC_INT != 0u)   && \
     (ADI_IIR_CFG_SEC_INT != 1u))
#error "ADI_IIR_CFG_SEC_INT macro wrongly configured"
#endif /* ADI_IIR_CFG_SEC_INT verification */
#endif /* defined(__ADSPSHARCFX__) */


#endif /* ADI_IIR_CONFIG_SC8XX_H */

/*@}*/

/*@}*/