/* This file contains the code to communicate with the accelerator GUI over UART.
  */
#include "uart.h"
#include "Processing.h"

/* Source buffer for UART data */
ADI_CACHE_ALIGN static uint8_t SrcData[ADI_CACHE_ROUND_UP_SIZE(DESC_BUFFER_SIZE, uint8_t)] __attribute__ ((section (".L1.data")));

/* Destination buffer for UART data*/
ADI_CACHE_ALIGN static uint8_t DestData[ADI_CACHE_ROUND_UP_SIZE(DESC_BUFFER_SIZE, uint8_t)] __attribute__ ((section (".L1.data")));

/* UART DMA descriptor list */
ADI_CACHE_ALIGN ADI_PDMA_DESC_LIST  Src_List __attribute__ ((section (".L1.data")));
ADI_CACHE_ALIGN ADI_PDMA_DESC_LIST  Dest_List __attribute__ ((section (".L1.data")));

/* Driver memory required for UART0 */
static uint8_t UARTMemory[ADI_UART_BIDIR_MEMORY_SIZE] __attribute__ ((section (".L1.data")));

/* UART0 Handle */
ADI_UART_HANDLE hUART;

/* Function to check driver API result */
static void CheckResult(ADI_UART_RESULT Result) {
    if (Result != ADI_UART_SUCCESS) {
        REPORT_ERROR("Failed with error code 0x%08X\n", Result);
    }
}

/* Count to track the number of callBacks */
volatile static uint32_t TxCallbackCount=0,RxCallbackCount=0,CallbackCount=0;

/* Function to prepare UART descriptors */
static void PrepareDescriptors (void);

/* SPU handle */
static ADI_SPU_HANDLE hSpu;

/* Memory required for the SPU operation */
static uint8_t  SpuMemory[ADI_SPU_MEMORY_SIZE];

/* Function to initialize SPU*/
static int SpuInit(void);

/* For cycle count */
volatile int cycle_count;
volatile int clock_stop;
volatile int clock_start;
volatile int cycles;
volatile int core_cycles;
volatile int fira_cycles;
volatile int iira_cycles;

/* Flag to indicate frame work re-initialization */
volatile int FWReInit = 0;

/* Flag to indicate use case change */
volatile int ChangeUseCase = 0;

/* External flag to enable/disable bypass processing */
extern volatile bool BypassProcessing;

/* External array which holds IIR gain value */
extern float IIRGains[IIR_BANDS];

extern float IIRGainsFinal[NCH][IIR_BANDS];
extern float IIRGains1[NCH][IIR_BANDS];

/* Variable to hold value of current accelerator usage model */
ACC_USAGE_MODEL ACCUsageModel = CORE_ONLY;

/* Variable to hold value of next accelerator usage model - initiated by UART command */
ACC_USAGE_MODEL ACCUsageModelNext = CORE_ONLY;

/* Variables to hold MIPS information for different accelerator usage models */
MIPSInfo CoreOnlyMIPS			=	{200.0, 300.0, 400.0, 0.0, 0.0} ;		/* Core Only MIPS Info */
MIPSInfo DirectReplacementMIPS	=	{160.0, 100.0, 60.0, 40.0, 0.0};		/* Direct Replacement Info */
MIPSInfo PipelinedMIPS			=   {10.0,  100.0, 60.0, 190.0, 5.33};		/* Pipelined MIPS */
MIPSInfo SplitTaskMIPS			=	{110.0, 60.0, 50.0, 90.0, 0.0};			/* Split task MIPS */

/* Variable to hold IIR equalizer settings */
IIRInfo IIRInfo1;

/* UART callback */
void Callback(void *pCBParam, uint32_t Event, void *pArg);

void copychar(char* dst, char* src, int n);

char UARTTxBuffer[1] = {6};

/* One time initialization of UART */
void UartInit(void)
{
    /* UART return code */
    ADI_UART_RESULT    eUARTResult;
    ADI_UART_RESULT    eSPUResult;

    /* Tx size */
    uint32_t nTxSize = 1u;

    /* Flag which indicates whether to stop the program */
    volatile bool bStopFlag = false;

    /* Set prescaler */
    bool         Edbo = 0u;

   /* Divisor value set such that Baud rate is set to 9600 for SCLK0_0 = 125 MHz*/
   uint32_t     Divisor = 814u;

     /* Open UART */
    eUARTResult = adi_uart_Open(ADI_UART_DEVNUM, ADI_UART_DIR_BIDIRECTION, UARTMemory, ADI_UART_BIDIR_MEMORY_SIZE, &hUART);
    CheckResult(eUARTResult);

    /* Register UART Callback function for DMA mode */
    eUARTResult = adi_uart_RegisterCallback(hUART, Callback, NULL);
    CheckResult(eUARTResult);


    /* Configure UART baud rate */
    eUARTResult = adi_uart_ConfigBaudRate(hUART, Edbo, Divisor);
    CheckResult(eUARTResult);

    /* Prepare descriptors for memory copy using List mode */
    PrepareDescriptors();

    /* Read UART data to Rx buffer */
    eUARTResult = adi_uart_DMARead(hUART, &Dest_List, DMA_NUM_DESC_UART, ADI_PDMA_DESCRIPTOR_LIST);
    CheckResult(eUARTResult);

#if 0
    printf("\n\n REG_CGU0_CTL=0x%x", *pREG_CGU0_CTL);
    printf("\n\n REG_CGU0_DIV=0x%x", *pREG_CGU0_DIV);
    printf("\n\n REG_UART0_CTL=0x%x", *pREG_UART0_CTL);
    printf("\n\n REG_UART0_CLK=0x%x", *pREG_UART0_CLK);
    printf("\n\n REG_UART0_CLK=0x%x", *pREG_UART0_CLK);

#endif


//    while(1)
//    {
//    	  /* Write back the character. */
//    	eUARTResult = adi_uart_CoreWrite(hUART, &UARTTxBuffer[0], 1);
//		CheckResult(eUARTResult);
//    }
}

static void PrepareDescriptors (void)
{
	uint32_t i;

	Src_List.pStartAddr            = &SrcData[0];
	Src_List.XCount                = (DESC_BUFFER_SIZE/DMA_MSIZE_IN_BYTES);
	Src_List.XModify               = DMA_MSIZE_IN_BYTES;
	Src_List.Config                = ENUM_DMA_CFG_XCNT_INT;

	Dest_List.pStartAddr           = &DestData[0];
	Dest_List.XCount               = (DESC_BUFFER_SIZE/DMA_MSIZE_IN_BYTES);
	Dest_List.XModify              = DMA_MSIZE_IN_BYTES;
	Dest_List.Config               = ENUM_DMA_CFG_XCNT_INT;

	Src_List.pNxtDscp  = NULL;
	Dest_List.pNxtDscp = &Dest_List;
}

#include "time.h"

volatile int UartCyclesBuff[100];
volatile int UartCycles=0;
volatile int UartCyclesCnt=0;

/* UART callback function */
void Callback(void *pCBParam, uint32_t Event, void *pArg)
{
	char* wp=(char*)&IIRInfo1;
	char* rp=(char*)&DestData;
	char* tempp;
	bool skip_tx=0;

	volatile int temp = clock();

//	UartCyclesBuff[UartCyclesCnt++] = temp - UartCycles;
//
//	UartCycles = temp;
//
//	if(UartCyclesCnt==100)
//	{
//		UartCyclesCnt=0;
//	}

	int i, j;

    if(Event == (uint32_t)ADI_UART_EVENT_TX_PROCESSED)
    {
      CallbackCount++;
      TxCallbackCount++;
    }
    if(Event == (uint32_t)ADI_UART_EVENT_RX_PROCESSED)
    {
    	/* Copy received bytes equal to the size of  IIRInfo to IIRInfo1 structure */
    	for(int i=0;i<sizeof(IIRInfo);i++)
    	{
    		*wp++ = *rp++;
    	}

		CallbackCount++;
		RxCallbackCount++;

		/* Check if the command was sent to change IIR equalizer settings */
      	if(strcmp(IIRInfo1.msg, "ChangeIIR")==0)
      	{
      		/*if(IIRInfo1.BypassProcessing==0)
      			BypassProcessing = false;
      		else
      			BypassProcessing = true;*/

      		float scale;

      		for(j=0; j<5;j++)
      		{
      			scale += ((float)IIRInfo1.IIRGains[j]/10.0f);
      		}

      		scale=5.0/scale;

      		for(i=0;i<12;i++)
      		{
				for(j=0; j<5;j++)
				{
					IIRGainsFinal[i][j] = ((float)IIRInfo1.IIRGains[j]*IIRGains1[i][j]*scale)/10.0;
				}
      		}
    		skip_tx = 1;
      	}

      	else if(strcmp((char*)DestData, "Bypass Processing")==0)
      	{
      		BypassProcessing = true;
      		skip_tx = 1;

      		for(i=0;i<NCH;i++)
			{
				for(j=0;j<FIR_TAPS+1;j++)
					Input[i][j+BLOCK_SIZE]=0;

				for(j=0;j<2*NSECTIONS*IIR_BANDS;j++)
					iir_state[i][j]=0;
			}
      	}

      	else if(strcmp((char*)DestData, "Enable Processing")==0)
      	{
      		BypassProcessing = false;

     		skip_tx = 1;
      	}

      	else if(strcmp((char*)DestData, "Change Use Case, 2")==0)
		{
			ChangeUseCase = 2;
			skip_tx = 1;
		}

      	else if(strcmp((char*)DestData, "Change Use Case, 3")==0)
		{
			ChangeUseCase = 3;
			skip_tx = 1;
		}

      	/* Check if the command was to change usage model to Core Only */
      	else if(strcmp((char*)DestData, "Change usage model, Core Only")==0)
			{
      			ACCUsageModelNext = CORE_ONLY; /* Store the value of next usage model temporarily */
      			FWReInit = 1;				 /* Set the FWReInit flag */
      			strcpy((char*)SrcData, "Usage model changed, Core Only"); /* Send ACK to the PC */
			}
      	/* Check if the command was to change usage model to Direct Replacement */
      	else if(strcmp((char*)DestData, "Change usage model, Direct Replacement")==0)
			{
      			ACCUsageModelNext = DIRECT_REPLACEMENT;	/* Store the value of next usage model temporarily */
				FWReInit = 1;							/* Set the FWReInit flag */
				strcpy((char*)SrcData, "Usage model changed, Direct Replacement"); /* Send ACK to the PC */
			}
      	/* Check if the command was to change usage model to Split Task */
      	else if(strcmp((char*)DestData, "Change usage model, Split Task")==0)
		{
      		ACCUsageModelNext = SPLIT_TASK;			/* Store the value of next usage model temporarily */
			FWReInit = 1;							/* Set the FWReInit flag */
			strcpy((char*)SrcData, "Usage model changed, Split Task");/* Send ACK to the PC */
		}
      	/* Check if the command was to change usage model to Data Pipelining */
    	else if(strcmp((char*)DestData, "Change usage model, Data Pipelining")==0)
		{
    		ACCUsageModelNext = DATA_PIPELINING;		/* Store the value of next usage model temporarily */
			FWReInit = 1;								/* Set the FWReInit flag */
			strcpy((char*)SrcData, "Usage model changed, Data Pipelining");/* Send ACK to the PC */
		}
      	/* Check if the command was to send MIPS info for Core Only usage model, if yes prepare the MIPS information to be sent */
		else if(strcmp((char*)DestData, "Send MIPS Info, Core Only")==0)
			{
				tempp =(char*)&CoreOnlyMIPS.CoreMIPS; copychar((char*)SrcData, tempp, 4);
				tempp =(char*)&CoreOnlyMIPS.FIRAMIPS; copychar((char*)&(SrcData[4]), tempp, 4);
				tempp =(char*)&CoreOnlyMIPS.IIRAMIPS; copychar((char*)&(SrcData[8]), tempp, 4);
				tempp =(char*)&CoreOnlyMIPS.CoreMIPSSaving; copychar((char*)&(SrcData[12]), tempp, 4);
				tempp =(char*)&CoreOnlyMIPS.Latency; copychar((char*)&(SrcData[16]), tempp, 4);
			}
      	/* Check if the command was to send MIPS info for Direct Replacement usage model, if yes prepare the MIPS information to be sent */
		else if(strcmp((char*)DestData, "Send MIPS Info, Direct Replacement")==0)
			{
				tempp =(char*)&DirectReplacementMIPS.CoreMIPS; copychar((char*)SrcData, tempp, 4);
				tempp =(char*)&DirectReplacementMIPS.FIRAMIPS; copychar((char*)&(SrcData[4]), tempp, 4);
				tempp =(char*)&DirectReplacementMIPS.IIRAMIPS; copychar((char*)&(SrcData[8]), tempp, 4);
				tempp =(char*)&DirectReplacementMIPS.CoreMIPSSaving; copychar((char*)&(SrcData[12]), tempp, 4);
				tempp =(char*)&DirectReplacementMIPS.Latency; copychar((char*)&(SrcData[16]), tempp, 4);
			}
      	/* Check if the command was to send MIPS info for Split Task usage model, if yes prepare the MIPS information to be sent */
		else if(strcmp((char*)DestData, "Send MIPS Info, Split Task")==0)
			{
				tempp =(char*)&SplitTaskMIPS.CoreMIPS; copychar((char*)SrcData, tempp, 4);
				tempp =(char*)&SplitTaskMIPS.FIRAMIPS; copychar((char*)&(SrcData[4]), tempp, 4);
				tempp =(char*)&SplitTaskMIPS.IIRAMIPS; copychar((char*)&(SrcData[8]), tempp, 4);
				tempp =(char*)&SplitTaskMIPS.CoreMIPSSaving; copychar((char*)&(SrcData[12]), tempp, 4);
				tempp =(char*)&SplitTaskMIPS.Latency; copychar((char*)&(SrcData[16]), tempp, 4);
			}
      	/* Check if the command was to send MIPS info for Data Pipelining usage model, if yes prepare the MIPS information to be sent */
		else if(strcmp((char*)DestData, "Send MIPS Info, Data Pipelining")==0)
			{
				tempp =(char*)&PipelinedMIPS.CoreMIPS; copychar((char*)SrcData, tempp, 4);
				tempp =(char*)&PipelinedMIPS.FIRAMIPS; copychar((char*)&(SrcData[4]), tempp, 4);
				tempp =(char*)&PipelinedMIPS.IIRAMIPS; copychar((char*)&(SrcData[8]), tempp, 4);
				tempp =(char*)&PipelinedMIPS.CoreMIPSSaving; copychar((char*)&(SrcData[12]), tempp, 4);
				tempp =(char*)&PipelinedMIPS.Latency; copychar((char*)&(SrcData[16]), tempp, 4);
			}

      	/* Prepare UART TX DMA to send the ACK to the PC if needed */
      	if(skip_tx==0)
      	{
      		adi_uart_DMAWrite(hUART, &Src_List, DMA_NUM_DESC_UART, ADI_PDMA_DESCRIPTOR_LIST);
      	}
    }

    UartCyclesBuff[UartCyclesCnt++] = clock() - temp;

	if(UartCyclesCnt==100)
	{
		UartCyclesCnt=0;
	}

}

void copychar(char* dst, char* src, int n)
{
	for(int i=0;i<n;i++)
	{
		*dst++=*src++;
	}
}
