/******************************************************************************

 Copyright(c) 2022-2023 Analog Devices, Inc. All Rights Reserved.

 This software is proprietary.  By using this software you agree
 to the terms of the associated Analog Devices License Agreement.

*****************************************************************************/

/** @addtogroup FIR_Driver
 *  @{
 */
 
/** @defgroup FIR_Driver_Static_Configuration FIR Driver Static Configuration
 *  @{
 */

/**
 *  @file    adi_fir_config_SC8xx.h
 *  @brief   Configuration options for FIR Driver
 */

#ifndef ADI_FIR_CONFIG_SC8XX_H
#define ADI_FIR_CONFIG_SC8XX_H


/*! This macro enables/disables cache management by the driver.\n If the application
 *  wants to manage cache, this macro has to be set to 0.
*/
#define ADI_CACHE_MANAGEMENT      (0)

/*! Legacy Mode */
#define ADI_FIR_ACCELERATOR_MODE_LEGACY                                     (0u)
/*! Auto Configuration Mode */
#define ADI_FIR_ACCELERATOR_MODE_ACM                                        (1u)


/*! This Macro allows to set the Accelerator Mode for the Tasks created.\n
    It can be configured to one of the following macros:
    - #ADI_FIR_ACCELERATOR_MODE_LEGACY
    - #ADI_FIR_ACCELERATOR_MODE_ACM
 */
#define ADI_FIR_CFG_ACCELERATOR_MODE    (ADI_FIR_ACCELERATOR_MODE_LEGACY)

/*! @cond PRIVATE */
#if  ADI_FIR_CFG_ACCELERATOR_MODE == ADI_FIR_ACCELERATOR_MODE_LEGACY
/*! @endcond */

/*! IEEE round to nearest (even) */
#define ADI_FIR_RNDG_MODE_NEAREST_EVEN                                      (0u)

/*! Round away from zero */
#define ADI_FIR_RNDG_MODE_TO_ZERO                                           (1u)


/*! This Macro allows to set the Output Rounding Mode for the Tasks created.\n
    This macro only has effect in Legacy mode.\n
	In Auto Configuration Mode, the Output Rounding Mode can be set as part of ADI_FIR_CHANNEL_INFO.\n
    It can be configured to one of the following macros:
    - #ADI_FIR_RNDG_MODE_NEAREST_EVEN
    - #ADI_FIR_RNDG_MODE_TO_ZERO
 */
#define ADI_FIR_CFG_ROUNDING_MODE    (ADI_FIR_RNDG_MODE_NEAREST_EVEN)

/*!
 * This macro enables/disables fixed point mode across all tasks.\n
 * This macro only has effect in Legacy mode.\n
 * In Auto Configuration Mode, the Fixed Point Mode can be enabled as part of ADI_FIR_CHANNEL_INFO.\n
 * (0u) - Floating Point Mode\n
 * (1u) - Fixed point Mode
 */
#define ADI_FIR_FIXED_POINT_MODE            (0u)

/*! @cond PRIVATE */
#if (ADI_FIR_FIXED_POINT_MODE == 1u)
/*! @endcond */

/*! Unsigned Integer Input Format */
#define ADI_FIR_IN_FORMAT_UINT              (0u)
/*! Signed Integer Input Format*/
#define ADI_FIR_IN_FORMAT_SINT              (1u)

/*! This macro allows to set the  Fixed point format as Unsigned Integer or Signed Integer for all tasks.\n
    This macro only has effect in Legacy mode and when ADI_FIR_FIXED_POINT_MODE is set to 1.\n
	In Auto Configuration Mode, the Fixed Point format can be set as part of ADI_FIR_CHANNEL_INFO.\n
    It can be configured to one of the following macros:
    - #ADI_FIR_IN_FORMAT_UINT
    - #ADI_FIR_IN_FORMAT_SINT
*/
#define ADI_FIR_CFG_FIXED_INPUT_FORMAT    (ADI_FIR_IN_FORMAT_UINT)

#endif /* (ADI_FIR_FIXED_POINT_MODE == 1u) */

#endif /* ADI_FIR_CFG_ACCELERATOR_MODE == ADI_FIR_ACCELERATOR_MODE_LEGACY */

#if defined(__ADSPSHARCFX__)
/*! This Macro configures whether the accelerator driver uses SEC interrupts or Direct Core Interrupt\n
    It can be configured to one of the following macros:
    - 0 : Direct Interrupt
    - 1 : SEC Interrupt
 */
#define ADI_FIR_CFG_SEC_INT    (1)
#endif /* defined(__ADSPSHARCFX__) */
/*
** Verify the macro configuration
*/
#if ((ADI_FIR_CFG_ACCELERATOR_MODE != ADI_FIR_ACCELERATOR_MODE_LEGACY) && \
     (ADI_FIR_CFG_ACCELERATOR_MODE != ADI_FIR_ACCELERATOR_MODE_ACM))
#error "ADI_FIR_CFG_ACCELERATOR_MODE macro is wrongly configured"
#endif /* ADI_FIR_CFG_ACCELERATOR_MODE verification */

#if  ADI_FIR_CFG_ACCELERATOR_MODE == ADI_FIR_ACCELERATOR_MODE_LEGACY

#if ((ADI_FIR_CFG_ROUNDING_MODE != ADI_FIR_RNDG_MODE_NEAREST_EVEN)       && \
	 (ADI_FIR_CFG_ROUNDING_MODE != ADI_FIR_RNDG_MODE_TO_ZERO))
#error "ADI_FIR_CFG_ROUNDING_MODE macro wrongly configured"
#endif /* ADI_FIR_CFG_ROUNDING_MODE verification */

#if  ((ADI_FIR_FIXED_POINT_MODE != 0u)   && \
     (ADI_FIR_FIXED_POINT_MODE != 1u))
#error "ADI_FIR_FIXED_POINT_MODE macro wrongly configured"
#endif /* ADI_FIR_FIXED_POINT_MODE verification */

#if (ADI_FIR_FIXED_POINT_MODE == 1u)

#if ((ADI_FIR_CFG_FIXED_INPUT_FORMAT != ADI_FIR_IN_FORMAT_UINT)       && \
     (ADI_FIR_CFG_FIXED_INPUT_FORMAT != ADI_FIR_IN_FORMAT_SINT))
#error "ADI_FIR_CFG_FIXED_INPUT_FORMAT macro wrongly configured"
#endif /* ADI_FIR_CFG_FIXED_INPUT_FORMAT verification */

#endif /* (ADI_FIR_FIXED_POINT_MODE == 1u) */
#endif /* ADI_FIR_CFG_ACCELERATOR_MODE == ADI_FIR_ACCELERATOR_MODE_LEGACY */

#if defined(__ADSPSHARCFX__)
#if  ((ADI_FIR_CFG_SEC_INT != 0u)   && \
     (ADI_FIR_CFG_SEC_INT != 1u))
#error "ADI_FIR_CFG_SEC_INT macro wrongly configured"
#endif /* ADI_FIR_CFG_SEC_INT verification */
#endif /* defined(__ADSPSHARCFX__) */


#endif /* ADI_FIR_CONFIG_SC8XX_H */

/*@}*/

/*@}*/
